/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <glib.h>		// for g_strlcpy

#include <drm_client.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>

#include "ivug-define.h"
#include "ivug-debug.h"
#include "ivug-file-info.h"
#include "ivug-util.h"

#include "ivug-drm-common.h"


#include <Ecore_File.h>		//ecore_file_dir_is_empty

#define IVUG_DRM_TEMP_DIR	DATADIR"/ug-image-viewer-efl/.tmp"

#define MOUNTH_COUNT		12					//used for drm file info,
#define DAY_COUNT		30
#define HOUR_COUNT		24


char*
ivug_drm_get_drm_vendor_info(const char *path)
{
	IV_ASSERT(path != NULL);

	drm_result_e ret = DRM_RETURN_INTERNAL_ERROR;
	drm_content_info_s content_info;
	memset(&content_info, 0x0, sizeof(drm_content_info_s));

	ret = drm_get_content_info(path, &content_info);
	if(ret != DRM_RETURN_SUCCESS)
	{
		MSG_IVUG_ERROR("drm_get_content_info Error. %s, ret = %d", path, ret);
		return NULL;
	}

	if(strlen(content_info.author) > 0)
	{
		return strdup(content_info.author);
	}

	return NULL;
}

bool
ivug_drm_get_forwarding_info(const char *path)
{
	IV_ASSERT(path != NULL);

	drm_bool_type_e is_allowed = DRM_UNKNOWN;
	drm_action_type_e action = DRM_IS_FORWARDING_ALLOWED;
	drm_action_allowed_data_s action_data;
	int ret = -1;

	memset(&action_data, 0x00, sizeof(drm_action_allowed_data_s));

	g_strlcpy(action_data.file_path, path, sizeof(action_data.file_path));

	ret = drm_is_action_allowed(action, &action_data, &is_allowed);
	if(ret != DRM_RETURN_SUCCESS)
	{
		MSG_IVUG_ERROR("drm_is_action_allowed failed: %d", ret);
		return false;
	}

	if(is_allowed == DRM_TRUE)
	{
		MSG_IVUG_ERROR("Possible to forwarding. file=%s", path);
		return true;
	}
	else
	{
		MSG_IVUG_ERROR("Impossible to forwarding. file=%s", path);
		return false;
	}
}

char *
ivug_drm_get_right_url(const char *path)
{
	drm_content_info_s content_info;
	memset(&content_info,0x0,sizeof(drm_content_info_s));
	int ret = -1;

	ret = drm_get_content_info(path, &content_info);
	if(ret != DRM_RETURN_SUCCESS)
	{
		MSG_IVUG_ERROR("drm_get_content_info Error. %s, ret = %d", path, ret);
		return NULL;
	}

	if('\0' == content_info.rights_url[0])
	{
		MSG_IVUG_ERROR("rights_url is NULL. %s", path);
		return NULL;
	}
	MSG_IVUG_HIGH("rights_url = %s", content_info.rights_url);

	return strdup(content_info.rights_url);
}

char *
ivug_drm_get_mime(const char *path)
{
	drm_content_info_s content_info;
	memset(&content_info,0x0,sizeof(drm_content_info_s));
	int ret = -1;

	ret = drm_get_content_info(path, &content_info);
	if(ret != DRM_RETURN_SUCCESS)
	{
		MSG_IVUG_ERROR("drm_get_content_info Error. %s, ret = %d", path, ret);
		return NULL;
	}

	if('\0' == content_info.mime_type[0])
	{
		MSG_IVUG_ERROR("mime_type is NULL. %s", path);
		return NULL;
	}
	MSG_IVUG_HIGH("mime_type = %s", content_info.mime_type);

	return strdup(content_info.mime_type);
}


bool
ivug_drm_has_valid_ro(const char *path)
{
	IV_ASSERT(path != NULL);

	drm_result_e ret;

	drm_license_status_e status = DRM_LICENSE_STATUS_UNDEFINED;

	ret = drm_get_license_status(path, DRM_PERMISSION_TYPE_DISPLAY, &status);
	if(ret != DRM_RETURN_SUCCESS)
	{
		MSG_IVUG_ERROR("drm_get_license_status Error. %s, ret = %d", path, ret);
		return false;
	}

	if(status != DRM_LICENSE_STATUS_VALID)
	{
		MSG_IVUG_ERROR("No valid right. %s, status = %d", path, status);
		return false;
	}

	return true;
}

bool
ivug_drm_is_drm_file(const char* file_path)
{
	IV_ASSERT(file_path != NULL);

	drm_bool_type_e isDRM = DRM_FALSE;

	if(drm_is_drm_file(file_path, &isDRM) == DRM_RETURN_SUCCESS)
	{
		if(isDRM == DRM_TRUE)
		{
			MSG_IVUG_MED("DRM file. path=%s", file_path);
			return true;
		}
		return false;
	}

	return false;
}

bool ivug_drm_is_possible_to_setas(const char* path, _ivug_drm_setas_type type)
{
	ivug_retvm_if(!path, false, "path is NULL");

	drm_bool_type_e is_allowed = DRM_UNKNOWN;
	drm_action_type_e action = DRM_HAS_VALID_SETAS_STATUS;
	drm_action_allowed_data_s action_data;
	int ret = -1;

	memset(&action_data, 0x00, sizeof(drm_action_allowed_data_s));

	ret = drm_is_action_allowed(action, &action_data, &is_allowed);
	if(ret != DRM_RETURN_SUCCESS)
	{
		MSG_IVUG_ERROR("drm_is_action_allowed failed: %d", ret);
		return false;
	}

	if(is_allowed == DRM_TRUE)
	{
		MSG_IVUG_ERROR("Possible to setas. file=%s", path);
		return true;
	}
	else
	{
		MSG_IVUG_ERROR("Impossible to setas. file=%s", path);
		return false;
	}
}


drm_data_t * ivug_drm_check_constraint(const char* file_path)
{

	drm_file_info_s info;
	drm_constraint_info_s constraint_info;
	drm_permission_type_e perm_type = DRM_PERMISSION_TYPE_DISPLAY;
	drm_data_t *result = NULL;
	int ret = -1;

	memset(&info, 0x0, sizeof(drm_file_info_s));

	ret = drm_get_file_info(file_path, &info);
	if(ret != DRM_RETURN_SUCCESS)
	{
		MSG_IVUG_ERROR("drm_get_file_info Failed!!, ret=0x%x", ret);
		return NULL;
	}

	if (info.oma_info.method == DRM_METHOD_TYPE_FORWARD_LOCK)
	{
		MSG_IVUG_HIGH("DRM_METHOD_TYPE_FORWARD_LOCK");
		result = calloc(1, sizeof(drm_data_t));
		result->constraints = IVUG_DRM_RESULT_UNLIMITED;
		result->data = -1;
		return result;
	}

	memset(&constraint_info, 0x0, sizeof(drm_constraint_info_s));

	ret = drm_get_constraint_info(file_path, perm_type, &constraint_info);
	if(ret != DRM_RETURN_SUCCESS)
	{
		MSG_IVUG_ERROR("drm_get_constraint_info Failed!!, ret=0x%x", ret);
		return NULL;
	}

	result = calloc(1, sizeof(drm_data_t));

	if(constraint_info.const_type.is_unlimited)
	{
		MSG_IVUG_HIGH("Unlimited:for permission=%d", perm_type);

		result->constraints = IVUG_DRM_RESULT_UNLIMITED;
		result->data = -1;
		return result;
	}

	if(constraint_info.const_type.is_accumulated){
		MSG_IVUG_HIGH("Accumulated Constraint is present");
		MSG_IVUG_HIGH("accumulated_original_seconds=%d",
				constraint_info.accumulated_original_seconds);
		MSG_IVUG_HIGH("accumulated_remaining_seconds=%d",
				constraint_info.accumulated_remaining_seconds);

		result->constraints = IVUG_DRM_RESULT_ACCUMULATED;
		result->data = -1;
		return result;
	}

	if(constraint_info.const_type.is_count){
		MSG_IVUG_HIGH("Count Constraint is present");
		MSG_IVUG_HIGH("original_count=%d",
				constraint_info.original_count);
		MSG_IVUG_HIGH("remaining_count=%d",
				constraint_info.remaining_count);

		result->constraints = IVUG_DRM_RESULT_COUNT;
		result->data = constraint_info.remaining_count;
		return result;
	}

	if(constraint_info.const_type.is_datetime){
		MSG_IVUG_HIGH("DATETIME Constraint is present");
		MSG_IVUG_HIGH("start_time:"
				"Year=%d,Month=%d,Day=%d,Hour=%d, minute=%d, Second=%d",
			constraint_info.start_time.tm_year,
			constraint_info.start_time.tm_mon,
			constraint_info.start_time.tm_mday,
			constraint_info.start_time.tm_hour,
			constraint_info.start_time.tm_min,
			constraint_info.start_time.tm_sec);

		MSG_IVUG_HIGH("end_time:"
				"Year=%d,Month=%d,Day=%d,Hour=%d, minute=%d, Second=%d",
			constraint_info.end_time.tm_year,
			constraint_info.end_time.tm_mon,
			constraint_info.end_time.tm_mday,
			constraint_info.end_time.tm_hour,
			constraint_info.end_time.tm_min,
			constraint_info.end_time.tm_sec);

		result->constraints = IVUG_DRM_RESULT_DATETIME;
		//result->data = constraint.curCount;
		result->data = -1;
		return result;
	}

	if(constraint_info.const_type.is_individual){
		MSG_IVUG_HIGH("INDIVIDUAL Constraint is present");
		MSG_IVUG_HIGH("individual_identity_type=%d "
				"individual_id=%s",
				constraint_info.individual_identity_type,
				constraint_info.individual_id);

		result->constraints = IVUG_DRM_RESULT_INDIVIDUAL;
		result->data = -1;
		return result;
	}

	if(constraint_info.const_type.is_interval){
		MSG_IVUG_HIGH("INTERVAL Constraint is present");
		MSG_IVUG_HIGH("interval_time:"
				"Year=%d,Month=%d,Day=%d,Hour=%d, minute=%d, Second=%d",
			constraint_info.interval_time.tm_year,
			constraint_info.interval_time.tm_mon,
			constraint_info.interval_time.tm_mday,
			constraint_info.interval_time.tm_hour,
			constraint_info.interval_time.tm_min,
			constraint_info.interval_time.tm_sec);

		result->constraints = IVUG_DRM_RESULT_INTERVAL_DAYS;
		result->data = ((constraint_info.interval_time.tm_year*MOUNTH_COUNT+constraint_info.interval_time.tm_mon)*
							DAY_COUNT+constraint_info.interval_time.tm_mday);
		return result;
	}

	if(constraint_info.const_type.is_system){
		MSG_IVUG_HIGH("SYSTEM Constraint is present");
		MSG_IVUG_HIGH("system_identity_type=%d "
				"system_id=%s",
				constraint_info.system_identity_type,
				constraint_info.system_id);

		result->constraints = IVUG_DRM_RESULT_SYSTEM;
		result->data = -1;
		return result;
	}

	if(constraint_info.const_type.is_timedcount){
		MSG_IVUG_HIGH("TIMED-COUNT Constraint is present");
		MSG_IVUG_HIGH("timed_original_count=%d",
				constraint_info.timed_original_count);

		MSG_IVUG_HIGH("timed_remaining_count=%d",
				constraint_info.timed_remaining_count);

		MSG_IVUG_HIGH("timed_count_timer=%d",
				constraint_info.timed_count_timer);

		result->constraints = IVUG_DRM_RESULT_TIMED_COUNT;
		result->data = -1;
		return result;
	}

	MSG_IVUG_WARN("Undefined constraints");
	free(result);

	return NULL;
}

