/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#include "ivug-define.h"
#include "ivug-uuid.h"

#include "ivug-util.h"
#include "ivug-debug.h"
#include "ivug-file-info.h"

#include "ivug-db.h"

#include <unistd.h>
#include <stdlib.h>

#include <math.h>

#include <power.h>
#include <string.h>

#include <sys/stat.h>	// mkdir
#include <sys/types.h>	// mkdir

#include <glib.h>

#include <vconf.h>
#include <vconf-keys.h>

#include <runtime_info.h>

#include <unicode/udat.h>
#include <unicode/ustring.h>
#include <unicode/uloc.h>
#include <unicode/ucal.h>
#include <unicode/udatpg.h>
#include <unicode/utmscale.h>

/*
	If filepath is web url, return TRUE.

*/
bool ivug_is_web_uri(const char* uri)
{
//check file url type. local , http, ftp.
	IV_ASSERT(uri != NULL);

	static const char* web_protocal_name[] =
	{
		"http://",
		"https://",
		"ftp://",
		NULL,
	};

	int i = 0;
	while (web_protocal_name[i] != NULL)
	{
		if ( strlen(uri) > strlen(web_protocal_name[i]) )
		{
			if (strncmp(uri, web_protocal_name[i], strlen(web_protocal_name[i])) == 0)
			{
				return true;
			}
		}

		i++;
	}

	MSG_UTIL_MED("Not web uri. %s", uri);

	return false;
}


unsigned int get_distance(int prevX, int prevY, int X, int Y)
{
	int dx = prevX - X;
	int dy = prevY - Y;

	return sqrt(dx*dx + dy*dy);
}



#define USE_ECORE_FILE

#include <Ecore_File.h>

/*
	Remove fname file.
	Returns true fname is not exist or removed sucessfully
*/
bool ivug_remove_file(const char *filepath)
{
	char error_msg[256];
	if (ecore_file_exists(filepath) == EINA_FALSE)
	{
		MSG_UTIL_ERROR("Already removed.%s", filepath);
		return true;
	}

#ifdef USE_ECORE_FILE
	if ( ecore_file_unlink(filepath) == EINA_FALSE)
	{
		MSG_UTIL_ERROR("Cannot remove file : %s %s", filepath, strerror_r(errno, error_msg, sizeof(error_msg)) );
		return false;
	}

	return true;
#else
	if ( unlink(filepath) != 0 )
	{
		MSG_UTIL_ERROR("Cannot remove file : %s %s", filepath, strerror_r(errno, error_msg, sizeof(error_msg)) );
		return false;
	}

	return true;
#endif
}

bool ivug_rename_file(const char *src, const char *dst)
{
	if (ecore_file_exists(src) == EINA_FALSE)
	{
		MSG_UTIL_ERROR("Source file is not exist : %s", src);
		return false;
	}

	if (ecore_file_exists(dst) == EINA_TRUE)
	{
		MSG_UTIL_ERROR("Destination file is exist : %s", dst);
		return false;
	}

	if (dst == NULL)
	{
		MSG_UTIL_ERROR("Destination file is NULL");
		return false;
	}

	char error_msg[256];

	if ( rename(src, dst) < 0)
	{
		MSG_UTIL_ERROR("Cannot rename from %s to %s : %s", src, dst, strerror_r(errno, error_msg, sizeof(error_msg)) );
		return false;
	}

	return true;
}

bool ivug_copy_file(const char *filename, const char *dest)
{
#define DIR_MASK_DEFAULT 0775
	if ( filename == NULL)
	{
		MSG_MAIN_ERROR("File does not existfilepath=%s", filename);
		return false;
	}

	if(ecore_file_cp(filename, dest) == EINA_FALSE)
	{
		MSG_MAIN_ERROR("ecore_file_cp failed. From %s To %s", filename, dest);
		return false;
	}

	return true;
}


char *
ivug_mktemp(char* filepath, char*ext)
{
	ivug_retv_if(!filepath || !ext, NULL);

	MSG_IVUG_HIGH("filepath %s, ext %s", filepath, ext);

	char tempname[IVUG_MAX_FILE_PATH_LEN+1] = {0};
	snprintf(tempname, sizeof(tempname), "%s_0.%s",filepath, ext);
	int i = 1;

	while(ecore_file_exists(tempname) == EINA_TRUE )
	{
		snprintf(tempname, sizeof(tempname),"%s_%d.%s", filepath, i, ext);
		i++;
	}

	MSG_IVUG_HIGH( " tempname %s, i %d", tempname, i);

	return strdup(tempname);
}


/*
 	Check whether given filepath file exists

 	CAUTION : filepath cannot be NULL.
*/
bool ivug_is_file_exist(const char* filepath)
{
	IV_ASSERT(filepath != NULL);

	if ( ecore_file_exists(filepath) == EINA_TRUE)
	{
		return true;
	}

	return false;
}


/*
	USer should freed returned value!
*/
char *ivug_get_directory(const char *filepath)
{
	if ( filepath == NULL )
	{
		MSG_UTIL_WARN("File path is NULL");
		return "NULL";
	}

#ifdef USE_ECORE_FILE
	return ecore_file_dir_get(filepath);
#else
	#error "Not implemented yet."
#endif
}


/*
	Returns start pointer of filename within filepath.
	No memory allocated in this function. so user do not free returned pointer.

	CAUTION : filepath cannot be NULL.
*/
char * ivug_get_filename(const char *filepath)
{
	if ( filepath == NULL )
	{
		MSG_UTIL_WARN("File path is NULL");
		return "NULL";
	}

#ifdef USE_ECORE_FILE
	return ecore_file_file_get(filepath);
#else

#define DIRECORY_SPLITTER '/'
	const char*pFileName = NULL;

	pFileName  =  strrchr(filepath, DIRECORY_SPLITTER);

	if ( pFileName == NULL)		// If not found
	{
		return filepath;
	}

	return (pFileName+1);
#endif

}

bool ivug_prohibit_lcd_off(void)
{
	MSG_UTIL_MED("START : Sleep disabled");
	return (power_lock_state(POWER_STATE_NORMAL, 0)==0 ? true : false);
}

bool ivug_allow_lcd_off(void)
{
	MSG_UTIL_MED("END : Sleep disabled");
	return (power_unlock_state(POWER_STATE_NORMAL)==0 ? true : false);
}

long int ivug_atox(const char *number)
{
	char *endptr = NULL;
	long long int val = 0;

	errno = 0;

	val = strtoll(number, &endptr, 16);

	if((errno == ERANGE && (val == LLONG_MAX || val == LLONG_MIN)) || (errno != 0 && val == 0))
	{
		MSG_UTIL_ERROR("ERANGE = %d, LONG_MAX = %d, LONG_MIN = %d", ERANGE, LLONG_MAX, LLONG_MIN);
		MSG_UTIL_ERROR("strtol, val = %d, 0x%x, errno = %d, ", val, val, errno);
		return -1L;
	}

	if(endptr == number)
	{
		MSG_UTIL_ERROR("No digits were found, number = %s", number);
		return -1L;
	}

	return (long int)val;
}

int ivug_atoi(const char *number)
{
	char *endptr = NULL;
	long val = 0;

	errno = 0;

	val = strtol(number, &endptr, 10);

	if((errno == ERANGE && (val == LONG_MAX || val == LONG_MIN)) || (errno != 0 && val == 0))
	{
		MSG_UTIL_ERROR("strtol, val = %d", val);
		return -1;
	}

	if(endptr == number)
	{
		MSG_UTIL_ERROR("No digits were found, number = %s", number);
		return -1;
	}

	return (int)val;
}

double ivug_atod(const char *number)
{
	char *endptr = NULL;
	double val = 0;

	errno = 0;

	val = strtod(number, &endptr);

	if((errno == ERANGE && (val == -HUGE_VAL || val == HUGE_VAL)) || (errno != 0 && val == 0))
	{
		MSG_UTIL_ERROR("strtod, val = %d", val);
		return -1;
	}

	if(endptr == number)
	{
		MSG_UTIL_ERROR("No digits were found, number = %s", number);
		return -1;
	}

	return val;
}

char * ivug_generate_file_name(const char *filepath, const char *extension, const char *dest_dir)
{
	IV_ASSERT(filepath != NULL);

	MSG_SETAS_HIGH("filepath %s", filepath);

	char tempname[IVUG_MAX_FILE_PATH_LEN+1] = {0,};
	char *ext = NULL;
	char *filename = ecore_file_strip_ext(ecore_file_file_get(filepath));
	char *dir = NULL;

	if(extension)
	{
		ext = strdup(extension);
	}
	else
	{
		ext = ivug_fileinfo_get_file_extension(filepath);
	}

	if(dest_dir)
	{
		dir = strdup(dest_dir);
	}
	else
	{
		dir = ecore_file_dir_get(filepath);
	}

	int i = 0;

	for(i=1; i<IVUG_MAX_FILE_PATH_LEN; i++)
	{
		snprintf(tempname, sizeof(tempname), "%s/%s_%d.%s", dir, filename, i, ext);
		if(ecore_file_exists(tempname) == EINA_FALSE)
			break;
	}

	if(filename)
	{
		free(filename);
	}
	if(dir)
	{
		free(dir);
	}
	if(ext)
	{
		free(ext);
	}
	if(i == IVUG_MAX_FILE_PATH_LEN)
	{
		MSG_UTIL_ERROR("Cannot make file");
		return NULL;
	}

	MSG_UTIL_HIGH( "tempname: %s", tempname);

	return strdup(tempname);
}

char * ivug_strip_string(char* name)
{
	MSG_UTIL_HIGH("Input name: %s", name);
	/* Check name length */
	if (strlen(name) == 0)
	{
		MSG_UTIL_WARN("Inserted text is empty!");
		return NULL;
	}
	MSG_UTIL_HIGH("Inserted name: %s, length: %d", name, strlen(name));

	/* Removes leading and trailing whitespace */
	g_strstrip(name);
	if (strlen(name) == 0)
	{
		MSG_UTIL_WARN("name includes only space!");
		return NULL;
	}
	return name;
}



bool ivug_is_bestpic(const char *file)
{
	ivug_retv_if(file == NULL, false);

	const char szBest[] = "_bestshot";		// sizeof(szBest) = 10;

	const char *find = NULL;

	find = strrchr(file, '.');

	if(find == NULL)
	{
		// All files should have extension
		return false;
	}

	unsigned int dist = find - file;

	if ( dist < sizeof(szBest) -1)
	{
		return false;
	}

	if ( strncmp(find - sizeof(szBest) + 1 , szBest , sizeof(szBest) - 1) == 0 )
	{
		return true;
	}

	return false;

}

/*
	Shuld free returned value after use
*/
static char*
_get_icu_time_string(const char *locale, const char *customSkeleton, const char *timezone, UDate date)
{
#define UG_ICU_ARR_LENGTH 			256

	IV_ASSERT(customSkeleton != NULL);
	IV_ASSERT(timezone != NULL);

	/*
		Copy a byte string encoded in the default codepage to a ustring.
		Copies at most n characters. The result will be null terminated if the length of src is less than n. Performs a host byte to UChar conversion
	*/
	UChar ucustomSkeleton[UG_ICU_ARR_LENGTH] = {0,};

	if(u_uastrncpy(ucustomSkeleton, customSkeleton, UG_ICU_ARR_LENGTH) == NULL)
	{
		MSG_UTIL_ERROR("u_uastrncpy() error.");
		return NULL;
	}

	UChar utimezone[UG_ICU_ARR_LENGTH] = {0,};

	if ( u_uastrncpy(utimezone, timezone, UG_ICU_ARR_LENGTH) == NULL )
	{
		MSG_UTIL_ERROR("u_uastrncpy() error.");
		return NULL;
	}

	UErrorCode status = U_ZERO_ERROR;
	UDateTimePatternGenerator *generator;
	UDateFormat	*formatter;

	UChar bestPattern[UG_ICU_ARR_LENGTH] = {0,};
	UChar formatted[UG_ICU_ARR_LENGTH] = {0,};
	char formattedString[UG_ICU_ARR_LENGTH] = {0,};
	int32_t bestPatternLength, formattedLength;

	ucal_setDefaultTimeZone(utimezone , &status);

	if (U_FAILURE(status))
	{
		MSG_UTIL_ERROR("ucal_setDefaultTimeZone() is failed");
		return NULL;
	}

	uloc_setDefault(getenv("LC_TIME"), &status);

	if (U_FAILURE(status))
	{
		MSG_UTIL_ERROR("ucal_setDefaultTimeZone() is failed");
		return NULL;
	}

	generator = udatpg_open(locale, &status);
	if(generator == NULL)
	{
		return NULL;
	}

	bestPatternLength = udatpg_getBestPattern(generator, ucustomSkeleton, u_strlen(ucustomSkeleton), bestPattern, UG_ICU_ARR_LENGTH, &status);
	if(bestPatternLength <= 0)
	{
		return NULL;
	}

	formatter = udat_open(UDAT_IGNORE, UDAT_IGNORE, locale, NULL, -1, bestPattern, -1, &status);
	if(formatter == 0)
	{
		return NULL;
	}

	formattedLength = udat_format(formatter, date, formatted, UG_ICU_ARR_LENGTH, NULL, &status);
	if(formattedLength <= 0)
	{
		return NULL;
	}

	u_austrcpy(formattedString, formatted);
	udatpg_close(generator);
	udat_close(formatter);

	if(strlen(formattedString) == 0)
	{
		return NULL;
	}

	return strdup(formattedString);
}

char*
ivug_get_icu_date(time_t mtime)
{
#define UG_DATE_FORMAT_12			"yMMMdhms"
#define UG_DATE_FORMAT_24			"yMMMdHms"

	char* skeleton = NULL;

	bool hours_24 = false;
	int ret = -1;

	ret = runtime_info_get_value_bool(RUNTIME_INFO_KEY_24HOUR_CLOCK_FORMAT_ENABLED, &hours_24);
	if(ret != RUNTIME_INFO_ERROR_NONE)
	{
		MSG_UTIL_ERROR("Cannot get 24 hours format");
		return NULL;
	}

	if(hours_24 == true)
	{
		skeleton = UG_DATE_FORMAT_24;
	}
	else
	{
		skeleton = UG_DATE_FORMAT_12;
	}

	char *locale = vconf_get_str(VCONFKEY_REGIONFORMAT);		/* eg. en_US.UTF-8*/
	if(locale == NULL)
	{
		MSG_UTIL_ERROR("Cannot get region format.");
		locale = "en_US.UTF-8";		// Default value.
	}

	char *timezone = NULL;

	timezone = vconf_get_str(VCONFKEY_SETAPPL_TIMEZONE_ID);	// eg Asia/Seoul
	if(timezone == NULL)
	{
		MSG_UTIL_ERROR("Cannot get time zone");
		return NULL;
	}

	MSG_UTIL_HIGH("Locale:%s TimeZone:%s TimeFormat:%s", locale, skeleton, timezone);

	char* datestr = NULL;

	datestr = _get_icu_time_string(locale, skeleton, timezone, (UDate)mtime * 1000);

	if(datestr == NULL)
	{
		MSG_UTIL_ERROR("Cannot get time string");
		return NULL;
	}

	MSG_UTIL_HIGH("ICU Date : %s", datestr);
	return datestr;

}

bool ivug_validate_file_name(const char *fname)
{
	if (!fname)
		return false;

	const char badchar[] = "\\/;:*?<>|\"";
	const char *tmp = fname;

	/* check bad character */
	while (*tmp != '\0') {
		if (strchr(badchar, *tmp) != NULL) {
			MSG_UTIL_HIGH("Invalid file name=%s char=%c", fname, *tmp);
			return false;
		}
		tmp++;
	}
	return true;
}

