/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <ui-gadget.h>

#include <app.h>
#include <Elementary.h>

#include "ivug-ext-ug.h"
#include "ivug-util.h"
#include "ivug-debug.h"


//definition
#define MESSAGE_UG_NAME 		"msg-composer-efl"
#define EMAIL_UG_NAME 			"email-composer-efl"
#define CONTACT_UG_NAME 		"contacts-list-efl"
#define MYFILE_DETAIL_UG_NAME	"myfile-detail-efl"
#define BLUETOOTH_UG_NAME 		"ug-bluetooth-efl"
#define MYFILE_UG_NAME			"myfile-efl"

#define CONTACT_SELECT_UG_NAME	"contacts-tabui-efl"
#define BLUETOOTH_SETTING_NAME	"setting-bluetooth-efl"

#define VIDEOPLAYER_PKG_NAME 	"org.tizen.video-player"
#define BLUETOOTH_PKG_NAME 		"org.tizen.bluetooth"
#define BROWSER_PKG_NAME 		"org.tizen.browser"
#define MESSAGE_PKG_NAME 		"org.tizen.message"
#define EMAIL_PKG_NAME 			"org.tizen.email"

#define WIFI_FILE_TRANSFER_UG_NAME	"fileshare-efl"

#define PRINT_PKG_NAME "org.tizen.mobileprint"

#define OPERATION_NAME_PRINT "http://tizen.org/appcontrol/operation/PRINT"

#define SERVICE_PRINT_FILES_TYPE "service_print_files_type"

#define MIME_TYPE_LEN			255

typedef struct {
	ug_destroy_cb destroy_func;
	ug_result_cb result_func;
	void *cb_data;
}ext_ug_t;
#if 0
static void
_bundle_print(const char *key, const char *val, void *data)
{
	MSG_IVUG_HIGH("  %s:%s", key, val);
}
#endif

static bool _data_print(service_h service, const char *key, void *user_data)
{
	char *value;

	char **value_array;
	int array_len = 1;
	int i;
	bool array = false;

	service_is_extra_data_array(service, key, &array);
	if(array == false)
	{
		service_get_extra_data(service, key, &value);
		MSG_IVUG_HIGH("  %s : %s", key, value);
		free(value);
	}
	else
	{
		service_get_extra_data_array(service, key, &value_array, &array_len);
		MSG_IVUG_HIGH("  %s :", key);
		for(i=0; i<array_len; i++)
		{
			MSG_IVUG_HIGH(" %s", value_array[i]);
		}
		for(i=0; i<array_len; i++)
		{
			free(value_array[i]);
		}
		free(value_array);
	}

	return true;
}

static void print_service_data(service_h service)
{
	int ret = service_foreach_extra_data(service, _data_print, NULL);

	if(SERVICE_ERROR_NONE != ret)
	{
		MSG_IVUG_ERROR("service_foreach_extra_data ERROR");
	}
}

static void
_ivug_ext_ug_layout_cb(ui_gadget_h ug, enum ug_mode mode, void *priv)
{
    Evas_Object *base, *win;

	if (!ug ) return;

    base = ug_get_layout(ug);
    if (!base)
    {
    	ug_destroy(ug);
    	return;
    }

    win = ug_get_window();

    switch (mode)
	{
    	case UG_MODE_FULLVIEW:
		MSG_IMAGEVIEW_HIGH("Creating UG layout");
		evas_object_size_hint_weight_set(base, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
		//elm_win_resize_object_add(win, base);
		evas_object_show(base);
	break;
    default:
		MSG_IMAGEVIEW_ERROR("Unhandled Mode : %d", mode);
		break;
    }
}


static void
_ivug_ext_ug_result_cb(ui_gadget_h ug, service_h result, void *priv)
{
	if (!ug ) return;

	if (result)
	{
		print_service_data(result);
	}

	ext_ug_t *ug_struct = (ext_ug_t *)priv;

	if(ug_struct->result_func)
		ug_struct->result_func(ug, result, ug_struct->cb_data);
}


static void
_ivug_ext_ug_destroy_cb(ui_gadget_h ug, void *priv)
{
	if (!ug ) return;

	ext_ug_t *ug_struct = (ext_ug_t *)priv;

	MSG_IMAGEVIEW_HIGH("Destroy ug");
	ug_destroy(ug);

	ug_struct->destroy_func(ug, ug_struct->cb_data);

	free(ug_struct);
}

ui_gadget_h _ivug_ext_launch_ug(const char *pkgname, service_h service, ug_destroy_cb func, void *data)
{
	ui_gadget_h ug = NULL;
	struct ug_cbs cbs = {0, };

	ext_ug_t *ug_struct = calloc(1, sizeof(ext_ug_t));
	ug_struct->result_func = NULL;
	ug_struct->destroy_func = func;
	ug_struct->cb_data = data;

	cbs.layout_cb = _ivug_ext_ug_layout_cb;
	cbs.result_cb = _ivug_ext_ug_result_cb;
	cbs.destroy_cb = _ivug_ext_ug_destroy_cb;
	cbs.priv = ug_struct;

	ug = ug_create( NULL, pkgname, UG_MODE_FULLVIEW, service, &cbs );

	return ug;
}

ui_gadget_h _ivug_ext_launch_ug_with_result(const char *pkgname,
	service_h service, ug_result_cb result_func, ug_destroy_cb destroy_func, void *data)
{
	ui_gadget_h ug = NULL;
	struct ug_cbs cbs = {0, };

	ext_ug_t *ug_struct = calloc(1, sizeof(ext_ug_t));
	ug_struct->result_func = result_func;
	ug_struct->destroy_func = destroy_func;
	ug_struct->cb_data = data;

	cbs.layout_cb = _ivug_ext_ug_layout_cb;
	cbs.result_cb = _ivug_ext_ug_result_cb;
	cbs.destroy_cb = _ivug_ext_ug_destroy_cb;
	cbs.priv = ug_struct;

	ug = ug_create( NULL, pkgname, UG_MODE_FULLVIEW, service, &cbs );

	return ug;
}


void ivug_ext_service_reply_cb(service_h request, service_h reply, service_result_e result, void *user_data)
{
	MSG_IMAGEVIEW_HIGH("ivug_ext_service_reply_cb");
	switch(result)
	{
		case SERVICE_RESULT_SUCCEEDED:
			MSG_IMAGEVIEW_HIGH("SERVICE_RESULT_SUCCEEDED");
			break;
		case SERVICE_RESULT_FAILED:
			MSG_IMAGEVIEW_HIGH("SERVICE_RESULT_FAILED");
			break;
		case SERVICE_RESULT_CANCELED:
			MSG_IMAGEVIEW_HIGH("SERVICE_RESULT_CANCELED");
			break;
		default:
			MSG_IMAGEVIEW_ERROR("unhandled value %d", result);
			break;
	}
}

ui_gadget_h  ivug_ext_launch_wifi_file_transfer(const char *uri, ug_destroy_cb func, void *data)
{
	MSG_IMAGEVIEW_HIGH("%s. URI=%s", __func__, uri);

	int ret = -1;
	int destroy_ret = -1;

	service_h handle;
	ui_gadget_h ug = NULL;

	ret = service_create(&handle);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_create failed, %d", ret);
		return NULL;
	}

	/*ret = service_set_operation(handle, SERVICE_OPERATION_VIEW);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_set_operation failed, %d", ret);
		goto WIFI_TRANSFER_END;
	}*/

	ret = service_set_uri (handle, uri);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_set_uri failed, %d", ret);
		goto WIFI_TRANSFER_END;
	}

	ret = service_add_extra_data(handle, "filecount", "1");
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_add_extra_data failed, %d", ret);
		goto WIFI_TRANSFER_END;
	}

	ret = service_add_extra_data(handle, "files", uri);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_add_extra_data failed, %d", ret);
		goto WIFI_TRANSFER_END;
	}

	/*const char *pkgname = WIFI_FILE_TRANSFER_UG_NAME;

	ret = service_set_package(handle, pkgname);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_set_uri failed, %d", ret);
		goto WIFI_TRANSFER_END;
	}*/

	ug = _ivug_ext_launch_ug(WIFI_FILE_TRANSFER_UG_NAME, handle, func, data);

WIFI_TRANSFER_END:
	destroy_ret = service_destroy(handle);
	if(destroy_ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_destroy failed, %d", destroy_ret);
		return NULL;
	}

	return (ret == SERVICE_ERROR_NONE ? ug : NULL);
}

ui_gadget_h  ivug_ext_launch_message(const char *uri, ug_destroy_cb func, void *data)
{
	MSG_IMAGEVIEW_HIGH("%s. URI=%s", __func__, uri);

	int ret = -1;
	int destroy_ret = -1;

	service_h handle;
	ui_gadget_h ug = NULL;

	ret = service_create(&handle);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_create failed, %d", ret);
		return NULL;
	}

	/*ret = service_set_operation(handle, SERVICE_OPERATION_VIEW);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_set_operation failed, %d", ret);
		goto MESSAGE_END;
	}*/

	ret = service_set_uri (handle, uri);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_set_uri failed, %d", ret);
		goto MESSAGE_END;
	}

	if (ivug_is_web_uri(uri) == true)
	{
		ret = service_add_extra_data(handle, "BODY", uri);
		if(ret != SERVICE_ERROR_NONE)
		{
			MSG_IMAGEVIEW_ERROR("service_add_extra_data failed, %d", ret);
			goto MESSAGE_END;
		}
	}
	else
	{
		ret = service_add_extra_data(handle, "ATTACHFILE", uri);
		if(ret != SERVICE_ERROR_NONE)
		{
			MSG_IMAGEVIEW_ERROR("service_add_extra_data failed, %d", ret);
			goto MESSAGE_END;
		}
	}

	/*const char *pkgname = MESSAGE_UG_NAME;

	ret = service_set_package(handle, pkgname);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_set_uri failed, %d", ret);
		goto MESSAGE_END;
	}*/

	ug = _ivug_ext_launch_ug(MESSAGE_UG_NAME, handle, func, data);

MESSAGE_END:
	destroy_ret = service_destroy(handle);
	if(destroy_ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_destroy failed, %d", destroy_ret);
		return NULL;
	}
	return (ret == SERVICE_ERROR_NONE ? ug : NULL);
}

bool  ivug_ext_launch_email(const char *uri)
{
	MSG_IMAGEVIEW_HIGH("%s. URI=%s", __func__, uri);

	int ret = -1;
	int destroy_ret = -1;

	service_h handle;

	ret = service_create(&handle);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_create failed, %d", ret);
		return false;
	}

	ret = service_set_operation(handle, SERVICE_OPERATION_SEND);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_set_operation failed, %d", ret);
		goto EMAIL_END;
	}

	ret = service_set_uri(handle, uri);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_set_uri failed, %d", ret);
		goto EMAIL_END;
	}

	Ecore_X_Window win_id = elm_win_xwindow_get(ug_get_window());
	ret = service_set_window(handle, win_id);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_add_extra_data failed, %d", ret);
		goto EMAIL_END;
	}

	ret = service_add_extra_data(handle, "RUN_TYPE", "5");
		if(ret != SERVICE_ERROR_NONE)
		{
			MSG_IMAGEVIEW_ERROR("service_add_extra_data failed, %d", ret);
			goto EMAIL_END;
		}

	if (ivug_is_web_uri(uri) == true)
	{
		ret = service_add_extra_data(handle, "BODY", uri);
		if(ret != SERVICE_ERROR_NONE)
		{
			MSG_IMAGEVIEW_ERROR("service_add_extra_data failed, %d", ret);
			goto EMAIL_END;
		}
	}
	else
	{
		ret = service_add_extra_data(handle, "ATTACHMENT", uri);
		if(ret != SERVICE_ERROR_NONE)
		{
			MSG_IMAGEVIEW_ERROR("service_add_extra_data failed, %d", ret);
			goto EMAIL_END;
		}
	}

	const char *pkgname = EMAIL_UG_NAME;

	ret = service_set_package(handle, pkgname);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_set_package failed, %d", ret);
		goto EMAIL_END;
	}

	ret = service_send_launch_request(handle, ivug_ext_service_reply_cb, NULL);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_send_launch_request failed, %d", ret);
		goto EMAIL_END;
	}

EMAIL_END:
	destroy_ret = service_destroy(handle);
	if(destroy_ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_destroy failed, %d", destroy_ret);
		return false;
	}

	return (ret == SERVICE_ERROR_NONE ? true : false);
}

ui_gadget_h  ivug_ext_launch_contact(const char *uri, ug_destroy_cb func, void *data)
{
	MSG_IMAGEVIEW_HIGH("%s. URI=%s", __func__, uri);

	int ret = -1;
	int destroy_ret = -1;

	service_h handle;
	ui_gadget_h ug = NULL;

	ret = service_create(&handle);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_create failed, %d", ret);
		return NULL;
	}

	/*ret = service_set_operation(handle, SERVICE_OPERATION_VIEW);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_set_operation failed, %d", ret);
		goto MESSAGE_END;
	}*/

	ret = service_set_uri (handle, uri);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_set_uri failed, %d", ret);
		goto CONTACT_END;
	}

	ret = service_add_extra_data(handle, "type", "41");
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_add_extra_data failed, %d", ret);
		goto CONTACT_END;
	}

	ret = service_add_extra_data(handle, "ct_path", uri);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_add_extra_data failed, %d", ret);
		goto CONTACT_END;
	}

	/*const char *pkgname = CONTACT_UG_NAME;

	ret = service_set_package(handle, pkgname);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_set_uri failed, %d", ret);
		goto CONTACT_END;
	}*/

	ug = _ivug_ext_launch_ug(CONTACT_UG_NAME, handle, func, data);

CONTACT_END:
	destroy_ret = service_destroy(handle);
	if(destroy_ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_destroy failed, %d", destroy_ret);
		return NULL;
	}

	return (ret == SERVICE_ERROR_NONE ? ug : NULL);
}

ui_gadget_h  ivug_ext_launch_bluetooth_send(const char *uri, ug_destroy_cb func, void *data)
{
	MSG_IMAGEVIEW_HIGH("%s. URI=%s", __func__, uri);

	int ret = -1;
	int destroy_ret = -1;

	service_h handle;
	ui_gadget_h ug = NULL;

	ret = service_create(&handle);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_create failed, %d", ret);
		return NULL;
	}

	/*ret = service_set_operation(handle, SERVICE_OPERATION_VIEW);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_set_operation failed, %d", ret);
		goto BT_END;
	}*/

	ret = service_set_uri (handle, uri);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_set_uri failed, %d", ret);
		goto BT_END;
	}

	ret = service_add_extra_data(handle, "launch-type", "send");
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_add_extra_data failed, %d", ret);
		goto BT_END;
	}

	ret = service_add_extra_data(handle, "filecount", "1");
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_add_extra_data failed, %d", ret);
		goto BT_END;
	}

	ret = service_add_extra_data(handle, "files", uri);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_add_extra_data failed, %d", ret);
		goto BT_END;
	}

	/*const char *pkgname = BLUETOOTH_UG_NAME;

	ret = service_set_package(handle, pkgname);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_set_uri failed, %d", ret);
		goto BT_END;
	}*/

	ug = _ivug_ext_launch_ug(BLUETOOTH_SETTING_NAME, handle, func, data);

BT_END:
	destroy_ret = service_destroy(handle);
	if(destroy_ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_destroy failed, %d", destroy_ret);
		return NULL;
	}

	return (ret == SERVICE_ERROR_NONE ? ug : NULL);
}

ui_gadget_h  ivug_ext_launch_bluetooth_print(const char *uri, ug_destroy_cb func, void *data)
{
	MSG_IMAGEVIEW_HIGH("%s. URI=%s", __func__, uri);

	int ret = -1;
	int destroy_ret = -1;

	service_h handle;
	ui_gadget_h ug = NULL;

	ret = service_create(&handle);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_create failed, %d", ret);
		return NULL;
	}

	/*ret = service_set_operation(handle, SERVICE_OPERATION_VIEW);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_set_operation failed, %d", ret);
		goto BT_END;
	}*/

	ret = service_set_uri (handle, uri);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_set_uri failed, %d", ret);
		goto BT_PRINT_END;
	}

	ret = service_add_extra_data(handle, "launch-type", "print");
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_add_extra_data failed, %d", ret);
		goto BT_PRINT_END;
	}

	ret = service_add_extra_data(handle, "filecount", "1");
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_add_extra_data failed, %d", ret);
		goto BT_PRINT_END;
	}

	ret = service_add_extra_data(handle, "files", uri);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_add_extra_data failed, %d", ret);
		goto BT_PRINT_END;
	}

	/*const char *pkgname = BLUETOOTH_UG_NAME;

	ret = service_set_package(handle, pkgname);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_set_uri failed, %d", ret);
		goto BT_PRINT_END;
	}*/

	ug = _ivug_ext_launch_ug(BLUETOOTH_UG_NAME, handle, func, data);

BT_PRINT_END:
	destroy_ret = service_destroy(handle);
	if(destroy_ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_destroy failed, %d", destroy_ret);
		return NULL;
	}

	return (ret == SERVICE_ERROR_NONE ? ug : NULL);
}


bool ivug_ext_launch_videoplayer(const char *uri)
{
	MSG_IMAGEVIEW_HIGH("%s. URI=%s", __func__, uri);

	int ret = -1;
	int destroy_ret = -1;

	service_h handle;

	ret = service_create(&handle);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_create failed, %d", ret);
		return false;
	}

	ret = service_set_operation(handle, SERVICE_OPERATION_VIEW);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_set_operation failed, %d", ret);
		goto VIDEO_PLAYER_END;
	}

	ret = service_set_uri (handle, uri);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_set_uri failed, %d", ret);
		goto VIDEO_PLAYER_END;
	}

	ret = service_add_extra_data(handle, "path", uri);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_add_extra_data failed, %d", ret);
		goto VIDEO_PLAYER_END;
	}

	ret = service_add_extra_data(handle, "launching_application", "image_viewer");
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_add_extra_data failed, %d", ret);
		goto VIDEO_PLAYER_END;
	}

	const char *pkgname = VIDEOPLAYER_PKG_NAME;

	ret = service_set_package(handle, pkgname);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_set_uri failed, %d", ret);
		goto VIDEO_PLAYER_END;
	}

	ret = service_send_launch_request(handle, ivug_ext_service_reply_cb, NULL);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_send_launch_request failed, %d", ret);
		goto VIDEO_PLAYER_END;
	}

VIDEO_PLAYER_END:
	destroy_ret = service_destroy(handle);
	if(destroy_ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_destroy failed, %d", destroy_ret);
		return false;
	}

	return (ret == SERVICE_ERROR_NONE ? true : false);
}

bool ivug_ext_launch_videoplayer_simple(const char *uri)
{
	MSG_IMAGEVIEW_HIGH("%s. URI=%s", __func__, uri);

	int ret = -1;
	int destroy_ret = -1;

	service_h handle;

	ret = service_create(&handle);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_create failed, %d", ret);
		return false;
	}

	ret = service_set_operation(handle, SERVICE_OPERATION_VIEW);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_set_operation failed, %d", ret);
		goto VIDEO_PLAYER_SIMPLE_END;
	}

	ret = service_set_uri (handle, uri);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_set_uri failed, %d", ret);
		goto VIDEO_PLAYER_SIMPLE_END;
	}

	ret = service_add_extra_data(handle, "path", uri);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_add_extra_data failed, %d", ret);
		goto VIDEO_PLAYER_SIMPLE_END;
	}

	// Camera -> Image Viewer -> Video player, In this case, launching_application bundle's value should be "light_play_view"
	ret = service_add_extra_data(handle, "launching_application", "light_play_view");
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_add_extra_data failed, %d", ret);
		goto VIDEO_PLAYER_SIMPLE_END;
	}

	const char *pkgname = VIDEOPLAYER_PKG_NAME;

	ret = service_set_package(handle, pkgname);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_set_uri failed, %d", ret);
		goto VIDEO_PLAYER_SIMPLE_END;
	}

	ret = service_send_launch_request(handle, ivug_ext_service_reply_cb, NULL);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_send_launch_request failed, %d", ret);
		goto VIDEO_PLAYER_SIMPLE_END;
	}

VIDEO_PLAYER_SIMPLE_END:
	destroy_ret = service_destroy(handle);
	if(destroy_ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_destroy failed, %d", destroy_ret);
		return false;
	}

	return (ret == SERVICE_ERROR_NONE ? true : false);
}

bool ivug_ext_launch_browser(const char *uri)
{
	MSG_IMAGEVIEW_HIGH("%s. URI=%s", __func__, uri);

	int ret = -1;
	int destroy_ret = -1;

	service_h handle;

	ret = service_create(&handle);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_create failed, %d", ret);
		return false;
	}

	ret = service_set_operation(handle, SERVICE_OPERATION_VIEW);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_set_operation failed, %d", ret);
		goto BROWSER_END;
	}

	ret = service_add_extra_data(handle, "url", uri);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_add_extra_data failed, %d", ret);
		goto BROWSER_END;
	}

	const char *pkgname = BROWSER_PKG_NAME;

	ret = service_set_package(handle, pkgname);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_set_uri failed, %d", ret);
		goto BROWSER_END;
	}

	ret = service_send_launch_request(handle, ivug_ext_service_reply_cb, NULL);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_send_launch_request failed, %d", ret);
		goto BROWSER_END;
	}

BROWSER_END:
	destroy_ret = service_destroy(handle);
	if(destroy_ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_destroy failed, %d", destroy_ret);
		return false;
	}

	return (ret == SERVICE_ERROR_NONE ? true : false);
}

ui_gadget_h  ivug_ext_launch_select_contact(ug_result_cb result_func, ug_destroy_cb destroy_func, void *data)
{
	MSG_IMAGEVIEW_HIGH("%s", __func__);

	int ret = -1;
	int destroy_ret = -1;

	service_h handle;
	ui_gadget_h ug = NULL;

	ret = service_create(&handle);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_create failed, %d", ret);
		return NULL;
	}

	/*ret = service_set_operation(handle, SERVICE_OPERATION_VIEW);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_set_operation failed, %d", ret);
		goto SELECT_CONTACT_END;
	}*/

	ret = service_add_extra_data(handle, "type", "11");
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_add_extra_data failed, %d", ret);
		goto SELECT_CONTACT_END;
	}

	/*const char *pkgname = CONTACT_SELECT_UG_NAME;

	ret = service_set_package(handle, pkgname);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_set_uri failed, %d", ret);
		goto SELECT_CONTACT_END;
	}*/

	char xwin_id_str[12] = {0,};
	Ecore_X_Window xwin_id = elm_win_xwindow_get(ug_get_window());
	eina_convert_itoa(xwin_id, xwin_id_str);
	ret = service_add_extra_data(handle, "XWINDOW_ID", xwin_id_str);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_add_extra_data failed, %d", ret);
		goto SELECT_CONTACT_END;
	}

	ug = _ivug_ext_launch_ug_with_result(CONTACT_SELECT_UG_NAME, handle, result_func, destroy_func, data);

SELECT_CONTACT_END:
	destroy_ret = service_destroy(handle);
	if(destroy_ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_destroy failed, %d", destroy_ret);
		return NULL;
	}

	return (ret == SERVICE_ERROR_NONE ? ug : NULL);
}

bool ivug_ext_launch_default(const char *uri, const char *operation, const char *pkg, void *data)
{
	MSG_IMAGEVIEW_HIGH("%s.", __func__);

	int ret = -1;
	int destroy_ret = -1;

	service_h handle;

	ret = service_create(&handle);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_create failed, %d", ret);
		return false;
	}

	ret = service_set_operation(handle, operation);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_set_operation %s failed, %d", operation, ret);
		goto LAUNCH_END;
	}

	ret = service_set_uri (handle, uri);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_set_uri %s failed, %d", uri, ret);
		goto LAUNCH_END;
	}

	if(pkg)
	{
		ret = service_set_package(handle, pkg);
		if(ret != SERVICE_ERROR_NONE)
		{
			MSG_IMAGEVIEW_ERROR("service_set_package %s failed, %d", pkg, ret);
			goto LAUNCH_END;
		}
	}

	ret = service_send_launch_request(handle, ivug_ext_service_reply_cb, NULL);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_send_launch_request failed, %d", ret);
		goto LAUNCH_END;
	}

LAUNCH_END:
	destroy_ret = service_destroy(handle);
	if(destroy_ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_destroy failed, %d", destroy_ret);
		return false;
	}

	return (ret == SERVICE_ERROR_NONE ? true : false);
}

bool ivug_ext_launch_to_send(const char *uri, const char *pkg,  ug_destroy_cb func, void *data, ui_gadget_h ug)
{
	MSG_IMAGEVIEW_HIGH("%s: %s %s", __func__, uri, pkg);

	bool ret = false;
	ug = NULL;

	if (!strcmp(pkg, MESSAGE_UG_NAME))
	{
		ug = ivug_ext_launch_message(uri, func, data);
		ret = ug ? true : false;
	}
	else if (!strcmp(pkg, EMAIL_UG_NAME))
	{
		ret = ivug_ext_launch_email(uri);
	}
	else if (!strcmp(pkg, BLUETOOTH_PKG_NAME) || !strcmp(pkg, BLUETOOTH_UG_NAME))
	{
		ug = ivug_ext_launch_bluetooth_send(uri, func, data);
		ret = ug ? true : false;
	}
	else
	{
		/* Use default launch for other execs but bluetooth, that always requires settings */
		ret = ivug_ext_launch_default(uri, SERVICE_OPERATION_SEND, pkg, data);
	}

	return ret;
}

bool ivug_ext_launch_print(const char *uri)
{
	MSG_IMAGEVIEW_HIGH("%s. URI=%s", __func__, uri);

	int ret = -1;
	int destroy_ret = -1;
	const char *pkgname = PRINT_PKG_NAME;

	if(uri == NULL)
	{
		MSG_IMAGEVIEW_ERROR("URI is NULL");
		return false;
	}

	service_h handle;

	ret = service_create(&handle);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_create failed, %d", ret);
		return false;
	}

	ret = service_set_operation(handle, OPERATION_NAME_PRINT);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_set_operation failed, %d", ret);
		goto PRINT_END;
	}

	ret = service_set_package(handle, pkgname);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_set_package failed, %d", ret);
		goto PRINT_END;
	}

	ret = service_set_uri (handle, uri);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_set_uri failed, %d", ret);
		goto PRINT_END;
	}

	ret = service_add_extra_data(handle, SERVICE_PRINT_FILES_TYPE, "IMG");
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_add_extra_data failed, %d", ret);
		goto PRINT_END;
	}

	ret = service_send_launch_request(handle, ivug_ext_service_reply_cb, NULL);
	if(ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_send_launch_request failed, %d", ret);
		goto PRINT_END;
	}

PRINT_END:
	destroy_ret = service_destroy(handle);
	if(destroy_ret != SERVICE_ERROR_NONE)
	{
		MSG_IMAGEVIEW_ERROR("service_destroy failed, %d", destroy_ret);
		return false;
	}

	return (ret == SERVICE_ERROR_NONE ? true : false);
}

