/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "ivug-common.h"
#include "ivug-util.h"

#include "ivug-db.h"

#include <ui-gadget-module.h>		// ug_destroy_me
#include "ivug-context.h"

#include "ivug-callback.h"

#define MAX_INSTALNCE (5)

typedef struct
{
	ui_gadget_h ug;			// ug handle

	Evas_Object *parent_win;
// EFL theme
	Elm_Theme *th;

// Window Geometry
	int win_w, win_h;	//size
	int rot;		//window rotation value ( 0~360 )

	callback_handle_t *callback_handle;

	bool bDesktopMode;

} AppData;

static Eina_List *ContextList = NULL;

Elm_Theme*
gGetSystemTheme(void)
{
	AppData *ugContext;
	ugContext = eina_list_data_get(ContextList);

	IV_ASSERT(ugContext != NULL);

	return ugContext->th;
}

void
gGetSystemDimension(int *w, int *h)
{
	AppData *ugContext;
	ugContext = eina_list_data_get(ContextList);

	IV_ASSERT(ugContext != NULL);
	IV_ASSERT(w != NULL);
	IV_ASSERT(h != NULL);

	*w = ugContext->win_w;
	*h = ugContext->win_h;
}


void
gSetRotationDegree(int degree)
{
	AppData *ugContext;
	ugContext = eina_list_data_get(ContextList);

	IV_ASSERT(ugContext != NULL);

	ugContext->rot = degree;
}



int
gGetRotationDegree(void)
{
#if 0
	AppData *ugContext;
	ugContext = eina_list_data_get(ContextList);

	IV_ASSERT(ugContext != NULL);

	return ugContext->rot;
#else
	return elm_win_rotation_get(ug_get_window());
#endif

}

int gGetScreenWidth()
{
	AppData *ugContext;
	ugContext = eina_list_data_get(ContextList);

	IV_ASSERT(ugContext != NULL);

	return ugContext->win_w;

}

int gGetScreenHeight()
{
	AppData *ugContext;
	ugContext = eina_list_data_get(ContextList);

	IV_ASSERT(ugContext != NULL);

	return ugContext->win_h;

}

ui_gadget_h
gGetUGHandle(void)
{
	AppData *ugContext;
	ugContext = eina_list_data_get(ContextList);

	IV_ASSERT(ugContext != NULL);

	MSG_IMAGEVIEW_HIGH("Get ug handle = 0x%08x", ugContext->ug);

	return ugContext->ug;
}

Evas_Object *
gGetCurrentWindow(void)
{
	AppData *ugContext;
	ugContext = eina_list_data_get(ContextList);

	IV_ASSERT(ugContext != NULL);

	MSG_IMAGEVIEW_HIGH("Get ug handle = 0x%08x", ugContext->ug);

	return ugContext->parent_win;
}

callback_handle_t *
gGetCallbackHandle(void)
{
	AppData *ugContext;
	ugContext = eina_list_data_get(ContextList);

	IV_ASSERT(ugContext != NULL);

	return ugContext->callback_handle;
}

bool
gIsDesktopMode(void)
{
	AppData *ugContext;
	ugContext = eina_list_data_get(ContextList);

	IV_ASSERT(ugContext != NULL);

	return ugContext->bDesktopMode;
}

void
gSetDesktopMode(bool isDesktop)
{
	AppData *ugContext;
	ugContext = eina_list_data_get(ContextList);

	IV_ASSERT(ugContext != NULL);

	ugContext->bDesktopMode = isDesktop;
}

#define IVUG_NAVIFRAME_NAME EDJ_PATH"/ivug-naviframe.edj"
#define IVUG_TOOLBAR_NAME EDJ_PATH"/ivug-toolbar.edj"
#define IVUG_BUTTON_NAME EDJ_PATH"/ivug-button.edj"
#define IVUG_SELECTION_INFO_NAME	EDJ_PATH"/ivug-selectioninfo.edj"
#define IVUG_GENLIST	EDJ_PATH"/ivug-genlist.edj"
#define IVUG_GENGRID	EDJ_PATH"/ivug-gengrid.edj"

static char *edj[] = {
	IVUG_NAVIFRAME_NAME,
	IVUG_TOOLBAR_NAME,
	IVUG_BUTTON_NAME,
	IVUG_SELECTION_INFO_NAME,
	IVUG_GENLIST,
	IVUG_GENGRID,
	NULL,
};

bool
ivug_context_init(ui_gadget_h ug)
{
	AppData *Context = (AppData *)calloc(1, sizeof(AppData));
	if(Context == NULL)
	{
		MSG_IVUG_ERROR("Cannot allock memory");
		return false;
	}

	Context->ug = ug;
	Context->parent_win = ug_get_window();

	if ( Context->parent_win )
	{
		Context->rot = elm_win_rotation_get(Context->parent_win);
	}
	else
	{
		MSG_IMAGEVIEW_WARN("Cannot get parent window. rotation angle set as 0");
		Context->rot = 0;
	}

//get window width, height
	int w, h;

	evas_object_geometry_get(Context->parent_win, NULL, NULL, &w, &h);
	MSG_IMAGEVIEW_HIGH("Screen WH(%dx%d)", w, h);
//	ecore_x_window_size_get(ecore_x_window_root_first_get(), &w, &h);
	Context->win_w = w;
	Context->win_h = h;


	PERF_CHECK_BEGIN(LVL2, "theme add");

	int i = 0;

	Context->th = elm_theme_new();

	IV_ASSERT(Context->th != NULL);

	elm_theme_ref_set(Context->th, NULL);

	for ( i = 0; edj[i] != NULL; i++)
	{
		elm_theme_extension_add(Context->th, edj[i]);
	}

	PERF_CHECK_END(LVL2, "theme add");

	ContextList = eina_list_prepend(ContextList, Context);

	MSG_IVUG_HIGH("Append to list. Context=0x%08x", Context);

#ifdef USE_NEW_DB_API
	PERF_CHECK_BEGIN(LVL2, "media svc connect");

	ivug_db_create();

	PERF_CHECK_END(LVL2, "media svc connect");
#endif

	Context->callback_handle = ivug_callback_register();
	if(Context->callback_handle == NULL)
	{
		MSG_IVUG_ERROR("ivug_callback_register error");
		goto ERROR;
	}

	Context->bDesktopMode = false;
	const char *profile = elm_config_profile_get();
	if (strcmp(profile,"desktop") == 0)
	{
		Context->bDesktopMode = true;
	}

	bindtextdomain(IVUG_TEXT_DOMAIN, IVUG_LOCALEDIR); 	//bind text domain
	return TRUE;

/**** Error routine ****/
ERROR:

	if(Context->callback_handle)
	{
		ivug_callback_unregister(Context->callback_handle);
	}


#ifdef USE_NEW_DB_API
	ivug_db_destroy();
#endif

	Context = eina_list_data_get(ContextList);
	ContextList = eina_list_remove_list(ContextList, ContextList);

	if(Context->th)
	{
		int i = 0;
		for ( i = 0; edj[i] != NULL; i++)
		{
			elm_theme_extension_del(Context->th, edj[i]);
		}

		elm_theme_free(Context->th);
	}

	MSG_IVUG_HIGH("Remove from list. Context=0x%08x, ug=0x%08x", Context, Context->ug);

	free(Context);

	return false;
}


bool
ivug_context_deinit(ui_gadget_h ug)
{
	AppData *Context = NULL;

	if(ContextList == NULL)
	{
		MSG_IVUG_ERROR("ContextList is NULL");
		return false;
	}

	Eina_List *l = NULL;
	Eina_List *l_next = NULL;

	EINA_LIST_FOREACH_SAFE(ContextList, l, l_next, Context)
	{
		if(Context == NULL)
		{
			MSG_IVUG_ERROR("Context is NULL");
			return false;
		}
		if (Context->ug == ug)
		{
			ContextList = eina_list_remove_list(ContextList, l);
			MSG_IVUG_HIGH("Find ug handle(0x%08x) from ContextList.", ug);

			break;
		}
	}

	if(Context->callback_handle)
	{
		MSG_IVUG_HIGH("Removing Callback");
		ivug_callback_unregister(Context->callback_handle);
		Context->callback_handle = NULL;
	}

#ifdef USE_NEW_DB_API
	PERF_CHECK_BEGIN(LVL2, "ivug_db_destroy");
	ivug_db_destroy();
	PERF_CHECK_END(LVL2, "ivug_db_destroy");
#endif

	PERF_CHECK_BEGIN(LVL2, "elm_theme_free");
	if(Context->th)
	{
		int i = 0;
		for ( i = 0; edj[i] != NULL; i++)
		{
			elm_theme_extension_del(Context->th, edj[i]);
		}
		elm_theme_free(Context->th);
	}
	PERF_CHECK_END(LVL2, "elm_theme_free");

	MSG_IVUG_HIGH("Remove from list. Context=0x%08x, ug=0x%08x", Context, Context->ug);

	free(Context);

	return true;
}


void ivug_context_destroy_me(const char *file, int line)
{
	AppData *ugContext;
	ugContext = eina_list_data_get(ContextList);

	char *fname = strrchr(file, '/');

	if (NULL != ugContext)
	{
		MSG_IMAGEVIEW_HIGH("Request to destory ug. UGhandle(0x%08x) from L(%d) %s", ugContext->ug, line, fname);
		ug_destroy_me(ugContext->ug);
	}
	else
	{
		MSG_IMAGEVIEW_WARN("ugContext is NULL. from L(%d) %s", line, fname);
	}

}


