/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "ivug-detail-info.h"

#include <drm_client.h>

#include <glib.h>

#include "ivug-details-view.h"
#include <libexif/exif-data.h>

#include "ivug-drm-common.h"

#include "ivug-file-info.h"

#include "ivug-string.h"

#include "ivug-util.h"


#define NOMAL_BUF   		128

#define DETAILS_DIR_PATH_LEN_MAX	4096

enum IVUG_SIZE_TYPE
{
	SIZE_BYTE = 0,
	SIZE_KB,
	SIZE_MB,
	SIZE_GB
};

/*
	Should free returned value after use
*/
static char *
_get_drm_availabile_string(drm_permission_type_e permissionType)
{
	switch(permissionType)
	{
		case DRM_PERMISSION_TYPE_ANY:
			return strdup(IDS_ANY);
			break;

		case DRM_PERMISSION_TYPE_PLAY:
			return strdup(IDS_PLAY);
			break;

		case DRM_PERMISSION_TYPE_COLLABORATIVE_PLAY:
			return strdup(IDS_COLLABORATIVE_PLAY);
			break;

		case DRM_PERMISSION_TYPE_DISPLAY:
			return strdup(IDS_DISPLAY);
			break;

		case DRM_PERMISSION_TYPE_EXECUTE:
			return strdup(IDS_EXECUTE);
			break;

		case DRM_PERMISSION_TYPE_PRINT:
			return strdup(IDS_PRINT);
			break;

		case DRM_PERMISSION_TYPE_EXPORT_COPY:
			  return strdup(IDS_COPY);
			break;

		case DRM_PERMISSION_TYPE_EXPORT_MOVE:
			  return strdup(IDS_MOVE);
			break;

		default:
			return strdup(IDS_N_A);
			break;

	}
}

static char *
_get_filesize_string(long long size)
{
#define FILE_SIZE_LEN_MAX			64

	MSG_DETAIL_HIGH("Entered size = %lld", size);

	char *tmp = (char *)calloc(1, sizeof(char)*(FILE_SIZE_LEN_MAX+1));
	if (tmp == NULL)
	{
		return NULL;
	}

	if(size > (1<<30))	// GB
	{
		snprintf(tmp,FILE_SIZE_LEN_MAX,"%.1f GB", (double)size/(1<<30));
	}
	else if(size > (1<<20))	// MB
	{
		snprintf(tmp,FILE_SIZE_LEN_MAX,"%.1f MB", (double)size/(1<<20));
	}
	else if(size > (1<<10))	// KB
	{
		snprintf(tmp,FILE_SIZE_LEN_MAX,"%.0f KB", (double)size/(1<<10));
	}
	else	// Byte
	{
		snprintf(tmp,FILE_SIZE_LEN_MAX,"%lld B", size);
	}

	MSG_DETAIL_HIGH("Result Size = %s", tmp);
	return tmp;
}

void _add_string(char **org, char *add_string)
{
	char *string = strdup(add_string);
	if(*org == NULL)
	{
		*org = string;
	}
	else
	{
		*org = g_strconcat(*org, "/", string, NULL);
		free(string);
	}
}

bool
ivug_datail_get_exif_info(/*out*/char** name, /*out*/char **contents,
	const char *path, Media_Type stype, exif_info_type_e type)
{
#define UG_EXIF_ARR_LENGTH 			255

	bool bret = false;
	char *string = NULL;
	int status = 0;

	double latitude = 0.0;
	double longtitude = 0.0;

	switch(type)
	{
		case EXIF_INFO_ORIENTATION:
			status = ivug_fileinfo_get_orientation(path);
			if(status < 0)
				return false;
			*name = strdup(IDS_ORIENTATION);
			switch(status)
			{
				case 1:
				case 2:
					*contents = strdup("0");
					break;
				case 3:
				case 4:
					*contents = strdup("180");
					break;
				case 5:
				case 8:
					*contents = strdup("270");
					break;
				case 6:
				case 7:
					*contents = strdup("90");
					break;
				default:
					*contents = strdup(IDS_UNKNOWN);
					break;
			}
			break;
		case EXIF_INFO_LATITUDE:
			if(stype == SLIDE_TYPE_IMAGE)		// this will be removed!!!!
			{
				bret = ivug_fileinfo_get_image_gps_info(path, &latitude, &longtitude);
			}
			else if(stype == SLIDE_TYPE_VIDEO)
			{
				bret = ivug_fileinfo_get_video_gps_info(path, &latitude, &longtitude);
			}
			if(bret == false)
			{
				MSG_DETAIL_WARN("get latitude error");
				return false;
			}
			string = calloc(1, sizeof(char)*UG_EXIF_ARR_LENGTH);
			snprintf(string , UG_EXIF_ARR_LENGTH, "%.5f", latitude);

			*name = strdup(IDS_LATITUDE);
			*contents = string;
			break;

		case EXIF_INFO_LONGITTUDE:
			if(stype == SLIDE_TYPE_IMAGE)		// this will be removed!!!!
			{
				bret = ivug_fileinfo_get_image_gps_info(path, &latitude, &longtitude);
			}
			else if(stype == SLIDE_TYPE_VIDEO)
			{
				bret = ivug_fileinfo_get_video_gps_info(path, &latitude, &longtitude);
			}
			if(bret == false)
			{
				MSG_DETAIL_WARN("get longtitude error");
				return false;
			}
			string = calloc(1, sizeof(char)*UG_EXIF_ARR_LENGTH);
			snprintf(string , UG_EXIF_ARR_LENGTH, "%.5f", longtitude);

			*name = strdup(IDS_LONGITUDE);
			*contents = string;
			break;

		case EXIF_INFO_MANUFACTURE:
			string = ivug_fileinfo_get_maker_name(path);
			if(string == NULL)
				return false;
			*name = strdup(IDS_MANUFACTURE);
			*contents = string;
			break;
		case EXIF_INFO_MODEL:
			string = ivug_fileinfo_get_model_name(path);
			if(string == NULL)
				return false;
			*name = strdup(IDS_MODEL);
			*contents = string;
			break;
		case EXIF_INFO_FLASH:
			status = ivug_fileinfo_get_flash_status(path);
			if(status < 0)
				return false;
			*name = strdup(IDS_FLASH);
			if(status == 1)
			{
				*contents = strdup(IDS_FLASH);
			}
			else
			{
				*contents = strdup(IDS_NO_FLASH);
			}
			break;
		case EXIF_INFO_FOCAL_LENGTH:
			string = ivug_fileinfo_get_focal_length(path);
			if(string == NULL)
				return false;
			*name = strdup(IDS_FOCAL_LENGTH);
			*contents = string;
			break;
		case EXIF_INFO_WHITE_BALANCE:
			status = ivug_fileinfo_get_white_balance(path);
			if(status < 0)
				return false;
			*name = strdup(IDS_WHITE_BALANCE);
			if(status == 0)
			{
				*contents = strdup(IDS_AUTO);
			}
			else
			{
				*contents = strdup(IDS_MANUAL);
			}
			break;
		case EXIF_INFO_APERTURE:
			string = ivug_fileinfo_get_aperture(path);
			if(string == NULL)
				return false;
			*name = strdup(IDS_APERTURE);
			*contents = string;
			break;
		case EXIF_INFO_EXPOSURE_TIME:
			string = ivug_fileinfo_get_exposure_time(path);
			if(string == NULL)
				return false;
			*name = strdup(IDS_EXPOSURE_TIME);
			*contents = string;
			break;
		case EXIF_INFO_ISO:
			string = ivug_fileinfo_get_iso(path);
			if(string == NULL)
				return false;
			*name = strdup(IDS_ISO);
			*contents = string;
			break;
		default:
			break;
	}

	return true;
}


bool
ivug_datail_get_file_info(/*out*/char** name, /*out*/char **contents,
	const char *path, Media_Type stype, file_info_type_e type)
{
	struct stat statbuf;
	int width = 0;
	int height = 0;

	switch(type)
	{
		case FILE_INFO_NAME:
			*name = strdup(IDS_NAME);
			*contents = ecore_file_strip_ext(ecore_file_file_get(path));
			break;
		case FILE_INFO_FORMAT:
			*name = strdup(IDS_FORMAT);
			*contents = ivug_fileinfo_get_file_extension(path);
			if(*contents == NULL)
			{
				*contents = strdup(IDS_UNKNOWN);
			}
			break;
		case FILE_INFO_DATE_CREATED:
			*name = strdup(IDS_DATE_CREATED);
			if(stat(path, &statbuf) == -1)
			{
				MSG_DETAIL_ERROR("Cannot get stat()");
				*contents = strdup(IDS_UNKNOWN);
				break;
			}
			*contents = ivug_get_icu_date(statbuf.st_mtime);
			if(*contents == NULL)
			{
				*contents = strdup(IDS_UNKNOWN);
			}
			break;
		case FILE_INFO_SIZE:
			*name = strdup(IDS_SIZE);
			if(stat(path, &statbuf) == -1)
			{
				MSG_DETAIL_ERROR("Cannot get stat()");
				*contents = strdup(IDS_UNKNOWN);
				break;
			}
			*contents = _get_filesize_string(statbuf.st_size);
			if(*contents == NULL)
			{
				*contents = strdup(IDS_UNKNOWN);
			}
			break;
		case FILE_INFO_RESOLUTION:
			*name = strdup(IDS_RESOLUTION);
			*contents = (char *)calloc(1, sizeof(char)*NOMAL_BUF+1);
			if(stype == SLIDE_TYPE_IMAGE)
			{
				if(ivug_fileinfo_get_image_resolution(path, &width, &height) == false)
				{
					snprintf(*contents, NOMAL_BUF, "%s", IDS_N_A);
				}
				else
				{
					snprintf(*contents, NOMAL_BUF,"%dX%d", width, height);
				}
			}
			else if(stype == SLIDE_TYPE_VIDEO)
			{
				if ( ivug_fileinfo_get_video_resolution(path, &width, &height) == false)
				{
					snprintf(*contents, NOMAL_BUF, "%s", IDS_N_A);
				}
				else
				{
					snprintf(*contents, NOMAL_BUF,"%dX%d", width, height);
				}
			}
			else
			{
				snprintf(*contents, NOMAL_BUF, "%s", IDS_N_A);
			}

			break;
		case FILE_INFO_FILE_LOCATION:
			*name = strdup(IDS_LOCATION);
			*contents = strdup(path);
			break;
		default:
			break;
	}

	return true;
}


bool
ivug_datail_get_drm_info(/*out*/char** name, /*out*/char **contents,
	const char *path, Media_Type stype, drm_info_type_e type)
{
#define VALIDITY_SIZE	80

#define MOUNTH_COUNT	12					//used for drm file info,
#define DAY_COUNT		30
#define HOUR_COUNT		24
#define SECOND_COUNT	3600.0

	bool bret = false;

	if(ivug_drm_has_valid_ro(path) == false)
	{
		MSG_DETAIL_WARN("%s is not valid right object", path);
		return false;
	}

	char *string = NULL;
	drm_constraint_info_s constraint_info;
	drm_file_info_s info;
	int ret = -1;
	int i = -1;
	char validity[VALIDITY_SIZE] = "";

	switch(type)
	{
		case DRM_INFO_VENDOR:
			*name = strdup(IDS_VENDOR);
			*contents = ivug_drm_get_drm_vendor_info(path);
			if(*contents == NULL)
			{
				*contents = strdup(IDS_UNKNOWN);
			}
			break;
		case DRM_INFO_FORWARDING:
			*name = strdup(IDS_FORWARDING);
			bret = ivug_drm_get_forwarding_info(path);
			if(bret == true)
			{
				*contents = strdup(IDS_POSSIBLE);
			}
			else
			{
				*contents = strdup(IDS_IMPOSSIBLE);
			}
			break;
		case DRM_INFO_AVAILABLE_USES:
			{
				*name = strdup(IDS_AVAILABLE_USES);

				memset(&info, 0x0, sizeof(drm_file_info_s));
				ret = drm_get_file_info(path, &info);
				if(ret != DRM_RETURN_SUCCESS)
				{
					MSG_IVUG_ERROR("drm_get_file_info Failed!!, ret=0x%x", ret);
					*contents = strdup(IDS_UNKNOWN);
					return true;
				}
				if (info.oma_info.method == DRM_METHOD_TYPE_FORWARD_LOCK)
				{
					MSG_IVUG_HIGH("DRM_METHOD_TYPE_FORWARD_LOCK");
					*contents = strdup(IDS_ANY);
					return true;
				}

				for(i=DRM_PERMISSION_TYPE_NONE; i<=DRM_PERMISSION_TYPE_EXPORT_MOVE; i++)
				{
					memset(&constraint_info, 0x0, sizeof(drm_constraint_info_s));

					ret = drm_get_constraint_info(path, i, &constraint_info);
					if(ret != DRM_RETURN_SUCCESS)
					{
						MSG_IVUG_ERROR("drm_get_constraint_info Failed!!, perm_type=%d ret=0x%x", i, ret);
						continue;
					}

					string = _get_drm_availabile_string(i);
					_add_string(contents, string);
					free(string);
				}
				if(*contents == NULL)
				{
					*contents = strdup(IDS_UNKNOWN);
				}
			}
			break;
		case DRM_INFO_TYPE:
			{
				*name = strdup(IDS_TYPE);

				memset(&info, 0x0, sizeof(drm_file_info_s));
				ret = drm_get_file_info(path, &info);
				if(ret != DRM_RETURN_SUCCESS)
				{
					MSG_IVUG_ERROR("drm_get_file_info Failed!!, ret=0x%x", ret);
					*contents = strdup(IDS_UNKNOWN);
					return true;
				}
				if (info.oma_info.method == DRM_METHOD_TYPE_FORWARD_LOCK)
				{
					MSG_IVUG_HIGH("DRM_METHOD_TYPE_FORWARD_LOCK");
					*contents = strdup(IDS_UNLIMITED);
					return true;
				}

				for(i=DRM_PERMISSION_TYPE_NONE; i<=DRM_PERMISSION_TYPE_EXPORT_MOVE; i++)
				{
					memset(&constraint_info, 0x0, sizeof(drm_constraint_info_s));

					ret = drm_get_constraint_info(path, i, &constraint_info);
					if(ret != DRM_RETURN_SUCCESS)
					{
						MSG_IVUG_ERROR("drm_get_constraint_info Failed!!, perm_type=%d ret=0x%x", i, ret);
						continue;
					}
					if(constraint_info.const_type.is_unlimited)
						_add_string(contents, IDS_UNLIMITED);

					if(constraint_info.const_type.is_accumulated)
						_add_string(contents, IDS_ACCUMULATED_TIME);

					if(constraint_info.const_type.is_count)
						_add_string(contents, IDS_COUNT);

					if(constraint_info.const_type.is_datetime)
						_add_string(contents, IDS_TIME);

					if(constraint_info.const_type.is_individual)
						_add_string(contents, IDS_INDIVIDUAL);

					if(constraint_info.const_type.is_interval)
						_add_string(contents, IDS_INTERVAL);

					if(constraint_info.const_type.is_system)
						_add_string(contents, IDS_SYSTEM);

					if(constraint_info.const_type.is_timedcount)
						_add_string(contents, IDS_TIMED_COUNT);
				}
				if(*contents == NULL)
				{
					*contents = strdup(IDS_UNKNOWN);
				}
			}
			break;
		case DRM_INFO_VALIDITY:
			*name = strdup(IDS_VALIDITY);

			memset(&info, 0x0, sizeof(drm_file_info_s));
			ret = drm_get_file_info(path, &info);
			if(ret != DRM_RETURN_SUCCESS)
			{
				MSG_IVUG_ERROR("drm_get_file_info Failed!!, ret=0x%x", ret);
				*contents = strdup(IDS_UNKNOWN);
				return true;
			}
			if (info.oma_info.method == DRM_METHOD_TYPE_FORWARD_LOCK)
			{
				MSG_IVUG_HIGH("DRM_METHOD_TYPE_FORWARD_LOCK");
				*contents = strdup(IDS_UNLIMITED);
				return true;
			}

			for(i=DRM_PERMISSION_TYPE_NONE; i<=DRM_PERMISSION_TYPE_EXPORT_MOVE; i++)
			{
				memset(&constraint_info, 0x0, sizeof(drm_constraint_info_s));

				ret = drm_get_constraint_info(path, i, &constraint_info);
				if(ret != DRM_RETURN_SUCCESS)
				{
					MSG_IVUG_ERROR("drm_get_constraint_info Failed!!, perm_type=%d ret=0x%x", i, ret);
					continue;
				}

				if(constraint_info.const_type.is_unlimited)
				{
					MSG_IVUG_HIGH("Unlimited:for permission=%d", i);
					_add_string(contents, IDS_UNLIMITED);
				}

				if(constraint_info.const_type.is_accumulated)
				{
					MSG_IVUG_HIGH("Accumulated Constraint is present");
					MSG_IVUG_HIGH("accumulated_original_seconds=%d",
							constraint_info.accumulated_original_seconds);
					MSG_IVUG_HIGH("accumulated_remaining_seconds=%d",
							constraint_info.accumulated_remaining_seconds);

					snprintf(validity, sizeof(validity),"Total %.1f hours",
						constraint_info.accumulated_remaining_seconds/SECOND_COUNT);
					_add_string(contents, validity);
				}

				if(constraint_info.const_type.is_count)
				{
					MSG_IVUG_HIGH("Count Constraint is present");
					MSG_IVUG_HIGH("original_count=%d",
							constraint_info.original_count);
					MSG_IVUG_HIGH("remaining_count=%d",
							constraint_info.remaining_count);

					snprintf(validity, sizeof(validity),"%d left", constraint_info.remaining_count);
					_add_string(contents, validity);
				}

				if(constraint_info.const_type.is_datetime)
				{
					MSG_IVUG_HIGH("DATETIME Constraint is present");
					MSG_IVUG_HIGH("start_time:"
							"Year=%d,Month=%d,Day=%d,Hour=%d, minute=%d, Second=%d",
						constraint_info.start_time.tm_year,
						constraint_info.start_time.tm_mon,
						constraint_info.start_time.tm_mday,
						constraint_info.start_time.tm_hour,
						constraint_info.start_time.tm_min,
						constraint_info.start_time.tm_sec);

					MSG_IVUG_HIGH("end_time:"
							"Year=%d,Month=%d,Day=%d,Hour=%d, minute=%d, Second=%d",
						constraint_info.end_time.tm_year,
						constraint_info.end_time.tm_mon,
						constraint_info.end_time.tm_mday,
						constraint_info.end_time.tm_hour,
						constraint_info.end_time.tm_min,
						constraint_info.end_time.tm_sec);

					snprintf(validity, sizeof(validity),"%d.%d.%d %d:%d ~  %d.%d.%d %d:%d:%d",
							constraint_info.start_time.tm_mday,
							constraint_info.start_time.tm_mon,
							constraint_info.start_time.tm_year,
							constraint_info.start_time.tm_hour,
							constraint_info.start_time.tm_sec,
							constraint_info.end_time.tm_mday,
							constraint_info.end_time.tm_mon,
							constraint_info.end_time.tm_year,
							constraint_info.end_time.tm_hour,
							constraint_info.end_time.tm_min,
							constraint_info.end_time.tm_sec
							);
					_add_string(contents, validity);
				}

				if(constraint_info.const_type.is_individual)
				{
					MSG_IVUG_HIGH("INDIVIDUAL Constraint is present");
					MSG_IVUG_HIGH("individual_identity_type=%d "
							"individual_id=%s",
							constraint_info.individual_identity_type,
							constraint_info.individual_id);

					_add_string(contents, IDS_INDIVIDUAL);
				}

				if(constraint_info.const_type.is_interval)
				{
					MSG_IVUG_HIGH("INTERVAL Constraint is present");
					MSG_IVUG_HIGH("interval_time:"
							"Year=%d,Month=%d,Day=%d,Hour=%d, minute=%d, Second=%d",
						constraint_info.interval_time.tm_year,
						constraint_info.interval_time.tm_mon,
						constraint_info.interval_time.tm_mday,
						constraint_info.interval_time.tm_hour,
						constraint_info.interval_time.tm_min,
						constraint_info.interval_time.tm_sec);

					if(constraint_info.interval_time.tm_year == 0
					 	&& constraint_info.interval_time.tm_mon == 0
					 	&& constraint_info.interval_time.tm_mday == 0
					 	&& constraint_info.interval_time.tm_hour == 0)
					{
						snprintf(validity, sizeof(validity),"Total %d miniutes", constraint_info.interval_time.tm_min);
					}
					else
					{
						snprintf(validity, sizeof(validity),"Total %d hours",
								((constraint_info.interval_time.tm_year*MOUNTH_COUNT
									+ constraint_info.interval_time.tm_mon)*DAY_COUNT
									+ constraint_info.interval_time.tm_mday)*HOUR_COUNT
									+ constraint_info.interval_time.tm_hour
								);
					}
					_add_string(contents, validity);
				}

				if(constraint_info.const_type.is_system){
					MSG_IVUG_HIGH("SYSTEM Constraint is present");
					MSG_IVUG_HIGH("system_identity_type=%d "
							"system_id=%s",
							constraint_info.system_identity_type,
							constraint_info.system_id);

					_add_string(contents, IDS_SYSTEM);
				}

				if(constraint_info.const_type.is_timedcount){
					MSG_IVUG_HIGH("TIMED-COUNT Constraint is present");
					MSG_IVUG_HIGH("timed_original_count=%d",
							constraint_info.timed_original_count);

					MSG_IVUG_HIGH("timed_remaining_count=%d",
							constraint_info.timed_remaining_count);

					MSG_IVUG_HIGH("timed_count_timer=%d",
							constraint_info.timed_count_timer);

					snprintf(validity, sizeof(validity),"%d (%d sec)",
									constraint_info.timed_remaining_count,
									constraint_info.timed_count_timer
							);
					_add_string(contents, validity);
				}
			}
			if(*contents == NULL)
			{
				*contents = strdup(IDS_UNKNOWN);
			}
			break;
		default:
			break;
	}

	return true;
}

