/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <string.h>

#include "ivug-common.h"
#include "ivug-debug.h"
#include "ivug-parameter.h"
#include "ivug-file-info.h"

#include "ivug-db.h"
#include "ivug-util.h"

#include "ivug-filter.h"

//bundle key
#define IVUG_BUNDLE_KEY_VIEW_MODE 		"View Mode"
#define IVUG_BUNDLE_KEY_PATH			"Path"
#define IVUG_BUNDLE_KEY_THUMBNAIL_PATH	"Thumb Path"
#define IVUG_BUNDLE_KEY_ALBUM_IDX 		"Album index"
#define IVUG_BUNDLE_KEY_VIEW_BY			"View By"
#define IVUG_BUNDLE_KEY_INDEX			"Index"
#define IVUG_BUNDLE_KEY_SORT_BY			"Sort By"
#define IVUG_BUNDLE_KEY_FOOTSTEPS		"Footsteps"

#define IVUG_BUNDLE_KEY_MAX_LONGITUDE	"LON_MAX"
#define IVUG_BUNDLE_KEY_MIN_LONGITUDE	"LON_MIN"
#define IVUG_BUNDLE_KEY_MAX_LATITUDE	"LAT_MAX"
#define IVUG_BUNDLE_KEY_MIN_LATITUDE	"LAT_MIN"

#define IVUG_BUNDLE_KEY_SETAS_TYPE		"Setas type"
#define IVUG_BUNDLE_KEY_TAG_NAME		"Tag name"

#define IVUG_BUNDLE_KEY_RESOLUTION		"Resolution"
#define IVUG_BUNDLE_KEY_FIXED_RATIO		"Fixed ratio"

#define IVUG_BUNDLE_KEY_CROP_MODE		"http://tizen.org/appcontrol/data/image/crop_mode"
#define IVUG_BUNDLE_VALUE_CROP_MODE_AUTO		"auto"
#define IVUG_BUNDLE_VALUE_CROP_MODE_FIT_TO_SCREEN	"fit_to_screen"

#define IVUG_BUNDLE_KEY_MEDIA_TYPE		"Media type"

#define IVUG_BUNDLE_KEY_STANDALONE		"Standalone"

#define IVUG_BUNDLE_KEY_CONTACT_ID		"Contact id"
#define IVUG_BUNDLE_KEY_CLUSTER_ID		"Cluster id"
#define IVUG_BUNDLE_KEY_GROUP_ID		"Group id"

#define IVUG_BUNDLE_KEY_SELECTED_INDEX	"Selected index"

//default values
#define IVUG_DEFAULT_MODE IVUG_MODE_SINGLE
#define IVUG_DEFAULT_INDEX (1)				// First item

#define IVUG_FILE_PREFIX		"file://"

static inline
void _ivug_free(char **val)
{
	free(*val);
	*val = NULL;
}

static ivug_mode
_get_view_mode(ivug_parameter* data, const char* val)
{
	IV_ASSERT(val != NULL);

#define IVUG_BUNDLE_VALUE_VIEW_MODE_ALBUM 	"ALBUM"
#define IVUG_BUNDLE_VALUE_VIEW_MODE_NORMAL 	"NORMAL"
#define IVUG_BUNDLE_VALUE_VIEW_MODE_SINGLE 	"SINGLE"
#define IVUG_BUNDLE_VALUE_VIEW_MODE_GALLERY 	"GALLERY"
#define IVUG_BUNDLE_VALUE_VIEW_MODE_CAMERA	"CAMERA"
#define IVUG_BUNDLE_VALUE_VIEW_MODE_DISPLAY	"DISPLAY"
#define IVUG_BUNDLE_VALUE_VIEW_MODE_SETAS		"SETAS"
#define IVUG_BUNDLE_VALUE_VIEW_MODE_SLIDESHOW   "SLIDESHOW"

	int len = strlen(val);

	if (strncmp(val, IVUG_BUNDLE_VALUE_VIEW_MODE_NORMAL, len) == 0
		|| strncmp(val, IVUG_BUNDLE_VALUE_VIEW_MODE_GALLERY, len) == 0 )
	{
		return IVUG_MODE_NORMAL;
	}
	else if (strncmp(val, IVUG_BUNDLE_VALUE_VIEW_MODE_ALBUM, len) == 0)
	{
		// Probably, UnUsed.
		MSG_IVUG_ERROR("ALBUM is deprecated!!!! plz check");
		printf("bundle key ALBUM is deprecated!!!! plz check\n");
		return IVUG_MODE_NORMAL;
	}
	else if ( strncmp(val, IVUG_BUNDLE_VALUE_VIEW_MODE_CAMERA, len) == 0)
	{
		data->start_index = 1;						//apply window loading
		data->view_by = IVUG_VIEW_BY_FOLDER;		// In camera case, All images should be shown in camera folder.
		return IVUG_MODE_CAMERA;
	}
	else if (strncmp(val, IVUG_BUNDLE_VALUE_VIEW_MODE_SINGLE, len) == 0)
	{
		if(data->view_by != IVUG_VIEW_BY_FOLDER)
			data->view_by = IVUG_VIEW_BY_FILE;
		return IVUG_MODE_SINGLE;
	}
	else if ( strncmp(val, IVUG_BUNDLE_VALUE_VIEW_MODE_DISPLAY, len) == 0)
	{
		data->view_by = IVUG_VIEW_BY_FILE;
		return IVUG_MODE_DISPLAY;
	}
	else if(strncmp(val, IVUG_BUNDLE_VALUE_VIEW_MODE_SETAS, len) == 0)
	{
		data->view_by = IVUG_VIEW_BY_FILE;
		return IVUG_MODE_SETAS;
	}
	else if(strncmp(val, IVUG_BUNDLE_VALUE_VIEW_MODE_SLIDESHOW, len) == 0)
	{
		if(data->view_by == IVUG_VIEW_BY_INVAILD)
		{
			data->view_by = IVUG_VIEW_BY_FOLDER;
		}
		return IVUG_MODE_SLIDESHOW;
	}

	MSG_IVUG_ERROR("Invalid mode : %s", val);
	return IVUG_MODE_INVAILD;
}


static ivug_view_by
_get_view_by(const char* val)
{
	//bundle value
#define IVUG_BUNDLE_VALUE_VIEW_BY_ALL			"All"
#define IVUG_BUNDLE_VALUE_VIEW_BY_FAVORITES		"Favorites"
#define IVUG_BUNDLE_VALUE_VIEW_BY_TAGS			"Tags"
#define IVUG_BUNDLE_VALUE_VIEW_BY_FLODER		"By Folder"
#define IVUG_BUNDLE_VALUE_VIEW_BY_PLACES		"Places"

	IV_ASSERT(val != NULL);

	int len = strlen(val);

	if (strncmp(val, IVUG_BUNDLE_VALUE_VIEW_BY_FLODER, len) == 0 )
	{
		return IVUG_VIEW_BY_FOLDER;
	}
	else if (strncmp(val, IVUG_BUNDLE_VALUE_VIEW_BY_ALL, len) == 0 )
	{
		return IVUG_VIEW_BY_ALL;
	}
	else if (strncmp(val, IVUG_BUNDLE_VALUE_VIEW_BY_FAVORITES, len) == 0)
	{
		return IVUG_VIEW_BY_FAVORITES;
	}
	else if (strncmp(val, IVUG_BUNDLE_VALUE_VIEW_BY_TAGS, len) == 0)
	{
		return IVUG_VIEW_BY_TAG;
	}
	else if (strncmp(val, IVUG_BUNDLE_VALUE_VIEW_BY_PLACES, len) == 0)
	{
		return IVUG_VIEW_BY_PLACES;
	}

	MSG_IVUG_ERROR("Invalid view by : %s", val);

	return IVUG_VIEW_BY_FILE;

}


static ivug_setas_type
_get_setas_type(const char* val)
{
	IV_ASSERT(val != NULL);

#define IVUG_BUNDLE_VALUE_SETAS_UG_WALLPAPER	"Wallpaper"
#define IVUG_BUNDLE_VALUE_SETAS_UG_LOCKSCREEN	"Lockscreen"
#define IVUG_BUNDLE_VALUE_SETAS_UG_WALLPAPER_N_LOCKSCREEN	"Wallpaper & Lockscreen"
#define IVUG_BUNDLE_VALUE_SETAS_UG_CALLERID		"CallerID"
#define IVUG_BUNDLE_VALUE_SETAS_UG_VIDEO_CALLEID	"VideoCallID"
#define IVUG_BUNDLE_VALUE_SETAS_UG_CROP			"Crop"
#define IVUG_BUNDLE_VALUE_SETAS_UG_WALLPAPER_CROP	"Wallpaper Crop"

	int len = strlen(val);

	if (strncmp(val, IVUG_BUNDLE_VALUE_SETAS_UG_WALLPAPER, len) == 0 )
	{
		return IVUG_SET_AS_UG_TYPE_WALLPAPER;
	}
	else if (strncmp(val, IVUG_BUNDLE_VALUE_SETAS_UG_WALLPAPER_CROP, len) == 0 )
	{
		return IVUG_SET_AS_UG_TYPE_WALLPAPER_CROP;
	}
	else if (strncmp(val, IVUG_BUNDLE_VALUE_SETAS_UG_LOCKSCREEN, len) == 0 )
	{
		return IVUG_SET_AS_UG_TYPE_LOCKSCREEN;
	}
	else if (strncmp(val, IVUG_BUNDLE_VALUE_SETAS_UG_WALLPAPER_N_LOCKSCREEN, len) == 0 )
	{
		return IVUG_SET_AS_UG_TYPE_WALLPAPER_N_LOCKSCREEN;
	}
	else if (strncmp(val, IVUG_BUNDLE_VALUE_SETAS_UG_CALLERID, len) == 0)
	{
		return IVUG_SET_AS_UG_TYPE_CALLER_ID;
	}
	else if (strncmp(val, IVUG_BUNDLE_VALUE_SETAS_UG_VIDEO_CALLEID, len) == 0)
	{
		return IVUG_SET_AS_UG_TYPE_VIDEO_CALL_ID;
	}
	else if (strncmp(val, IVUG_BUNDLE_VALUE_SETAS_UG_CROP, len) == 0)
	{
		return IVUG_SET_AS_UG_TYPE_CROP;
	}

	MSG_IVUG_ERROR("Invalid setas ug type : %s", val);
	return IVUG_SET_AS_UG_TYPE_INVALID;
}

static ivug_media_type
_get_media_type(const char* val)
{
	IV_ASSERT(val != NULL);
#define IVUG_BUNDLE_VALUE_MEDIA_TYPE_ALL	"All"
#define IVUG_BUNDLE_VALUE_MEDIA_TYPE_IMAGE	"Image"
#define IVUG_BUNDLE_VALUE_MEDIA_TYPE_VIDEO	"Video"

	int len = strlen(val);

	if (strncmp(val, IVUG_BUNDLE_VALUE_MEDIA_TYPE_IMAGE, len) == 0 )
	{
		return IVUG_MEDIA_TYPE_IMAGE;
	}
	else if (strncmp(val, IVUG_BUNDLE_VALUE_MEDIA_TYPE_VIDEO, len) == 0)
	{
		return IVUG_MEDIA_TYPE_VIDEO;
	}
	else if(strncmp(val, IVUG_BUNDLE_VALUE_MEDIA_TYPE_ALL, len) == 0)
	{
		return IVUG_MEDIA_TYPE_ALL;
	}

	MSG_IVUG_ERROR("Invalid media type : %s", val);
	return IVUG_MEDIA_TYPE_MAX;
}

static bool _data_print(service_h service, const char *key, void *user_data)
{
	char *value;

	char **value_array;
	int array_len = 1;
	int i;
	bool array = false;

	service_is_extra_data_array(service, key, &array);
	if(array == false)
	{
		service_get_extra_data(service, key, &value);
		MSG_IVUG_HIGH("  %s : %s", key, value);
		free(value);
	}
	else
	{
		service_get_extra_data_array(service, key, &value_array, &array_len);
		MSG_IVUG_HIGH("  %s :", key);
		for(i=0; i<array_len; i++)
		{
			MSG_IVUG_HIGH(" %s", value_array[i]);
		}
		for(i=0; i<array_len; i++)
		{
			free(value_array[i]);
		}
		free(value_array);
	}

	return true;
}

static void _print_service_data(service_h service)
{
	int ret = service_foreach_extra_data(service, _data_print, NULL);

	if(SERVICE_ERROR_NONE != ret)
	{
		MSG_IVUG_ERROR("service_foreach_extra_data ERROR");
	}
}



//parsing bundle
ivug_parameter*
ivug_param_create_from_bundle(service_h service)
{
	if (service == NULL)
	{
		MSG_IVUG_WARN("bundle value is NULL");
		return NULL;
	}

	//print key and value.
	MSG_IVUG_HIGH("**********************************");
	_print_service_data(service);
	MSG_IVUG_HIGH("**********************************");

	//parsing param
	ivug_parameter* data = (ivug_parameter*)calloc(1, sizeof(ivug_parameter));

	if (data == NULL)
	{
		MSG_IMAGEVIEW_FATAL("Cannot allocate memory");
		return NULL;
	}

// appsvc
	char* standalone = NULL;

	service_get_extra_data (service, IVUG_BUNDLE_KEY_STANDALONE, &standalone);
	if (standalone != NULL)
	{
		data->bStandalone = true;
		_ivug_free(&standalone);
	}
	else
	{
		data->bStandalone = false;
	}

	//parse path
	char* szFilePath = NULL;
	{
		service_get_extra_data (service, IVUG_BUNDLE_KEY_PATH, &szFilePath);
		if(szFilePath == NULL)
		{
			MSG_IVUG_ERROR("File path is NULL");
			service_get_uri(service, &szFilePath);	// app gadget
			if(szFilePath)
			{
				MSG_IMAGEVIEW_ERROR("App gadget launched %s", szFilePath);
				service_add_extra_data (service, "View Mode", "SINGLE");
			}
		}
		if ( szFilePath != NULL )
		{
			if(strncmp(IVUG_FILE_PREFIX, szFilePath, strlen(IVUG_FILE_PREFIX)) == 0)
			{
				data->filepath = strdup(szFilePath + strlen(IVUG_FILE_PREFIX));
			}
			else
			{
				data->filepath = strdup(szFilePath);
			}
			MSG_IVUG_MED("Current File = %s", data->filepath);
			_ivug_free(&szFilePath);
		}
		else
		{
			ivug_param_delete(data);
			return NULL;
		}
	}

// View By
	char* szViewBy = NULL;

	service_get_extra_data (service, IVUG_BUNDLE_KEY_VIEW_BY, &szViewBy);
	if (szViewBy != NULL)
	{
		MSG_IVUG_MED("View By = %s", szViewBy);

		data->view_by = _get_view_by(szViewBy);

		if (data->view_by == IVUG_VIEW_BY_TAG)
		{
			char* szTagName = NULL;

			service_get_extra_data (service, IVUG_BUNDLE_KEY_TAG_NAME, &szTagName);
			if ( szTagName != NULL )
			{
				tag_handle t_handle = ivug_db_get_tag_handle(szTagName);
				if(t_handle == NULL)
				{
					MSG_IVUG_FATAL("View by Tag. but tag handle is NULL" );
					ivug_param_delete(data);
					return NULL;
				}

				data->tag_id = ivug_db_get_tag_id(t_handle);

				ivug_db_destroy_tag(t_handle);

				MSG_IVUG_HIGH("Tag name=%s", szTagName);
				MSG_IVUG_HIGH("Tag id=%d", data->tag_id);
				_ivug_free(&szTagName);
			}
			else
			{
				MSG_IVUG_FATAL("View by Tag. but Tagname is NULL" );

				_ivug_free(&szViewBy);
				ivug_param_delete(data);
				return NULL;
			}
		}
		_ivug_free(&szViewBy);
	}
	else
	{
		MSG_IVUG_ERROR("View By is NULL");
		data->view_by = IVUG_VIEW_BY_INVAILD;
	}

	media_handle file_handle = ivug_db_get_file_handle(data->filepath);
	if(file_handle == NULL)
	{
		MSG_IVUG_ERROR("Current File = %s is not in DB", data->filepath);
		data->view_by = IVUG_VIEW_BY_FILE;
	}

//parse image index at album
	char* val = NULL;

	service_get_extra_data (service, IVUG_BUNDLE_KEY_INDEX, &val);
	if(val)
	{
		data->start_index = ivug_atoi(val);
		MSG_IVUG_MED("Slide Index = %d", data->start_index);
		_ivug_free(&val);
	}
	else
	{
		if(data->view_by == IVUG_VIEW_BY_FOLDER)
		{
			MSG_IVUG_WARN("IVUG_VIEW_BY_FOLDER but index was not set");
			data->start_index = IVUG_INVALID_INDEX;
		}
		else
		{
		data->start_index = IVUG_DEFAULT_INDEX;
		}
		MSG_IVUG_WARN("Slide Index is not set. Set as default : %d", data->start_index);
	}

//parse mode
	char* szMode = NULL;

	service_get_extra_data (service, IVUG_BUNDLE_KEY_VIEW_MODE, &szMode);
	if (szMode != NULL)
	{
		data->mode = _get_view_mode(data, szMode);
		MSG_IVUG_MED("View mode is %s(%d)", szMode, data->mode);
		_ivug_free(&szMode);
	}
	else
	{
		data->mode = IVUG_DEFAULT_MODE;
		MSG_IVUG_HIGH("View mode is NULL. Set Default(%d)", data->mode);
	}

	//parse album id -album id is cluster id of media service
	service_get_extra_data (service, IVUG_BUNDLE_KEY_ALBUM_IDX, &val);
	if(val && data->view_by != IVUG_VIEW_BY_ALL)
	{
		data->album_id = uuid_getuuid(val);
		MSG_IVUG_MED("album_uuid is %s", uuid_getchar(data->album_id));
	}
	else
	{
		data->album_id = INVALID_UUID;
		MSG_IVUG_WARN("Album index is NULL");
	}
	if(val)
		_ivug_free(&val);

	if (data->view_by == IVUG_VIEW_BY_FOLDER && data->album_id == INVALID_UUID)
	{
/*
		example:
		**********************************
		  Bundle Count = 2
		   View Mode:CAMERA
		   Path:/opt/usr/media/Camera shots/IMAGE0021.jpg
		**********************************
*/
		// Get album id from file path.
		char *dir = ivug_get_directory(data->filepath);
		media_handle m_handle = ivug_db_get_folder_handle(dir);
		free(dir);

		if(data->album_id == INVALID_UUID)
		{
			if(m_handle == NULL)
			{
				MSG_IVUG_ERROR("View by Folder. but media handle is NULL" );
				//ivug_param_delete(data);
				//return NULL;
				data->view_by = IVUG_VIEW_BY_DIRECTORY;	// check manually
			}
			else
			{
				data->album_id = ivug_db_get_folder_id(m_handle);

				ivug_db_destroy_folder_handle(m_handle);

				MSG_IVUG_HIGH("Get Album ID(%s) from file %s", uuid_getchar(data->album_id), data->filepath);
			}
		}
	}

	service_get_extra_data (service, IVUG_BUNDLE_KEY_MEDIA_TYPE, &val);
	if(val != NULL)
	{
		data->media_type = _get_media_type(val);
		MSG_IVUG_MED("Media Type=%s(%d)", val, data->media_type);
		_ivug_free(&val);
	}
	else
	{
		MSG_IVUG_WARN("Media type is not specified.");
	}

	long long int m = 0;
	long int e = 0;

	service_get_extra_data (service, IVUG_BUNDLE_KEY_MAX_LONGITUDE, &val);
	if(val != NULL)
	{
		eina_convert_atod(val, strlen(val), &m, &e);
		data->max_longitude = ldexp((double)m, e);
		MSG_IVUG_MED("Max Longitude =%f", data->max_longitude);
		_ivug_free(&val);
	}
	else
	{
		MSG_IVUG_MED("Max Longitude is not specified.");
	}

	service_get_extra_data (service, IVUG_BUNDLE_KEY_MIN_LONGITUDE, &val);
	if(val != NULL)
	{
		eina_convert_atod(val, strlen(val), &m, &e);
		data->min_longitude = ldexp((double)m, e);
		MSG_IVUG_MED("Min Longitude =%f", data->min_longitude);
		_ivug_free(&val);
	}
	else
	{
		MSG_IVUG_MED("Min Longitude is not specified.");
	}

	service_get_extra_data (service, IVUG_BUNDLE_KEY_MAX_LATITUDE, &val);
	if(val != NULL)
	{
		eina_convert_atod(val, strlen(val), &m, &e);
		data->max_latitude = ldexp((double)m, e);
		MSG_IVUG_MED("Max Latitude =%f", data->max_latitude);
		_ivug_free(&val);
	}
	else
	{
		MSG_IVUG_MED("Max Latitude is not specified.");
	}

	service_get_extra_data (service, IVUG_BUNDLE_KEY_MIN_LATITUDE, &val);
	if(val != NULL)
	{
		eina_convert_atod(val, strlen(val), &m, &e);
		data->min_latitude = ldexp((double)m, e);
		MSG_IVUG_MED("Min Latitude =%f", data->min_latitude);
		_ivug_free(&val);
	}
	else
	{
		MSG_IVUG_MED("Min Latitude is not specified.");
	}

	service_get_extra_data (service, IVUG_BUNDLE_KEY_SETAS_TYPE, &val);
	if (val)
	{
		MSG_IVUG_MED("SetAs UG Type=%s", val);
		data->setas_type = _get_setas_type(val);

		char* resolution = NULL;

		service_get_extra_data (service, IVUG_BUNDLE_KEY_RESOLUTION, &resolution);
		if(resolution)
		{
			sscanf(resolution, "%5dx%5d", &(data->width), &(data->height));
			MSG_IVUG_MED("Rectangle width = %d, height = %d", data->width, data->height);
			if(data->width == 0 || data->height == 0)
			{
				MSG_IVUG_ERROR("Resolution is invalid");
				_ivug_free(&resolution);
				_ivug_free(&val);
				ivug_param_delete(data);
				return NULL;
			}
		}
		else
		{
			data->width = 100;
			data->height = 100;
			MSG_IVUG_HIGH("Rectangle ratio is not set. Set as default : %dx%d", data->width, data->height);
		}

		data->bRatioFix = false;
		char* bRatioFix = NULL;

		service_get_extra_data (service, IVUG_BUNDLE_KEY_FIXED_RATIO, &bRatioFix);
		if(bRatioFix)
		{
			MSG_IVUG_MED("Fixed ratio=%s", bRatioFix);
			if(strcmp(bRatioFix, "TRUE") == 0)
			{
				data->bRatioFix = true;
			}
			_ivug_free(&bRatioFix);
		}

		char* crop_mode = NULL;
		service_get_extra_data (service, IVUG_BUNDLE_KEY_CROP_MODE, &crop_mode);
		if(crop_mode)
		{
			MSG_IVUG_MED("Crop mode=%s", crop_mode);
			if(strcmp(crop_mode, IVUG_BUNDLE_VALUE_CROP_MODE_FIT_TO_SCREEN) == 0)
			{
				int lcd_x = 0;
				int lcd_y = 0;
				int lcd_w = 0;
				int lcd_h = 0;
				int rot = 0;
				evas_object_geometry_get((Evas_Object *)ug_get_window(), &lcd_x, &lcd_y, &lcd_w, &lcd_h);
				rot = gGetRotationDegree();
				if(rot == 90 || rot == 270)
				{
					int temp = lcd_w;
					lcd_w = lcd_h;
					lcd_h = temp;
				}
				data->width = lcd_w;
				data->height = lcd_h;
				data->bRatioFix = true;
			}
			_ivug_free(&crop_mode);
		}

		_ivug_free(&val);
	}

	data->footsteps = false;	/* default */
	service_get_extra_data (service, IVUG_BUNDLE_KEY_FOOTSTEPS, &val);
	if (val)
	{
		MSG_IVUG_MED("Footsteps=%s", val);
		if(strcmp(val, "TRUE") == 0)
		{
			data->footsteps = true;
		}
		_ivug_free(&val);
	}




	char **index_list = NULL;
	int index_len = 0;
	service_get_extra_data_array(service, IVUG_BUNDLE_KEY_SELECTED_INDEX, &index_list, &index_len);
	if(index_list != NULL)
	{
		int i;
		for(i=0; i<index_len; i++)
		{
			data->selected_list = eina_list_append(data->selected_list, (void *)ivug_atoi(index_list[i]));
		}
		for(i=0; i<index_len; i++)
		{
			free(index_list[i]);
		}
		free(index_list);
	}

	return data;
}

void
ivug_param_delete(ivug_parameter* data)
{
	IV_ASSERT(data != NULL);
	if (data->filepath)
	{
		free(data->filepath);
		data->filepath = NULL;
	}


	uuid_free(data->album_id);

	free(data);

	MSG_IVUG_HIGH("Parameter is freed.");

}





Filter_struct *ivug_param_create_filter(const ivug_parameter *param)
{
	IV_ASSERT(param != NULL);

	Filter_struct *filter_str = (Filter_struct *)calloc(1, sizeof(Filter_struct));
	IV_ASSERT(filter_str != NULL);

	media_handle handle = NULL;
	char *dir = NULL;

	if ( param->view_by == IVUG_VIEW_BY_DIRECTORY )
	{
		filter_str->type = FILTER_DIRECTORY;
		filter_str->view_by = param->view_by;

		Direcotry_Filter *filter = (Direcotry_Filter *)calloc(1, sizeof(Direcotry_Filter) );

		IV_ASSERT(filter != NULL);

		filter_str->dir_filter = filter;
		filter_str->dir_filter->basedir = ivug_get_directory(param->filepath);
		filter_str->dir_filter->current = strdup( param->filepath );

		return filter_str;
	}
	else
	{
		filter_str->type = FILTER_DB;
	}

	filter_str->view_by = param->view_by;
	filter_str->media_type = param->media_type;
	filter_str->sort_type = param->sort_type;
	filter_str->index = param->start_index;
	filter_str->selected_list = param->selected_list;
	if(param->filepath)
	{
		filter_str->filepath = strdup(param->filepath);
	}

	if(filter_str->type == FILTER_DB)
	{
		DB_Filter *filter = calloc(1, sizeof(DB_Filter));

		IV_ASSERT(filter != NULL);

		MSG_SDATA_HIGH("param->view_by is %d", param->view_by);
		switch(filter_str->view_by)
		{
		case IVUG_VIEW_BY_PLACES:
			filter->place.max_longitude = param->max_longitude;
			filter->place.min_longitude = param->min_longitude;
			filter->place.max_latitude = param->max_latitude;
			filter->place.min_latitude = param->min_latitude;
			break;

		case IVUG_VIEW_BY_TAG:
			filter->tag_id = param->tag_id;
			break;

		case IVUG_VIEW_BY_FAVORITES:
			filter->album_id = INVALID_UUID;
			break;
		case IVUG_VIEW_BY_FILE:
			filter->file_path = strdup(param->filepath);
			break;
		case IVUG_VIEW_BY_ALL:
			filter->album_id = uuid_assign(param->album_id);
			break;
		case IVUG_VIEW_BY_FOLDER:
			if(param->album_id == NULL)
			{
				dir = ivug_get_directory(param->filepath);
				handle = ivug_db_get_folder_handle(dir);
				filter->album_id = ivug_db_get_folder_id(handle);
				free(dir);
			}
			else
			{
				filter->album_id = uuid_assign(param->album_id);
			}
			break;
		case IVUG_VIEW_BY_DIRECTORY:
			break;
		case IVUG_VIEW_BY_INVAILD:
		default:
			MSG_SDATA_WARN("Invalid ViewBy : %d", param->view_by);
			break;

		}

		filter_str->db_filter = filter;
	}

	return filter_str;

}


