/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "ivug-common.h"

#include <Elementary.h>

#include "ivug-selectioninfo.h"

#define NOTIFY_TIMEOUT	3.0

static void
_ivug_selectioninfo_timeout_cb(void *data, Evas_Object *obj, void *event_info)
{
	ivug_ret_if(obj == NULL);

	evas_object_del(obj);
	obj = NULL;
}

static void _on_resize(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	Evas_Coord x,y,w,h;
	evas_object_geometry_get(obj, &x, &y, &w, &h);

	MSG_IMAGEVIEW_HIGH("Selection resized geomtery XYWH(%d,%d,%d,%d) angle=%d", x, y, w, h, gGetRotationDegree());

	Evas_Object* layout = elm_object_content_get(obj);
	ivug_retm_if(layout == NULL, "Notify content is NULL");

	//Get the selectioninfo text
	const char* text = edje_object_part_text_get(_EDJ(layout), "elm.text");

#ifdef USE_CUSTOM_STYLE
	elm_object_theme_set(layout, gGetSystemTheme());

	int rot_angle = gGetRotationDegree();
	if ( (rot_angle == 0) || (rot_angle == 180) )
	{
		elm_layout_theme_set(layout, "standard", "selectioninfo", "vertical/bottom_86");
	}
	else
	{
		elm_layout_theme_set(layout, "standard", "selectioninfo", "horizontal/bottom_64");
	}
#endif

	if(text)
	{
		//Reset the text, because the text will be missed if no reset
		edje_object_part_text_set(_EDJ(layout), "elm.text", text);
	}

}


Evas_Object*
ivug_selectioninfo_create(Evas_Object* parent, const char* text)
{
	Evas_Object *notify;

	notify = elm_notify_add(parent);
	ivug_retv_if(notify == NULL, NULL);
	elm_notify_align_set(notify, ELM_NOTIFY_ALIGN_FILL, 1.0);
	evas_object_size_hint_weight_set(notify, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(notify, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_smart_callback_add(notify, "timeout", _ivug_selectioninfo_timeout_cb, NULL);
	elm_notify_timeout_set(notify, NOTIFY_TIMEOUT);

	int rot_angle = gGetRotationDegree();

	Evas_Object* layout = elm_layout_add(parent);

#ifdef USE_CUSTOM_STYLE
	elm_object_theme_set(layout, gGetSystemTheme());

	if ( (rot_angle == 0) || (rot_angle == 180) )
	{
		elm_layout_theme_set(layout, "standard", "selectioninfo", "vertical/bottom_86");
	}
	else
	{
		elm_layout_theme_set(layout, "standard", "selectioninfo", "horizontal/bottom_64");
	}
#endif

	evas_object_size_hint_weight_set(layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(layout, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_object_content_set(notify, layout);
	edje_object_part_text_set(_EDJ(layout), "elm.text", text);

	evas_object_event_callback_add(notify, EVAS_CALLBACK_RESIZE, _on_resize, NULL);

	evas_object_show(notify);

	return notify;
}

Evas_Object*
ivug_notify_create(Evas_Object* parent, const char* text)
{
	Evas_Object *selection_info;

	selection_info = elm_layout_add(parent);
	ivug_retv_if(selection_info == NULL, NULL);
	evas_object_size_hint_weight_set(selection_info, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(selection_info, EVAS_HINT_FILL, EVAS_HINT_FILL);

#ifdef USE_CUSTOM_STYLE
	//elm_object_theme_set(selection_info, gGetSystemTheme());
#endif
	elm_layout_theme_set(selection_info, "standard", "selectioninfo", "default");

	elm_object_part_text_set(selection_info, "elm.text", text);

	evas_object_show(selection_info);

	return selection_info;
}

static void _on_timeout(void *data, Evas_Object *obj, void *event_info)
{
	evas_object_del(obj);	// remove notify
}

void
ivug_notify_timeout_create(Evas_Object* parent, const char* text, double align,
	Evas_Smart_Cb timeout_cb, void *data)
{
	Evas_Object *notify = elm_notify_add(parent);
	ivug_ret_if(notify == NULL);
	elm_notify_align_set(notify, ELM_NOTIFY_ALIGN_FILL, align);
	elm_notify_timeout_set(notify, 2.0);

	Evas_Object *selection_info = elm_layout_add(notify);
	ivug_ret_if(selection_info == NULL);
	evas_object_size_hint_weight_set(selection_info, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(selection_info, EVAS_HINT_FILL, EVAS_HINT_FILL);

#ifdef USE_CUSTOM_STYLE
	//elm_object_theme_set(selection_info, gGetSystemTheme());
#endif
	elm_layout_theme_set(selection_info, "standard", "selectioninfo", "default");

	elm_object_part_text_set(selection_info, "elm.text", text);

	elm_object_content_set(notify, selection_info);

	if(timeout_cb)
		evas_object_smart_callback_add(notify, "timeout", timeout_cb, data);
	else
		evas_object_smart_callback_add(notify, "timeout", _on_timeout, NULL);

	evas_object_show(notify);
}

Evas_Object*
ivug_notify_static_create(Evas_Object* parent, const char* text, double align)
{
	Evas_Object *notify = elm_notify_add(parent);
	ivug_retv_if(notify == NULL, NULL);
	elm_notify_align_set(notify, ELM_NOTIFY_ALIGN_FILL, align);

	Evas_Object *selection_info = elm_layout_add(notify);
	ivug_retv_if(selection_info == NULL, NULL);
	evas_object_size_hint_weight_set(selection_info, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(selection_info, EVAS_HINT_FILL, EVAS_HINT_FILL);

#ifdef USE_CUSTOM_STYLE
	//elm_object_theme_set(selection_info, gGetSystemTheme());
#endif
	elm_layout_theme_set(selection_info, "standard", "selectioninfo", "default");

	elm_object_part_text_set(selection_info, "elm.text", text);

	elm_object_content_set(notify, selection_info);

	evas_object_show(notify);

	return notify;
}


