/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "ivug-common.h"

#include "EFLUtil.h"

#include "ivug-slider-item.h"

#include "ivug-image.h"

#include "ivug-drm-common.h"

#include "ivug-config.h"

#define DEFAULT_THUMBNAIL_PATH			IMAGE_PATH"/T01_Nocontents_broken.png"
#define DRM_NO_RIGHT_IMAGE_PATH			IMAGE_PATH"/image_drm_no_rights.jpg"

#define IVUG_SLIDER_EDJ 				EDJ_PATH"/ivug-slider.edj"

#define USE_VIDEO_THUMBNAIL


/*
	Determine whether LIL(Large image layer) loads or not.

	if define this feature, slider does not try to load image. only thumbnail is shown.
*/
#undef DISABLE_LIL_LOADING

/*
	Use elm_icon istead of elm_image. elm_icon preserve image ratio. and seems more faster.
*/

static bool
_show_thumbnail( Slide_Item* si, const char *filepath)
{
	IV_ASSERT(si != NULL);

	if (si->thumbnail == NULL)
	{
		MSG_SITEM_FATAL("slide item or thumbnail object is NULL");
		return false;
	}

	if ( filepath == NULL )
	{
		MSG_SITEM_ERROR("thumbnail path is not exist");
		return false;
	}

	MSG_SITEM_MED("Set thumbnail : %s",  ivug_get_filename(filepath));

	if (ivug_is_file_exist(filepath) == false )
	{
		MSG_SITEM_ERROR("Cannot find thumbnail : %s", filepath);
		filepath = DEFAULT_THUMBNAIL_PATH;
	}

	if (elm_image_file_set(si->thumbnail, filepath, NULL) == EINA_FALSE)
	{
		MSG_SITEM_ERROR("Cannot load thumbnail : %s", filepath);
		return false;
	}

	int iw, ih;

	elm_image_object_size_get(si->thumbnail , &iw, &ih);

	evas_object_repeat_events_set(si->thumbnail, EINA_FALSE);
	edje_object_signal_emit(_EDJ(si->layout), "elm,state,show_thumbnail", "slider");

	MSG_SITEM_HIGH("Set thumbnail(%dx%d). %s", iw, ih, ivug_get_filename(filepath));

#ifdef _DEBUG
	int bx, by, bw, bh;
	evas_object_geometry_get(si->layout, &bx, &by, &bw, &bh);

	MSG_SITEM_MED("EDJE Geomtetry(%d,%d,%d,%d)", bx, by, bw, bh);
#endif

	return true;
}

static bool
_hide_thumbnail(Slide_Item* si)
{
	IV_ASSERT(si != NULL);

	evas_object_repeat_events_set(si->thumbnail, EINA_TRUE);

	edje_object_signal_emit(_EDJ(si->layout), "elm,state,hide_thumbnail", "slider");

	return true;
}

static void _show_progressbar(Slide_Item *si)
{
	IV_ASSERT(si != NULL);

	if ( si->progress == NULL )
	{
		//set progress
		PERF_CHECK_BEGIN(LVL6, "set progress");

		PERF_CHECK_BEGIN(LVL7, "elm_progressbar_add");
		si->progress = elm_progressbar_add(si->layout);
		PERF_CHECK_END(LVL7, "elm_progressbar_add");

		IV_ASSERT(si->progress != NULL);

		PERF_CHECK_BEGIN(LVL7, "set style");
		elm_object_style_set(si->progress, "list_process");
		PERF_CHECK_END(LVL7, "set style");

		PERF_CHECK_BEGIN(LVL7, "set pulse");
		elm_progressbar_pulse(si->progress, EINA_FALSE);
		PERF_CHECK_END(LVL7, "set pulse");

		PERF_CHECK_BEGIN(LVL7, "swallow progress");
		elm_object_part_content_set(si->layout, "slider.progress", si->progress);
		PERF_CHECK_END(LVL7, "swallow progress");

		PERF_CHECK_END(LVL6, "set progress");
	}

	elm_progressbar_pulse(si->progress, EINA_TRUE); //start pulse.

	edje_object_signal_emit(_EDJ(si->layout), "elm,state,show,progress", "slider");
}

static void _hide_progressbar(Slide_Item *si)
{
	IV_ASSERT(si != NULL);

	if ( si->progress )
	{
		elm_progressbar_pulse(si->progress, EINA_FALSE); //stop pulse.
	}
	edje_object_signal_emit(_EDJ(si->layout), "elm,state,hide,progress", "slider");
}


static void _show_videoicon(Slide_Item *si)
{
	IV_ASSERT(si != NULL);

	edje_object_signal_emit(_EDJ(si->layout), "elm,state,show_icon", "slider"); // Hide video play icon
}

static void _hide_videoicon(Slide_Item *si)
{
	IV_ASSERT(si != NULL);

	edje_object_signal_emit(_EDJ(si->layout), "elm,state,hide_icon", "slider"); // Hide video play icon

//	edje_object_signal_emit((si->layout), "elm,state,hide_icon", "slider"); // Hide video play icon
}

static sitem_type_t _get_sitem_type(Media_Type mtype)
{
	switch(mtype)
	{
	case SLIDE_TYPE_IMAGE:
		return SITEM_TYPE_IMAGE;

	case SLIDE_TYPE_VIDEO:
		return SITEM_TYPE_VIDEO;

	case SLIDE_TYPE_NONE:
	case SLIDE_TYPE_UNKNOWN:
	default:
		return SITEM_TYPE_UNKNOWN;
	}

	return SITEM_TYPE_UNKNOWN;
}

static char *_get_file_name(Slide_Item* si)
{
	Media_Data *mdata = ivug_medialist_get_data(si->mitem);

	if ( mdata == NULL )
	{
		return NULL;
	}

	return ivug_get_filename(mdata->filepath);
}


static bool _set_mitem(Slide_Item* si, Media_Item *item)
{
	Media_Data *mdata = ivug_medialist_get_data(item);
	IV_ASSERT(mdata != NULL);

	si->mitem = item;

	si->type = _get_sitem_type(mdata->slide_type);

	if ( si->type == SITEM_TYPE_IMAGE )
	{
		_hide_videoicon(si);
		elm_object_part_content_set(si->layout, "slider.contents", si->photocam);
	}
	else if ( si->type == SITEM_TYPE_VIDEO )
	{
		 _show_thumbnail(si, mdata->thumbnail_path);
		elm_image_file_set(si->vIcon, EDJ_PATH"/ivug-icons.edj", "btn.video.play");
		_show_videoicon(si);
	}
	else
	{
		MSG_SITEM_WARN("Unknown type : %s", ivug_get_filename(mdata->filepath));
		_hide_videoicon(si);
	}

	switch(mdata->slide_type)
	{
		case SLIDE_TYPE_IMAGE:
			MSG_SITEM_HIGH("Set image. %s", ivug_get_filename(mdata->filepath));

			if(mdata->bIsDRMContent == EINA_FALSE)
			{
				si->state = SLIDE_STATE_READY;
			}
			else
			{
				si->state = SLIDE_STATE_DRM_CHECKING;
			}
		break;

		case SLIDE_TYPE_VIDEO:
			MSG_SITEM_HIGH("Set video. %s", ivug_get_filename(mdata->filepath));

			si->state = SLIDE_STATE_READY; 	// Video dont need to load photocam
			break;

		case SLIDE_TYPE_UNKNOWN:
			MSG_SITEM_ERROR("Unknown image. %s", ivug_get_filename(mdata->filepath));

			si->state = SLIDE_STATE_ERROR;

			break;

		default:
			MSG_SITEM_ERROR("slide type invaild. Type=%d", mdata->slide_type);
			si->state = SLIDE_STATE_ERROR;
			return true;
			break;

	}

	return true;
}


static bool _unset_mitem(Slide_Item* si)
{
// Unswallow photocam
	elm_object_part_content_unset(si->layout, "slider.contents");

	switch(si->type)
	{
	case SITEM_TYPE_IMAGE:
		evas_object_hide(si->photocam);
		ivug_image_unset(si->photocam);	//init photocam.
		break;
	case SITEM_TYPE_VIDEO:
		break;
	default:
		break;
	}

	si->state = SLIDE_STATE_NONE;

// Resetting GUI
	_hide_progressbar(si);
	_hide_thumbnail(si);
	_hide_videoicon(si);

	si->type = SITEM_TYPE_UNKNOWN;
	si->mitem = NULL;

	MSG_SITEM_HIGH("Slider Item is cleared.");

	return true;
}



inline
void _call_user_callback(Slide_Item* si)
{
	if ( si->cb )
	{
		(si->cb)(si->state, si, si->client_data);
	}
}


static void
_photocam_loaded_cb(void *data, Evas_Object *obj, void *event_info)
{
	IV_ASSERT(data != NULL);

	Slide_Item* si = static_cast<Slide_Item*>(data);

//hide progress.
	_hide_progressbar(si);

	if ( si->mitem == NULL)
	{
		MSG_SITEM_ERROR("Debug Me! Data item is NULL.");
		return;
	}

	Media_Data* mdata = ivug_medialist_get_data(si->mitem);
	if(mdata == NULL)
	{
		MSG_SITEM_ERROR("ivug_medialist_get_data failed.");
		si->state = SLIDE_STATE_ERROR;
		return;
	}

	if ( si->state != SLIDE_STATE_LOADING )
	{
		// can be STATE_ERROR, STATE_NO_RIGHT
		MSG_SITEM_ERROR("Invalid slide state. %s", ivug_get_filename(mdata->filepath));
		MSG_SITEM_ERROR("state=%d %s (%d,%d,%d,%d)", si->state, mdata->filepath, si->x, si->y, si->w, si->h);
	}

#if 0
	typedef enum _Evas_Load_Error
	{
	   EVAS_LOAD_ERROR_NONE = 0, /**< No error on load */
	   EVAS_LOAD_ERROR_GENERIC = 1, /**< A non-specific error occurred */
	   EVAS_LOAD_ERROR_DOES_NOT_EXIST = 2, /**< File (or file path) does not exist */
	   EVAS_LOAD_ERROR_PERMISSION_DENIED = 3, /**< Permission deinied to an existing file (or path) */
	   EVAS_LOAD_ERROR_RESOURCE_ALLOCATION_FAILED = 4, /**< Allocation of resources failure prevented load */
	   EVAS_LOAD_ERROR_CORRUPT_FILE = 5, /**< File corrupt (but was detected as a known format) */
	   EVAS_LOAD_ERROR_UNKNOWN_FORMAT = 6 /**< File is not a known format */
	} Evas_Load_Error; /**< Load error you can get from loading of files - see evas_load_error_str() too */
#endif

	Evas_Load_Error error = static_cast<Evas_Load_Error>(reinterpret_cast<int>(event_info));
//	Evas_Load_Error error = evas_object_image_load_error_get(ivug_image_internal_image_get(si->photocam));

	if ( error != EVAS_LOAD_ERROR_NONE )
	{
		MSG_SITEM_ERROR("Image loading failed. Error=%d File=%s", error, mdata->filepath);
		si->state = SLIDE_STATE_ERROR;

		evas_object_hide(si->photocam);

		if ( error == EVAS_LOAD_ERROR_PERMISSION_DENIED)
		{
			_show_thumbnail(si, DRM_NO_RIGHT_IMAGE_PATH);
		}
		else
		{
			_show_thumbnail(si, DEFAULT_THUMBNAIL_PATH);
		}

		_call_user_callback(si);

		return ;
	}

	si->state = SLIDE_STATE_LOADED;

	int w, h;

	ivug_image_image_size_get(si->photocam, &w, &h);

	if (mdata->slide_type == SLIDE_TYPE_VIDEO) 	//if slide type is video,
	{
		MSG_SITEM_FATAL("slide type is video. thumbnail image ");
		return;
	}

	evas_object_show(obj);	//photocam show.

	_call_user_callback(si);

	MSG_SITEM_HIGH("Photocam Pre-loaded. File=%s", ivug_get_filename(mdata->filepath));
}

static void _on_event_mouse_down(void *data, Evas *evas, Evas_Object *obj, void *event_info)
{
	MSG_SITEM_HIGH("Event layer clicked : %s Layer=%d", evas_object_name_get(obj), evas_object_layer_get(obj));
}


Slide_Item*
ivug_slider_item_add(Evas_Object *slider)
{
	Slide_Item* si = NULL;

	PERF_CHECK_BEGIN(LVL6, "init variable");

	//create item
	si = (Slide_Item*)calloc(1, sizeof (Slide_Item));

	if ( si == NULL )
	{
		MSG_SITEM_FATAL("Cannot allocated memory : %d.", sizeof (Slide_Item));
		return NULL;
	}

	si->state = SLIDE_STATE_NONE;		//set slide state
	si->type = SITEM_TYPE_UNKNOWN;

	PERF_CHECK_END(LVL6, "init variable");

//Create the slider item layout
	PERF_CHECK_BEGIN(LVL6, "create slider item layout");

	Evas_Object *layout = EFL::create_layout(slider, IVUG_SLIDER_EDJ, "slider");
	if(layout == NULL)
	{
		MSG_SITEM_ERROR("Cannot load file : %s", IVUG_SLIDER_EDJ);
		free(si);
		return NULL;
	}

	evas_object_name_set(layout, "Slider item edj");
	evas_object_event_callback_add(layout, EVAS_CALLBACK_MOUSE_DOWN, _on_event_mouse_down, NULL);

	si->layout = layout;

	PERF_CHECK_END(LVL6, "create slider item layout");

//set thumbnail.
	PERF_CHECK_BEGIN(LVL6, "set thumbnail");

	si->thumbnail = elm_icon_add(slider);
	evas_object_name_set(si->thumbnail, "Thumbnail");
	elm_image_resizable_set(si->thumbnail, EINA_TRUE, EINA_TRUE);
	elm_image_preload_disabled_set(si->thumbnail, EINA_FALSE);

	evas_object_event_callback_add(si->thumbnail, EVAS_CALLBACK_MOUSE_DOWN, _on_event_mouse_down, NULL);

	elm_object_part_content_set(si->layout, "slider.thumbnail", si->thumbnail);

	PERF_CHECK_END(LVL6, "set thumbnail");

//Add the photocam
	PERF_CHECK_BEGIN(LVL6, "ivug_image_create");

	si->photocam =	ivug_image_create(si->layout);
	evas_object_name_set(si->photocam, "photocam");
	evas_object_smart_callback_add(si->photocam, "loaded", _photocam_loaded_cb, si);

	PERF_CHECK_END(LVL6, "ivug_image_create");

	PERF_CHECK_BEGIN(LVL6, "video icon");

// z-order be decided along added order
	si->vIcon = elm_icon_add(slider);
	elm_image_file_set(si->vIcon, EDJ_PATH"/ivug-icons.edj", "btn.video.play");

	evas_object_name_set(si->vIcon, "video icon");
	evas_object_event_callback_add(si->vIcon, EVAS_CALLBACK_MOUSE_DOWN, _on_event_mouse_down, NULL);

	elm_object_part_content_set(si->layout, "slider.icon", si->vIcon);
	PERF_CHECK_END(LVL6, "video icon");

	si->bMenuVisible = true;		// Default visibility is true

	evas_object_smart_member_add(si->layout, slider);

	int bx, by, bw, bh;
	evas_object_geometry_get(si->layout, &bx, &by, &bw, &bh);

	MSG_SITEM_HIGH("Creating sitem(0x%08x): (%d,%d,%d,%d)", si, bx, by, bw, bh);

	return si;
}


void
ivug_slider_item_resize(Slide_Item* si, int w, int h)
{
	IV_ASSERT(si != NULL );

	if ( si->w == w && si->h == h ) return;
	MSG_SITEM_HIGH("SLIDER Resize to XYWH(%d,%d,%d,%d)", si->x, si->y, w, h );

// w,h is not Image's scale, but Object's scale
	si->w = w;
	si->h = h;

//	MSG_SITEM_HIGH("resize layout: %d, %d", si->w, si->h);
	evas_object_resize(si->layout, si->w, si->h);

}


void ivug_slider_item_move(Slide_Item* si, int x, int y)
{
	IV_ASSERT(si != NULL );

	MSG_SITEM_MED("SLIDER Move to XYWH(%d,%d,%d,%d)", x, y, si->w, si->h );

	if ( si->x == x && si->y == y) return;

	//photocam
	si->x = x;
	si->y = y;

	if ( si->w == 0 || si->h == 0) return;

	evas_object_move(si->layout, x, y);

// Center align
//	evas_object_move();
//	si->vIcon

}

void ivug_slider_item_del(Slide_Item* si)
{
	IV_ASSERT(si != NULL );

	MSG_SITEM_HIGH("Removing Slider Item");

	ivug_slider_item_stop(si);

	si->mitem = NULL;	// for error handling

	if (si->photocam)
	{
		evas_object_del(si->photocam);
		si->photocam = NULL;
	}

	if (si->thumbnail)
	{
		evas_object_del(si->thumbnail);
		si->thumbnail = NULL;
	}

	if (si->progress)
	{
		evas_object_del(si->progress);
		si->progress = NULL;
	}

	if(si->layout)
	{
		evas_object_del(si->layout);
		si->layout = NULL;
	}

	if ( si->vIcon )
	{
		evas_object_del(si->vIcon);
		si->vIcon = NULL;
	}

	free(si);

	si = NULL;

	MSG_SITEM_HIGH("Removed Slider Item");

}

void ivug_slider_item_hide(Slide_Item* si)
{
	IV_ASSERT(si != NULL );

	MSG_SITEM_HIGH("Hide slide item");
	evas_object_hide(si->layout);
}


void ivug_slider_item_show(Slide_Item* si)
{
	IV_ASSERT(si != NULL );

	MSG_SITEM_HIGH("Show slide item");
	evas_object_show(si->layout);
}


void
ivug_slider_item_clip_set(Slide_Item* si, Evas_Object *clipper)
{
	IV_ASSERT(si != NULL );
// no need to check layout is NULL
	int x, y, w, h;

	evas_object_geometry_get(clipper, &x, &y, &w, &h);

	MSG_SITEM_MED("SItem clip SET!! Geometry(%d,%d,%d,%d)", x, y, w, h);

	evas_object_clip_set(si->layout, clipper);
//	evas_object_clip_set(si->photocam, clipper);
}

void
ivug_slider_item_clip_unset(Slide_Item* si)
{
	IV_ASSERT(si != NULL );
// no need to check layout is NULL
	evas_object_clip_unset(si->layout);
//	evas_object_clip_unset(si->photocam);
}

bool
ivug_slider_item_data_set(Slide_Item* si, Media_Item *item)
{
	IV_ASSERT(si != NULL );

	MSG_SITEM_HIGH("Data set. Item(0x%08x)", item);

	if ( item == NULL )
	{
		if ( si->state == SLIDE_STATE_NONE )
		{
			MSG_SITEM_WARN("Slide item is already cleared. just return. Mitem=0x%08x", si->mitem);
			return true;
		}

		MSG_SITEM_WARN("Set data NULL : %s", _get_file_name(si));
		_unset_mitem(si);
	}
	else
	{
		if ( si->state != SLIDE_STATE_NONE )
		{
			MSG_SITEM_WARN("State is %d. Remove old : %s", si->state, _get_file_name(si));
			_unset_mitem(si);
		}

		_set_mitem(si, item);
	}

	return true;
}


#ifdef DISABLE_LIL_LOADING
Eina_Bool _signal_send_idler(void *data)
{
	Slide_Item *si = (Slide_Item *)data;

	_call_user_callback(si);

	return EINA_FALSE;
}
#endif


// Ready to Loading
bool
ivug_slider_item_load(Slide_Item *si)
{
	IV_ASSERT(si != NULL);

	if (si->state != SLIDE_STATE_READY)
	{
		MSG_SITEM_ERROR("Invalid state : %d", si->state);
		return false;
	}

	Media_Data *mdata = ivug_medialist_get_data(si->mitem);
	IV_ASSERT(mdata != NULL);

	if(mdata->filepath == NULL)
	{
		MSG_SITEM_ERROR("file path is NULL");
		return false;
	}

	MSG_SITEM_HIGH("Load file. %s", ivug_get_filename(mdata->filepath));

	char path[IVUG_MAX_FILE_PATH_LEN] = {0,};

	{
		strncpy(path, mdata->filepath, sizeof(path));
	}


#ifdef DISABLE_LIL_LOADING
	// For debugging
	si->state = SLIDE_STATE_LOADED;

	ecore_idler_add(_signal_send_idler, si);

	return true;
#endif

	switch(mdata->slide_type)
	{
		case SLIDE_TYPE_IMAGE:
		{
			si->state = SLIDE_STATE_LOADING;

			if (ivug_image_file_set(si->photocam, path, NULL) != EVAS_LOAD_ERROR_NONE)
			{
				MSG_SITEM_ERROR("Failed photocam to set file file=%s", path);
				si->state = SLIDE_STATE_ERROR;

				_show_thumbnail(si, DEFAULT_THUMBNAIL_PATH);

				_call_user_callback(si);
				return false;
			}

			return true;

		}
		break;
	case SLIDE_TYPE_VIDEO:
		{
			si->state = SLIDE_STATE_LOADED;
		}

		break;

	default:
		MSG_SITEM_ERROR("slide type invaild. Type=%d", mdata->slide_type);
		return false;
		break;
	}

	return true;
}

void ivug_slider_item_play(Slide_Item* si)
{
	IV_ASSERT(si != NULL );

	Media_Data *mdata = ivug_medialist_get_data(si->mitem);
	if(mdata == NULL)
	{
		MSG_SITEM_ERROR("current data is NULL");
		return;
	}

	MSG_SITEM_HIGH("Start sitem. Type(%d) State(%d) %s", si->type, si->state, mdata->filepath);

	if ( si->state != SLIDE_STATE_LOADED )
	{
		MSG_SITEM_ERROR("Play. but state is %d", si->state);
	}

// Start animation
	if ( si->type == SITEM_TYPE_IMAGE )
	{
		ivug_image_animated_set(si->photocam, EINA_TRUE);
	}
}

void ivug_slider_item_stop(Slide_Item* si)
{
	IV_ASSERT(si != NULL );

	if ( si->state != SLIDE_STATE_LOADED )
	{
		// When very first/last image and invalid
		return;
	}

	Media_Data *mdata = ivug_medialist_get_data(si->mitem);
	MSG_SITEM_HIGH("Stop sitem. %s", mdata->filepath);

	if (si->type == SITEM_TYPE_VIDEO)
	{
		;
	}
	else if ( si->type == SITEM_TYPE_IMAGE )
	{
		ivug_image_animated_set(si->photocam, EINA_FALSE);
	}
}


void ivug_slider_item_pause(Slide_Item* si)
{
	IV_ASSERT(si != NULL );

	if ( si->state != SLIDE_STATE_LOADED )
	{
		// When very first/last image and invalid
		return;
	}

	Media_Data *mdata = ivug_medialist_get_data(si->mitem);
	MSG_SITEM_HIGH("Pause sitem. %s", mdata->filepath);

	if (si->type == SITEM_TYPE_VIDEO)
	{
		// Reset state.
	}
	else if ( si->type == SITEM_TYPE_IMAGE )
	{
		ivug_image_animated_set(si->photocam, EINA_FALSE);
	}
}

void ivug_slider_item_resume(Slide_Item* si)
{
	IV_ASSERT(si != NULL );

	if ( si->state != SLIDE_STATE_LOADED )
	{
		// When very first/last image and invalid
		return;
	}

	Media_Data *mdata = ivug_medialist_get_data(si->mitem);
	MSG_SITEM_HIGH("Resume sitem. %s", mdata->filepath);

	if (si->type == SITEM_TYPE_IMAGE)
	{
		ivug_image_animated_set(si->photocam, EINA_TRUE);
	}
	else if (si->type == SITEM_TYPE_VIDEO)
	{
	}
}

bool
ivug_slider_item_set_callback(Slide_Item* si, OnSliderStateChanged cb, void *data)
{
	IV_ASSERT(si != NULL );

	si->cb = cb;
	si->client_data = data;

	return true;
}


const Media_Item *
ivug_slider_item_data_get(Slide_Item *si)
{
	IV_ASSERT(si != NULL);

	return si->mitem;
}

// Photocam
bool
ivug_slider_item_region_size_get(Slide_Item* si, int *x, int *y, int *w, int *h)
{
	IV_ASSERT(si != NULL );

	if ( si->mitem == NULL)
	{
		MSG_SITEM_ERROR("Debug Me! Data item is NULL.");
		return false;
	}

	Media_Data *mdata = ivug_medialist_get_data(si->mitem);

	if ( mdata == NULL )
	{
		MSG_SITEM_ERROR("slide type is NULL. Item=0x%08x", si);
		return false;
	}

	switch(si->type)
	{
		case SITEM_TYPE_IMAGE:
		{
			ivug_image_region_get(si->photocam, x, y, w, h);
		}
		break;

		case SITEM_TYPE_VIDEO:
		{
			{
				return false;
			}
		}
		break;

		case SITEM_TYPE_UNKNOWN:
			return false;
		break;

		default:
			MSG_SITEM_ERROR("Unknwon type: %d", si->type);
			return false;
		break;
	}

	return true;
}



void
ivug_slider_item_hold_set(Slide_Item* si, Eina_Bool hold)
{
	IV_ASSERT(si != NULL );

	switch(si->type)
	{
		case SITEM_TYPE_IMAGE:
		{
			ivug_image_hold_set(si->photocam, hold);
		}
		break;

		case SITEM_TYPE_VIDEO:
		{

		}
		break;

		case SITEM_TYPE_UNKNOWN:
		default:
			MSG_SITEM_ERROR("Unknwon type: %d", si->type);
		break;
	}


}


void
ivug_slider_item_zoom_reset(Slide_Item* si, Eina_Bool bAnim, int center_x, int center_y)
{
	IV_ASSERT(si != NULL );

	switch(si->type)
	{
		case SITEM_TYPE_IMAGE:
		{
			Evas_Point pt;

			pt.x = center_x;
			pt.y = center_y;

			ivug_image_zoom_reset(si->photocam, &pt);
		}
		break;

		case SITEM_TYPE_VIDEO:
		{

		}
		break;

		case SITEM_TYPE_UNKNOWN:
		break;

		default:
			MSG_SITEM_ERROR("Unknwon type: %d", si->type);
			break;

	}

}


void
ivug_slider_item_zoom_set(Slide_Item* si, double zoom, int center_x, int center_y)
{
	IV_ASSERT(si != NULL );

	switch(si->type)
	{
		case SITEM_TYPE_IMAGE:
		{
			Evas_Point Center;

			Center.x = center_x;
			Center.y = center_y;
			ivug_image_zoom_set(si->photocam, zoom, &Center);

		}
		break;

		case SITEM_TYPE_VIDEO:
		break;

		case SITEM_TYPE_UNKNOWN:
		break;

		default:
			MSG_SITEM_ERROR("Unknwon type: %d", si->type);
			break;

	}

}

double
ivug_slider_item_zoom_get(Slide_Item* si)
{
	IV_ASSERT(si != NULL );

	double ret = 1.0;

	switch(si->type)
	{
		case SITEM_TYPE_IMAGE:
		{
			ret = ivug_image_zoom_get(si->photocam);
		}
		break;

		case SITEM_TYPE_VIDEO:
		break;

		case SITEM_TYPE_UNKNOWN:
		break;

		default:
			MSG_SITEM_ERROR("Unknwon type: %d", si->type);
			break;

	}


	return ret;
}


bool
ivug_slider_item_icon_click_check(Slide_Item *si, int x, int y)
{
	MSG_SITEM_HIGH("Check play icon clicked. SI=0x%08x, XY(%d,%d)", si, x, y);

	IV_ASSERT(si != NULL );

	Media_Data *mdata = ivug_medialist_get_data(si->mitem);

	if ( mdata == NULL )
	{
		MSG_SITEM_ERROR("Debug Me! Data item is NULL.");
		return false;
	}

	if ( mdata->slide_type == SLIDE_TYPE_VIDEO )
	{
		if (si->vIcon)
		{
			Evas_Coord ix,iy,iw,ih;

			evas_object_geometry_get(si->vIcon, &ix, &iy, &iw, &ih);

			bool bInside =  !(  (x) > (ix + iw) || (x) < (ix)
								|| (y) > (iy + iw) || (y) < (iy) );

			MSG_SITEM_HIGH("Item XYWH(%d,%d,%d,%d) XY(%d,%d) bInside=%d", ix,iy,iw,ih, x, y, bInside);

			return bInside;
		}
	}

	MSG_SITEM_HIGH("Slide type = %d", mdata->slide_type);
	return false;
}

drm_data_t * ivug_slider_item_check_drm_constraint(Slide_Item *si)
{
	IV_ASSERT(si != NULL);

	Media_Data *mdata = ivug_medialist_get_data(si->mitem);
	if(mdata == NULL)
	{
		MSG_SLIDER_ERROR("mdata is NULL");
		return NULL;
	}

	if(mdata->bIsDRMContent == EINA_FALSE)
	{
		MSG_SITEM_HIGH("It is not DRM file, path = %s", mdata->filepath);
		return NULL;
	}

	MSG_SITEM_HIGH("before check valid");
	if(ivug_drm_has_valid_ro(mdata->filepath) == 0)
	{
		MSG_SITEM_HIGH("RO is invalid, path = %s", mdata->filepath);
		si->state = SLIDE_STATE_ERROR;
		_show_thumbnail(si, DRM_NO_RIGHT_IMAGE_PATH);
		evas_object_smart_callback_call(si->photocam, "loaded",	(void *)EVAS_LOAD_ERROR_PERMISSION_DENIED);
		//for lauch WAP purchase page
		drm_data_t *result = (drm_data_t *)calloc(1, sizeof(drm_data_t));
		result->constraints = IVUG_DRM_RESULT_LIMITED;
		result->data = -1;
		return result;
	}
	MSG_SITEM_HIGH("RO is valid, path = %s", mdata->filepath);

	return ivug_drm_check_constraint(mdata->filepath);
}



bool
ivug_slider_item_image_rotate_set(Slide_Item *si, int angle)
{
	IV_ASSERT(si != NULL);

	ivug_image_rotate_set(si->photocam, ( (ivug_image_rotate_get(si->photocam) - angle) % 360 ));

	return true;
}


// Video specific APIs

void ivug_slider_item_video_play(Slide_Item* si)
{
	IV_ASSERT(si != NULL );

	MSG_SITEM_HIGH("Start Video play");

	if (si->type == SITEM_TYPE_VIDEO)
	{

	}

}


void ivug_slider_item_video_pause(Slide_Item* si)
{
	IV_ASSERT(si != NULL );

	MSG_SITEM_HIGH("Pause Video play");

	if (si->type == SITEM_TYPE_VIDEO)
	{

	}
}

void ivug_slider_item_hide_menu(Slide_Item* si)
{
	IV_ASSERT(si != NULL );

	MSG_SITEM_HIGH("Hide menu");

	si->bMenuVisible = false;

// Item is video and state is playing then hide paused icon
	if (si->type == SITEM_TYPE_VIDEO)
	{
		_hide_videoicon(si);
	}

}


void ivug_slider_item_show_menu(Slide_Item* si)
{
	IV_ASSERT(si != NULL );

	MSG_SITEM_HIGH("Show menu");

	si->bMenuVisible = true;

// Item is video and state is pause then show paused icon
	if (si->type == SITEM_TYPE_VIDEO)
	{
		_show_videoicon(si);
	}
}

Evas_Object *
ivug_slider_item_image_get(Slide_Item* si)
{
	IV_ASSERT(si != NULL );

	if ( (si->type == SITEM_TYPE_IMAGE) && (si->state == SLIDE_STATE_LOADED) )
	{
		return ivug_image_internal_image_get(si->photocam);
	}

	return NULL;

}


void ivug_slider_item_image_size_get(Slide_Item* si, int *w, int *h)
{
	IV_ASSERT(si != NULL );

	if ( (si->type == SITEM_TYPE_IMAGE) && (si->state == SLIDE_STATE_LOADED) )
	{
		ivug_image_image_size_get(si->photocam, w, h);
	}
}

void ivug_slider_item_reload(Slide_Item* si)
{
	evas_object_del(si->photocam);

	si->photocam = ivug_image_create(si->layout);
	evas_object_name_set(si->photocam, "photocam");
	evas_object_smart_callback_add(si->photocam, "loaded", _photocam_loaded_cb, si);

	elm_object_part_content_set(si->layout, "slider.contents", si->photocam);

	si->state = SLIDE_STATE_READY;

	ivug_slider_item_load(si);
}

