/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <ui-gadget-module.h>

#include "ivug-common.h"
#include "ivug-main-view.h"
#include "ivug-string.h"
#include "ivug-file-info.h"

#include "ivug-details-view.h"
#include "ivug-detail-info.h"

#include "ivug-drm-common.h"

#define DETAILS_EDJ_FILE EDJ_PATH"/ivug-details.edj"

/*
	this structure represent one item in genlist.
*/
typedef struct
{
	char *m_ItemName;
	char *m_ItemContents;
	exif_info_type_e type;
	Ivug_DetailsView *pDetailsView;
} ivug_file_list_item;


static char *
_ivug_details_gl_label_get(void *data, Evas_Object *obj, const char *part )
{
	ivug_file_list_item *params = (ivug_file_list_item *)data;

	ivug_retvm_if(params == NULL, NULL, "params is NULL");

	Ivug_DetailsView *pDetailsView = (Ivug_DetailsView *)params->pDetailsView;
	ivug_retvm_if(pDetailsView == NULL, NULL, "pDetailsView is NULL");

	//MSG_DETAIL_WARN("m_ItemName is %s", params->m_ItemName);

	if(strcmp(part, "elm.text.1") == 0)
	{
		if(params->m_ItemName)
		{
			return strdup(params->m_ItemName);

		}
		else
		{
			return strdup(IDS_UNKNOWN);
		}
	}
	else if(strcmp(part, "elm.text.2")==0)
	{
		if (params->m_ItemContents) {
			return elm_entry_utf8_to_markup(params->m_ItemContents);
		} else {
			return elm_entry_utf8_to_markup(IDS_UNKNOWN);
		}
	 }

	else
	{
		return NULL;
	}

}

static void
_ivug_details_gl_del(void *data, Evas_Object *obj )
{
	ivug_ret_if(NULL == data);

	ivug_file_list_item *params = (ivug_file_list_item *)data;

	ivug_retm_if(params == NULL, "params is NULL");
	if(params)
	{
		if(params->m_ItemName)
		{
			free(params->m_ItemName);
			params->m_ItemName = NULL;
		}
		if(params->m_ItemContents)
		{
			free(params->m_ItemContents);
			params->m_ItemContents = NULL;
		}
		free(params);
		params = NULL;
	}

	return;
}

static char*
_ivug_details_gl_label_get_title(void *data, Evas_Object *obj,const char *part )
{
	ivug_retv_if(NULL == data, NULL);

	if (!strcmp(part, "elm.text"))
	{
		return strdup(data);
	}

	return NULL;
}

static void _rename_button_clicked_cb(void *data, Evas_Object * obj, void *event_info)
{
	Ivug_DetailsView *pDetailsView = (Ivug_DetailsView *)data;

	MSG_DETAIL_HIGH("clicked event on Button");

	evas_object_smart_callback_call(pDetailsView->layout, "rename,clicked", (void *)pDetailsView);

	return;
}

static Evas_Object *_ivug_details_gl_icon_get_title(void *data, Evas_Object *obj, const char *part)
{
	ivug_file_list_item *params = (ivug_file_list_item *)data;

	ivug_retvm_if(params == NULL, NULL, "params is NULL");

	Ivug_DetailsView *pDetailsView = (Ivug_DetailsView *)params->pDetailsView;
	ivug_retvm_if(pDetailsView == NULL, NULL, "pDetailsView is NULL");

	Evas_Object *btn = NULL;

	if (!strcmp(part, "elm.icon") || !strcmp(part, "elm.icon.1")) {
		btn = elm_button_add(pDetailsView->layout);
		elm_object_style_set(btn, "rename");
		evas_object_smart_callback_add(btn, "clicked", _rename_button_clicked_cb, (void *)pDetailsView);
	}

	return btn;
}

static Eina_Bool
_ivug_details_gl_state_get(void *data, Evas_Object *obj, const char *part )
{
	return EINA_FALSE;
}

static Evas_Object *
_ivug_details_view_genlist_create(Ivug_DetailsView *pDetailsView, Evas_Object *parent)
{
	IV_ASSERT(pDetailsView != NULL);

	Evas_Object *genlist = NULL;
	genlist = elm_genlist_add(parent);

	ivug_retvm_if(NULL == genlist, NULL, "Create Details genlist failed..");

	elm_genlist_mode_set(genlist, ELM_LIST_COMPRESS);

	elm_object_theme_set(genlist, gGetSystemTheme());

	memset(&pDetailsView->itc, 0, sizeof(Elm_Genlist_Item_Class) );

	pDetailsView->itc.version = ELM_GENLIST_ITEM_CLASS_VERSION;
	pDetailsView->itc.item_style = "ivug/2text.2";
	pDetailsView->itc.func.text_get = _ivug_details_gl_label_get;
	pDetailsView->itc.func.content_get = NULL;
	pDetailsView->itc.func.state_get = _ivug_details_gl_state_get;
	pDetailsView->itc.func.del = _ivug_details_gl_del;

	memset(&pDetailsView->title_itc, 0, sizeof(Elm_Genlist_Item_Class) );

	// Set item class for dialogue group title
	pDetailsView->title_itc.version = ELM_GENLIST_ITEM_CLASS_VERSION;
	pDetailsView->title_itc.item_style = "grouptitle";
	pDetailsView->title_itc.func.text_get = _ivug_details_gl_label_get_title;
	pDetailsView->title_itc.func.content_get = NULL;
	pDetailsView->title_itc.func.state_get = NULL;
	pDetailsView->title_itc.func.del = NULL;

	memset(&pDetailsView->multiline_icon_itc, 0, sizeof(Elm_Genlist_Item_Class) );

	// Set item class for dialogue group title with icon
	pDetailsView->multiline_icon_itc.version = ELM_GENLIST_ITEM_CLASS_VERSION;
	pDetailsView->multiline_icon_itc.item_style = "ivug/multiline/2text.5icon";
	pDetailsView->multiline_icon_itc.func.text_get = _ivug_details_gl_label_get;
	pDetailsView->multiline_icon_itc.func.content_get = _ivug_details_gl_icon_get_title;
	pDetailsView->multiline_icon_itc.func.state_get = NULL;
	pDetailsView->multiline_icon_itc.func.del = NULL;

	memset(&pDetailsView->multiline_itc, 0, sizeof(Elm_Genlist_Item_Class) );

	pDetailsView->multiline_itc.version = ELM_GENLIST_ITEM_CLASS_VERSION;
	pDetailsView->multiline_itc.item_style = "ivug/multiline/2text";
	pDetailsView->multiline_itc.func.text_get = _ivug_details_gl_label_get;
	pDetailsView->multiline_itc.func.content_get = NULL;
	pDetailsView->multiline_itc.func.state_get = NULL;
	pDetailsView->multiline_itc.func.del = _ivug_details_gl_del;

	return genlist;
}

static void
_ivug_detail_genlist_view_exif_info_create(Ivug_DetailsView *pDetailsView, const char *path, Media_Type stype)
{
	IV_ASSERT(pDetailsView != NULL);

	Evas_Object *genlist = NULL;
	int index = 0;
	bool ret = false;

	Elm_Object_Item *it = NULL;

	ivug_file_list_item *m_TempItem = NULL;

	genlist = pDetailsView->genlist;

	char *name = NULL;
	char *contents = NULL;

	for (index = 0; index < EXIF_INFO_MAX; index++)
	{
		ret = ivug_datail_get_exif_info(&name, &contents, path, stype, index);
		if(ret == false)
		{
			continue;
		}

		m_TempItem = (ivug_file_list_item *)calloc(1, sizeof(ivug_file_list_item));
		ivug_retm_if(NULL == m_TempItem, "m_TempItem calloc failed");

		m_TempItem->m_ItemName = name;
		m_TempItem->m_ItemContents = contents;
		m_TempItem->pDetailsView = pDetailsView;

		it = elm_genlist_item_append(genlist, &pDetailsView->multiline_itc, m_TempItem, NULL, ELM_GENLIST_ITEM_NONE, NULL, NULL);
		elm_genlist_item_select_mode_set(it, ELM_OBJECT_SELECT_MODE_DISPLAY_ONLY);
	}

}

static void
_ivug_detail_genlist_view_file_view_create(Ivug_DetailsView *pDetailsView, const char *path, Media_Type stype)
{
	IV_ASSERT(pDetailsView != NULL);

	Evas_Object *genlist = NULL;
	int index = 0;

	Elm_Object_Item *git = NULL;
	Elm_Object_Item *it = NULL;

	ivug_file_list_item *m_TempItem = NULL;

	genlist = pDetailsView->genlist;

	char *name = NULL;
	char *contents = NULL;
	bool ret = false;

	git = elm_genlist_item_append(genlist, &pDetailsView->title_itc, strdup(IDS_DETAILS), NULL, ELM_GENLIST_ITEM_GROUP, NULL, NULL);
	elm_genlist_item_select_mode_set(git, ELM_OBJECT_SELECT_MODE_DISPLAY_ONLY);

	for (index = 0; index < FILE_INFO_MAX; index++)
	{
		ret = ivug_datail_get_file_info(&name, &contents, path, stype, index);
		if(ret == false)
		{
			continue;
		}

		m_TempItem = (ivug_file_list_item *)calloc(1, sizeof(ivug_file_list_item));
		if(m_TempItem == NULL)
		{
			continue;
		}

		m_TempItem->m_ItemName = name;
		if(index == FILE_INFO_NAME)
		{
			pDetailsView->filename = &(m_TempItem->m_ItemContents);	// for rename
		}
		else if(index == FILE_INFO_FILE_LOCATION)
		{
			pDetailsView->path = &(m_TempItem->m_ItemContents);	// for rename
		}
		m_TempItem->m_ItemContents = contents;
		m_TempItem->pDetailsView = pDetailsView;

		if(index == 0)	// for rename icon
		{
			it = elm_genlist_item_append(genlist, &pDetailsView->multiline_icon_itc, m_TempItem, NULL, ELM_GENLIST_ITEM_NONE, NULL, NULL);
		}
		else
		{
			it = elm_genlist_item_append(genlist, &pDetailsView->multiline_itc, m_TempItem, NULL, ELM_GENLIST_ITEM_NONE, NULL, NULL);
		}
		elm_genlist_item_select_mode_set(it, ELM_OBJECT_SELECT_MODE_DISPLAY_ONLY);
	}
}

static void
_ivug_detail_genlist_view_drm_create(Ivug_DetailsView *pDetailsView, const char *path, Media_Type stype)
{
	IV_ASSERT(pDetailsView != NULL);

	Evas_Object *genlist = NULL;
	int index = 0;

	Elm_Object_Item *it = NULL;

	ivug_file_list_item *m_TempItem = NULL;

	genlist = pDetailsView->genlist;

	char *name = NULL;
	char *contents = NULL;
	bool ret = false;

	for (index = 0; index < DRM_INFO_MAX; index++)
	{
		name = NULL;
		contents = NULL;
		ret = ivug_datail_get_drm_info(&name, &contents, path, stype, index);
		if(ret == false)
		{
			continue;
		}

		m_TempItem = (ivug_file_list_item *)calloc(1, sizeof(ivug_file_list_item));
		if(m_TempItem == NULL)
		{
			continue;
		}

		m_TempItem->m_ItemName = name;
		m_TempItem->m_ItemContents = contents;
		m_TempItem->pDetailsView = pDetailsView;

		it = elm_genlist_item_append(genlist, &pDetailsView->multiline_itc, m_TempItem, NULL, ELM_GENLIST_ITEM_NONE, NULL, NULL);
		elm_genlist_item_select_mode_set(it, ELM_OBJECT_SELECT_MODE_DISPLAY_ONLY);
	}

}

Ivug_DetailsView *
ivug_details_view_screen_create(Evas_Object *parent, Media_Item *mitem, ivug_view_by view_by)
{
	ivug_retv_if(!parent, NULL);
	ivug_retv_if(!mitem, NULL);

	Media_Data *mdata = ivug_medialist_get_data(mitem);
	ivug_retv_if(!mdata, NULL);
	ivug_retv_if(!mdata->filepath, NULL);

	char *filepath = mdata->filepath;
	Media_Type stype = mdata->slide_type;

	Ivug_DetailsView *pDetailsView = (Ivug_DetailsView *)calloc(1, sizeof(Ivug_DetailsView));
	if ( pDetailsView == NULL )
	{
		MSG_DETAIL_FATAL("Cannot allocate memory");
		return NULL;
	}

	pDetailsView->mitem = mitem;

// Creating GUI
	pDetailsView->evas = evas_object_evas_get(parent);

//layout

	pDetailsView->layout = ivug_layout_add2(parent, DETAILS_EDJ_FILE, "details_view");
	if(pDetailsView->layout == NULL)
	{
		free(pDetailsView);
		return NULL;
	}

#define IVUG_DETAILS_BG_ALPHA		200

	pDetailsView->bg = evas_object_rectangle_add(evas_object_evas_get(pDetailsView->layout));
	if (pDetailsView->bg)
	{
		evas_object_size_hint_expand_set(pDetailsView->bg, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
		evas_object_size_hint_fill_set(pDetailsView->bg, EVAS_HINT_FILL, EVAS_HINT_FILL);
		evas_object_color_set(pDetailsView->bg, IVUG_DEFAULT_BG_COLOR,
				 IVUG_DEFAULT_BG_COLOR, IVUG_DEFAULT_BG_COLOR, IVUG_DETAILS_BG_ALPHA);
		elm_object_part_content_set(pDetailsView->layout, "details_view.swallow.bg",
					   pDetailsView->bg);
	}

	//create genlist
	Evas_Object *genlist = NULL;

	genlist = _ivug_details_view_genlist_create(pDetailsView, pDetailsView->layout);
	pDetailsView->genlist = genlist;

	_ivug_detail_genlist_view_file_view_create(pDetailsView, filepath, stype);

	if(mdata->bIsDRMContent == EINA_TRUE)
	{
		_ivug_detail_genlist_view_drm_create(pDetailsView, filepath, stype);
	}
	else
	{
		_ivug_detail_genlist_view_exif_info_create(pDetailsView, filepath, stype);
	}

	elm_object_part_content_set(pDetailsView->layout, "details_view.genlist" , pDetailsView->genlist );

	MSG_DETAIL_HIGH("Detail view(0x%08x) created for %s", pDetailsView, filepath);

	return pDetailsView;
}


Evas_Object *
ivug_details_view_object_get(Ivug_DetailsView *pDetailsView)
{
	IV_ASSERT(pDetailsView != NULL);

	return pDetailsView->layout;
}

void
ivug_details_view_destroy(Ivug_DetailsView *pDetailsView)
{
	IV_ASSERT(pDetailsView != NULL);
	MSG_DETAIL_HIGH("Removing Details VIew");

// Layout removed automatically when popping from navi item.
	if (pDetailsView->genlist) {
		elm_genlist_clear(pDetailsView->genlist);
		pDetailsView->genlist = NULL;
	}
	if(pDetailsView->bg)
	{
		evas_object_del(pDetailsView->bg);
		pDetailsView->bg = NULL;
	}

	if(pDetailsView->layout)
	{
		evas_object_del(pDetailsView->layout);
		pDetailsView->layout = NULL;
	}

	free(pDetailsView);

	pDetailsView = NULL;
}

void ivug_details_view_update(Ivug_DetailsView *pDetailsView, const char *new_path)
{
	MSG_IVUG_HIGH("ivug_details_view_update");

	free(*(pDetailsView->filename));
	*(pDetailsView->filename) = ecore_file_strip_ext(ecore_file_file_get(new_path));

	free(*(pDetailsView->path));
	*(pDetailsView->path) = strdup(new_path);
	//elm_genlist_realized_items_update(pDetailsView->genlist);
	Elm_Object_Item *item = NULL;
	if (pDetailsView->genlist) {
		item = elm_genlist_first_item_get(pDetailsView->genlist);
		while (item) {
			elm_genlist_item_update(item);
			item = elm_genlist_item_next_get(item);
		}
	}
}

