/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "ivug-common.h"
#include "ivug-main-view.h"
#include "ivug-main-view-toolbar.h"
#include "ivug-main-view-menu.h"
#include "ivug-main-view-priv.h"

#include "ivug-slider.h"
#include "ivug-slider-item.h"

#include "ivug-listpopup.h"
#include "ivug-ext-ug.h"
#include "ivug-popup.h"
#include "ivug-selectioninfo.h"

#include "ivug-media.h"
#include "ivug-slideshow.h"

#include "ivug-file-info.h"
#include "ivug-exif.h"

#include <ui-gadget-module.h>		// ug_destroy_me, ug_send_result
#include <media-thumbnail.h>
#include <app_manager.h>		//app_manager_get_app_name

#include "ivug-crop-view.h"



#include <media-util-register.h>	// media_file_register

#define IVUG_DEFINE_EXIF_ROTATE

#define GALLERY_PKG_NAME	"org.tizen.gallery"
#define SHORTCUT_PREFIX		"gallery:imageviewer:"
#define SHORTCUT_PREFIX_LEN	strlen(SHORTCUT_PREFIX)
#define SHORTCUT_ICON_PATH	"/usr/share/icons/default/small/org.tizen.image-viewer.png"

static void
_on_ext_ug_destroy_cb(ui_gadget_h ug, void *priv)
{
	if (!ug ) return;

	Ivug_MainView *pMainView = (Ivug_MainView *)priv;

	MSG_IMAGEVIEW_HIGH("UG destroyed");

	pMainView->ext_ug = NULL;

	ivug_main_view_set_hide_timer(pMainView);
}

static void _on_selectioninfo_deleted(void * data, Evas * e, Evas_Object * obj, void * event_info)
{
	Ivug_MainView *pMainView = (Ivug_MainView *)data;
	IV_ASSERT(pMainView != NULL);

	MSG_MAIN_HIGH("selection info deleted");

	pMainView->selectioninfo = NULL;
}

void _ivug_selectioninfo_create(Ivug_MainView *pMainView, const char *string)
{
	IV_ASSERT(pMainView != NULL);

	if(pMainView->selectioninfo)
	{
		evas_object_del(pMainView->selectioninfo);
		pMainView->selectioninfo = NULL;
	}

	pMainView->selectioninfo = ivug_selectioninfo_create(pMainView->layout, string);
	IV_ASSERT(pMainView->selectioninfo != NULL);

	evas_object_event_callback_add(pMainView->selectioninfo, EVAS_CALLBACK_DEL, _on_selectioninfo_deleted, pMainView);
}


void _delete_details_view(Ivug_MainView *pMainView )
{
	Evas_Object *detail_view_layout = elm_object_item_part_content_unset(pMainView->navi_it, "elm.swallow.detail");

	MSG_MAIN_HIGH("Delete detail view. old layout=0x%08x", detail_view_layout);

	ivug_details_view_destroy(pMainView->pDetailsView);  // Delete detail view.
	pMainView->pDetailsView = NULL;

}


static void _on_add_tag_view_destroy(void *data, Evas_Object *obj, void *event_info)
{
	MSG_MAIN_HIGH("transition finished");
	Ivug_MainView *pMainView = (Ivug_MainView *)data;

	ivug_name_view_destroy(pMainView->pNameView);
	pMainView->pNameView = NULL;	// Will removed in add tag view.

	evas_object_smart_callback_del(pMainView->navi_bar, "transition,finished",
				_on_add_tag_view_destroy);
}

static void _on_add_tag_view_response(ivug_name_response resp, const char *str, void *pClientData)
{
	Ivug_MainView *pMainView = (Ivug_MainView *)pClientData;
	bool ret = false;
	Media_Item *mitem = NULL;
	Media_Data *mdata = NULL;

	Evas_Object *conformant = NULL;

	switch(resp)
	{
	case NAME_VIEW_RESPONSE_OK:
		mitem = ivug_slider_get_item(pMainView->slider);
		mdata = ivug_medialist_get_data(mitem);

		ret = ivug_mediadata_set_tag(mdata, str);
		if(ret == false)
		{
			MSG_MAIN_ERROR("ivug_mediadata_set_tag failed: ret = %d", ret);
			_ivug_selectioninfo_create(pMainView, IDS_FAILED);
		}
		else
		{
			MSG_MAIN_HIGH("New tags is added");
			_ivug_selectioninfo_create(pMainView, IDS_ADDED);
		}
		evas_object_smart_callback_add(pMainView->navi_bar, "transition,finished",
				_on_add_tag_view_destroy, pMainView);

		elm_naviframe_item_pop(pMainView->navi_bar);

		pMainView->navi_it = elm_naviframe_top_item_get(pMainView->navi_bar);

		conformant = (Evas_Object *)ug_get_conformant();
		elm_object_signal_emit(conformant, "elm,state,indicator,overlap", "");
		break;
	case NAME_VIEW_RESPONSE_CANCEL:
		MSG_MAIN_HIGH("Add tag is canceled");
		evas_object_smart_callback_add(pMainView->navi_bar, "transition,finished",
				_on_add_tag_view_destroy, pMainView);

		elm_naviframe_item_pop(pMainView->navi_bar);

		pMainView->navi_it = elm_naviframe_top_item_get(pMainView->navi_bar);

		conformant = (Evas_Object *)ug_get_conformant();
		elm_object_signal_emit(conformant, "elm,state,indicator,overlap", "");
		break;
	default:
		MSG_MAIN_ERROR("Unhandled mode : %d", resp);
		break;
	}

	elm_naviframe_item_title_visible_set(pMainView->navi_it, EINA_TRUE);
	elm_object_item_signal_emit(pMainView->navi_it, "elm,state,toolbar,open", "");

	ivug_main_view_set_hide_timer(pMainView);
}

static	void
_on_setasview_deleted(void *data, Evas *e , Evas_Object *obj , void *event_info )
{
	ivug_retm_if(!data, "data is NULL");
	Ivug_MainView *pMainView = (Ivug_MainView *)data;
	MSG_MAIN_HIGH("SetAS View is destroyed");

	pMainView->pSetAsView = NULL;

	ivug_main_view_set_hide_timer(pMainView);
}

static void _on_setas_view_destroyed(void *data, Evas_Object *obj, void *event_info)
{
	IV_ASSERT(data != NULL);
	Ivug_MainView *pMainView = (Ivug_MainView *)data;

	elm_naviframe_item_pop(pMainView->navi_bar);

	pMainView->navi_it = elm_naviframe_top_item_get(pMainView->navi_bar);

	//Evas_Object *conformant = (Evas_Object *)ug_get_conformant();
	//elm_object_signal_emit(conformant, "elm,state,indicator,overlap", "");

	if(pMainView->pSetAsView)
	{
		ivug_setas_view_destroy(pMainView->pSetAsView);
		pMainView->pSetAsView = NULL;
	}
}

static void
_on_timeout_response(void *data, Evas_Object *obj, void *event_info)
{
	IV_ASSERT(data != NULL);
	Ivug_MainView *pMainView = (Ivug_MainView *)data;
	ivug_main_view_set_hide_timer(pMainView);
}


void _on_share_selected(void *data, Evas_Object *obj, void *event_info)
{
	IV_ASSERT(data != NULL);

	Ivug_MainView *pMainView = (Ivug_MainView *)data;
	IV_ASSERT(pMainView->ext_ug == NULL);

	Ivug_ListPopup_Item *item = (Ivug_ListPopup_Item *)event_info;

	const char *label = (const char *)ivug_listpopup_item_get_data(item);

	if(label == NULL)
	{
		MSG_MAIN_ERROR("label is NULL");
		evas_object_del(obj);
		ivug_main_view_set_hide_timer(pMainView);
		return;
	}
	MSG_MAIN_HIGH("text(%s) is clicked", label);

	Media_Item *mitem = ivug_slider_get_item(pMainView->slider);
	Media_Data *mdata = ivug_medialist_get_data(mitem);

	if(mdata == NULL)
	{
		MSG_MAIN_ERROR("sd is NULL");
		evas_object_del(obj);
		return;
	}

	char *path = NULL;
	path = mdata->filepath;

	bool ret = false;
	ret = ivug_ext_launch_to_send(path, label, _on_ext_ug_destroy_cb, data, pMainView->ext_ug);

	/* appsvc failed or ug_create failed */
	if(ret == false)
	{
		ivug_timeout_popup_show(pMainView->layout, _on_timeout_response, pMainView, NULL, IDS_APPLICATION_NOT_INSTALLED);
	}

	evas_object_del(obj);
}

static void  _ivug_crop_view_transit_finished_cb(void *data, Evas_Object *obj, void *event_info)
{
	Ivug_MainView *pMainView = (Ivug_MainView *)data;

	ivug_crop_view_destroy(pMainView->pCropView);

	pMainView->pCropView = NULL;

	evas_object_smart_callback_del(pMainView->navi_bar, "transition,finished",
			_ivug_crop_view_transit_finished_cb);
}

static void  _ivug_crop_view_destroyed_cb(void *data, Evas_Object *obj, void *event_info)
{
	Ivug_MainView *pMainView = (Ivug_MainView *)data;

	ivug_main_view_set_hide_timer(pMainView);

	evas_object_smart_callback_add(pMainView->navi_bar, "transition,finished",
			_ivug_crop_view_transit_finished_cb, pMainView);

	_ivug_crop_view_box_delete(pMainView->pCropView);

	elm_naviframe_item_pop(pMainView->navi_bar);
	pMainView->navi_it = elm_naviframe_top_item_get(pMainView->navi_bar);

	elm_naviframe_item_title_visible_set(pMainView->navi_it, EINA_TRUE);

	_update_title(pMainView);	// after crop index changed

	//Evas_Object *conformant = (Evas_Object *)ug_get_conformant();
	//elm_object_signal_emit(conformant, "elm,state,indicator,overlap", "");
}

#if 0
static void _create_setas_view(Ivug_MainView *pMainView, const char *filepath, ivug_set_screen_type type)
{
	pMainView->pSetAsView = ivug_setas_view_screen_create(pMainView->layout, filepath,
				IVUG_SETAS_NORMAL, type);
	Evas_Object *setasOjbect = ivug_setas_view_object_get(pMainView->pSetAsView);
	//evas_object_event_callback_add(setasOjbect, EVAS_CALLBACK_DEL, _on_setasview_deleted, pMainView);
	evas_object_smart_callback_add(setasOjbect, "destroy", _on_setas_view_destroyed, pMainView);

	Evas_Object *back_btn = ivug_button_add(pMainView->navi_bar, "naviframe/end_btn/default",
				IDS_BACK, NULL, _on_setas_view_destroyed, pMainView);

	pMainView->navi_it = elm_naviframe_item_push(pMainView->navi_bar,
							NULL, back_btn, NULL, setasOjbect, NULL);

	ivug_setas_view_create_menu(pMainView->pSetAsView, pMainView->navi_bar, type);
}
#else

static void _ivug_crop_view_ok_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
	char *path = (char *)event_info;
	media_handle m_handle = NULL;

	MSG_MAIN_HIGH("_ivug_crop_view_ok_clicked_cb path = %d", path);

	evas_object_smart_callback_del(obj, "ok,clicked", _ivug_crop_view_ok_clicked_cb);

	m_handle = ivug_db_insert_file_to_DB(path);
	if(m_handle == NULL)
	{
		MSG_MAIN_ERROR("ivug_db_insert_file_to_DB failed %s", path);
	}
	else
	{
		ivug_db_destroy_file_handle(m_handle);
}

	Ivug_MainView *pMainView = (Ivug_MainView *)data;

	Media_Item *mitem = ivug_medialist_prepend_item(pMainView->mList, path);

	Media_Item *c_mitem = ivug_slider_get_item(pMainView->slider);
	Media_Data *c_mdata = ivug_medialist_get_data(c_mitem);
	if(c_mdata->index == 1)	//left slide item added
	{
		MSG_MAIN_ERROR("add left slide item");
		ivug_slider_set_item_to_slide(pMainView->slider, PREV_SLIDE, mitem);
	}
}

static void _ivug_setas_view_ok_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
	char *path = (char *)event_info;

	MSG_MAIN_HIGH("_ivug_setas_view_ok_clicked_cb path = %d", path);

	evas_object_smart_callback_del(obj, "ok,clicked", _ivug_crop_view_ok_clicked_cb);

	// do not insert to db

	int setas_data = (int)evas_object_data_get(obj, "setas_type");
	ivug_set_screen_type type = (ivug_set_screen_type)setas_data;

	const char* homescreen_path = IVUG_HOME_SCREEN_PATH;
	const char* lockscreen_path = IVUG_LOCK_SCREEN_PATH;

	if(type == IVUG_CTRLBAR_SET_SCREEN_HOME)
	{
		ivug_copy_file(path, homescreen_path);
		ivug_config_set_homescreen_image(homescreen_path);
	}
	else if(type == IVUG_CTRLBAR_SET_SCREEN_LOCK)
	{
		ivug_copy_file(path, lockscreen_path);
		ivug_config_set_lockscreen_image(lockscreen_path);
	}
	else if(type == IVUG_CTRLBAR_SET_SCREEN_BOTH)
	{
		ivug_copy_file(path, homescreen_path);
		ivug_config_set_homescreen_image(homescreen_path);
		ivug_copy_file(path, lockscreen_path);
		ivug_config_set_lockscreen_image(lockscreen_path);
	}
}

static void _create_setas_view(Ivug_MainView *pMainView, const char *filepath, ivug_set_screen_type type)
{
	IvugCropView *pCropView = ivug_crop_view_create(pMainView->layout);

	int lcd_x = 0;
	int lcd_y = 0;
	int lcd_w = 0;
	int lcd_h = 0;
	int rot = 0;

	Evas_Object *layout = ivug_crop_view_get_object(pCropView);

	evas_object_smart_callback_add(layout, "ok,clicked", _ivug_setas_view_ok_clicked_cb, pMainView);
	evas_object_smart_callback_add(layout, "destroyed", _ivug_crop_view_destroyed_cb, pMainView);

	evas_object_data_set(layout, "setas_type", (void *)type);

	Evas_Object *btn_back = ivug_button_add(layout, "naviframe/end_btn/default",
		IDS_BACK, NULL, _ivug_crop_view_destroyed_cb, pMainView);

	pMainView->navi_it = elm_naviframe_item_push(pMainView->navi_bar, NULL, btn_back, NULL, pCropView->layout, NULL);

	//Evas_Object *conformant = (Evas_Object *)ug_get_conformant();
	//elm_object_signal_emit(conformant, "elm,state,indicator,nooverlap", "");

	ivug_crop_view_create_menu(pCropView, pMainView->navi_bar);

	ivug_crop_view_file_set(pCropView, filepath);

	ivug_crop_view_destination_set(pCropView, DATA_PATH, ".iv_setas.jpg");

	evas_object_geometry_get((Evas_Object *)ug_get_window(), &lcd_x, &lcd_y, &lcd_w, &lcd_h);
	rot = gGetRotationDegree();
	if(rot == 90 || rot == 270)
	{
		int temp = lcd_w;
		lcd_w = lcd_h;
		lcd_h = temp;
	}

	ivug_crop_view_box_size_set(pCropView, lcd_w, lcd_h);

	ivug_crop_view_box_ratio_fix(pCropView, true);

	pMainView->pCropView = pCropView;

	elm_naviframe_item_title_visible_set(pMainView->navi_it, EINA_FALSE);
}
#endif

void _on_setas_selected(void *data, Evas_Object *obj, void *event_info)
{
	IV_ASSERT(data != NULL);

	Ivug_MainView *pMainView = (Ivug_MainView *)data;

	Ivug_ListPopup_Item *item = (Ivug_ListPopup_Item *)event_info;

	const char *label = (const char *)ivug_listpopup_item_get_data(item);

	if(label == NULL)
	{
		MSG_MAIN_ERROR("label is NULL");
		evas_object_del(pMainView->ctx_popup2);
		pMainView->ctx_popup2 = NULL;
		ivug_main_view_set_hide_timer(pMainView);
		return;
	}
	MSG_MAIN_HIGH("text(%s) is clicked", label);

	Media_Item *mitem = ivug_slider_get_item(pMainView->slider);
	Media_Data *mdata = ivug_medialist_get_data(mitem);
	if(mdata == NULL)
	{
		MSG_MAIN_ERROR("sd is NULL");
		evas_object_del(pMainView->ctx_popup2);
		pMainView->ctx_popup2 = NULL;
		ivug_main_view_set_hide_timer(pMainView);
		return;
	}

	if(strncmp(label, IDS_HOME_SCREEN, strlen(label)) == 0)
	{
		_create_setas_view(pMainView, mdata->filepath, IVUG_CTRLBAR_SET_SCREEN_HOME);
	}
	else if(strncmp(label, IDS_LOCK_SCREEN, strlen(label)) == 0)
	{
		_create_setas_view(pMainView, mdata->filepath, IVUG_CTRLBAR_SET_SCREEN_LOCK);
	}
	else if(strncmp(label, IDS_HOME_AND_LOCK_SCREENS, strlen(label)) == 0)
	{
		_create_setas_view(pMainView, mdata->filepath, IVUG_CTRLBAR_SET_SCREEN_BOTH);
	}
	else if(strncmp(label, IDS_CALLER_IMAGE, strlen(label)) == 0)
	{
		pMainView->ext_ug = ivug_ext_launch_contact(mdata->filepath, _on_ext_ug_destroy_cb, data);
	}

	evas_object_del(pMainView->ctx_popup2);
	pMainView->ctx_popup2 = NULL;
}


void _on_addtag_selected(void *data, Evas_Object *obj, void *event_info)
{
	IV_ASSERT(data != NULL);

	Ivug_MainView *pMainView = (Ivug_MainView *)data;

	Ivug_ListPopup_Item *item = (Ivug_ListPopup_Item *)event_info;

	const char *label = (const char *)ivug_listpopup_item_get_data(item);

	if(label == NULL)
	{
		MSG_MAIN_ERROR("label is NULL");
		evas_object_del(pMainView->ctx_popup2);
		pMainView->ctx_popup2 = NULL;
		ivug_main_view_set_hide_timer(pMainView);
		return;
	}

	MSG_MAIN_HIGH("text(%s) is clicked", label);
	Media_Item *mitem = ivug_slider_get_item(pMainView->slider);
	Media_Data *mdata = ivug_medialist_get_data(mitem);
	if(mdata == NULL)
	{
		MSG_MAIN_ERROR("sd is NULL");
		evas_object_del(pMainView->ctx_popup2);
		pMainView->ctx_popup2 = NULL;
		ivug_main_view_set_hide_timer(pMainView);
		return;
	}

	if(strncmp(label, IDS_CREATE_TAG, strlen(label)) == 0)
	{
		pMainView->pNameView = ivug_name_view_create(pMainView->layout, IDS_CREATE_TAG);

		ivug_name_view_set_max_length(pMainView->pNameView, MAX_BYTE_LEN);

		ivug_name_view_set_response_callback(pMainView->pNameView, _on_add_tag_view_response, (void*)pMainView);

		Evas_Object *layout = ivug_name_view_object_get(pMainView->pNameView);

		pMainView->navi_it = elm_naviframe_item_push(pMainView->navi_bar, NULL, NULL, NULL, layout, NULL);

		Evas_Object *conformant = (Evas_Object *)ug_get_conformant();
		elm_object_signal_emit(conformant, "elm,state,indicator,nooverlap", "");

		elm_naviframe_item_title_visible_set(pMainView->navi_it, EINA_FALSE);
		elm_object_item_signal_emit(pMainView->navi_it, "elm,state,toolbar,close", "");

		ivug_name_view_set_focus(pMainView->pNameView);
	}
	else if(strncmp(label, IDS_FAVOURITES, strlen(label)) == 0)
	{
		if ( ivug_mediadata_set_favorite(mdata, true) == false )
		{
			MSG_MAIN_ERROR("Error!. Set favorite for ID=%s", uuid_getchar(mdata->mediaID));
			goto ADD_FAIL;
		}

		_ivug_selectioninfo_create(pMainView, IDS_ADDED);
		ivug_main_view_set_hide_timer(pMainView);
	}
	else
	{
		bool ret = ivug_mediadata_set_tag(mdata, (char *)label);
		if(ret == true)
		{
			_ivug_selectioninfo_create(pMainView, IDS_ADDED);
		}
		else
		{
			goto ADD_FAIL;
		}
		ivug_main_view_set_hide_timer(pMainView);
	}

	evas_object_del(pMainView->ctx_popup2);
	pMainView->ctx_popup2 = NULL;

	return;
ADD_FAIL:
	_ivug_selectioninfo_create(pMainView, IDS_FAILED);
	evas_object_del(pMainView->ctx_popup2);
	pMainView->ctx_popup2 = NULL;

	ivug_main_view_set_hide_timer(pMainView);

	return;

}


#ifndef USE_DEFAULT_DOWNLOADS_FOLDER

void _on_save_selected(void *data, Evas_Object *obj, void *event_info)
{
	IV_ASSERT(data != NULL);

	Ivug_MainView *pMainView = (Ivug_MainView *)data;
	ivug_listpopup_item *item = (ivug_listpopup_item *)event_info;

	if ( item == NULL )	// No item is selected
	{
		evas_object_del(pMainView->popup);
		pMainView->popup = NULL;
		ivug_main_view_set_hide_timer(pMainView);
		return;
	}
	MSG_MAIN_HIGH("Selected folder name = %s", item->caption);

	char *dir_path = (char *)item->data;		// Should be freed
	char dst_file[IVUG_MAX_FILE_PATH_LEN + 1] = {0,};
	const char *filename = NULL;
	int ret = -1;

	Media_Item *mitem = ivug_slider_get_item(pMainView->slider);
	Media_Data *mdata = ivug_medialist_get_data(mitem);
	if ( mdata == NULL )
	{
		MSG_MAIN_ERROR("Cannot get slide data.");
		goto SAVE_FAIL;
	}

	filename = ecore_file_file_get(mdata->filepath);
	if (filename == NULL)
	{
		MSG_MAIN_ERROR("File does not existfilepath=%s", mdata->filepath);
		goto SAVE_FAIL;
	}

	snprintf(dst_file, IVUG_MAX_FILE_PATH_LEN, "%s/%s", dir_path, filename);

	if(!ecore_file_cp(mdata->filepath, dst_file))
	{
		MSG_MAIN_ERROR("ecore_file_cp failed. From %s To %s", mdata->filepath, dst_file);
		goto SAVE_FAIL;
	}

	free(dir_path);

	ret = media_file_register(dst_file);
	if(ret < 0)
	{
		MSG_MAIN_ERROR("Cannot add to album file %s. ret=%d", dst_file, ret);
		_ivug_selectioninfo_create(pMainView, IDS_FAILED);
	}
	else
	{
		_ivug_selectioninfo_create(pMainView, IDS_SAVED);
	}

	evas_object_del(pMainView->popup);
	pMainView->popup = NULL;

	ivug_main_view_set_hide_timer(pMainView);

	return;

SAVE_FAIL:
	if(dir_path)
		free(dir_path);

	_ivug_selectioninfo_create(pMainView, IDS_FAILED);

	evas_object_del(pMainView->popup);
	pMainView->popup = NULL;

	ivug_main_view_set_hide_timer(pMainView);

	return;
}

#endif

static void
_on_delete_selected(void *data, Evas_Object *obj, void *event_info)
{
	Ivug_MainView *pMainView = (Ivug_MainView *)data;
	IV_ASSERT(pMainView != NULL);

	int response_id = (int)event_info;
	if(response_id == POPUP_RESPONSE_CANCEL)
	{
		MSG_MAIN_HIGH("cancel selected");
		ivug_main_view_set_hide_timer(pMainView);
		return;
	}

	if(pMainView->pDetailsView)
	{
		MSG_MAIN_HIGH("Delete details view.");
		//_change_close_to_details(pMainView);
		_delete_details_view(pMainView);
	}

	MSG_MAIN_HIGH("Removing current slide.");

	ivug_main_view_set_hide_timer(pMainView);

	_ivug_selectioninfo_create(pMainView, IDS_DELETED);

	ivug_slider_delete_item(pMainView->slider);		// this routine fire "item.changed"
}

static void
_on_rename_selected(void *data, Evas_Object *obj, void *event_info)
{
	int response_id = (int)event_info;
	IV_ASSERT(data != NULL);

	Ivug_MainView *pMainView = (Ivug_MainView *)data;

	MSG_MAIN_HIGH("DRM popup selected : %d", response_id);

	if(response_id == POPUP_RESPONSE_CANCEL)
	{
		MSG_MAIN_HIGH("cancel selected");
	}
	else
	{
		MSG_MAIN_HIGH("unlock process");

		Media_Item *mitem = ivug_slider_get_item(pMainView->slider);
		Media_Data *mdata = ivug_medialist_get_data(mitem);

		IV_ASSERT(mdata != NULL);

		const char *old_fullpath = mdata->filepath;
		char *old_dir = ecore_file_dir_get(old_fullpath);
		char *ext = ivug_fileinfo_get_file_extension(old_fullpath);

		Evas_Object *popup = obj;
		Evas_Object *layout = elm_object_content_get(popup);
		Evas_Object *entry = elm_object_part_content_get(layout, "elm.swallow.content");

		const char *new_name = elm_entry_entry_get(entry);
		char new_fullpath[IVUG_MAX_FILE_PATH_LEN] = {0,};
		snprintf(new_fullpath, sizeof(new_fullpath),"%s/%s.%s", old_dir, new_name, ext);

		if(ivug_rename_file(old_fullpath, new_fullpath) == false)
		{
			MSG_MAIN_ERROR("ivug_rename_file failed");
		}
		else
		{
			MSG_MAIN_HIGH("Rename %s -> %s", old_fullpath, new_fullpath);
			free(mdata->filepath);
			mdata->filepath = strdup(new_fullpath);
		}
	}
	ivug_main_view_set_hide_timer(pMainView);
}




////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// Button handlers
//

static void _dismissed_cb(void *data, Evas_Object *obj, void *event_info)
{
	Ivug_MainView *pMainView = (Ivug_MainView *)data;
	//evas_object_del(obj);
	ivug_main_view_set_hide_timer(pMainView);

	if(pMainView->popup)	//for tag popup
	{
		MSG_MAIN_HIGH("Removing popup 1");
		evas_object_del(pMainView->popup);
		pMainView->popup = NULL;
	}
	else
	{
		MSG_MAIN_HIGH("Removing popup 2");
		evas_object_del(obj);
	}


	MSG_MAIN_HIGH("Popup dismissed");
}

#ifndef USE_DEFAULT_DOWNLOADS_FOLDER
static bool
_iter_album_list(media_handle media, void *user_data)
{
	ivug_listpopup_itemlist items = (ivug_listpopup_itemlist)user_data;

	char *name = ivug_db_get_folder_name(media);
	char *path = ivug_db_get_folder_path(media);

	if(strlen(name) == 0)
	{
		ivug_listpopup_itemlist_add(items, 0, NULL, IDS_NO_NAME, (void *)strdup(path), false);
	}
	else
	{
		ivug_listpopup_itemlist_add(items, 0, NULL, name, (void *)strdup(path), false);
	}

	if(name)
		free(name);
	if(path)
		free(path);

    return 0;
}
#endif


static bool
_iter_tag_list(media_handle media, void *user_data)
{
	Ivug_MainView *pMainView = (Ivug_MainView *)user_data;

	char *name = ivug_db_get_tag_name((tag_handle)media);

	if(pMainView->ctx_popup2)
	{
		ivug_listpopup_item_append(pMainView->ctx_popup2, NULL, name, name);
	}
	else if(pMainView->popup)
	{
		ivug_listpopup_item_append(pMainView->popup, NULL, name, name);
	}

	MSG_IVUG_HIGH("tag name =%s", name);

	return 0;
}


void on_btn_back_clicked(void *data, Evas_Object *obj, void *event_info)
{
// Back from main view
	Ivug_MainView *pMainView = (Ivug_MainView *)data;
	IV_ASSERT(pMainView != NULL);

	MSG_MAIN_WARN("Back key from main view pressed");

	if ( pMainView->pDetailsView )
	{
		MSG_MAIN_HIGH("Details View Destroy");

		ivug_details_view_destroy(pMainView->pDetailsView);
		pMainView->pDetailsView = NULL;
		return;
	}

// otherwise, from gallery, ug will be terminated
	evas_object_smart_callback_del(obj, "clicked", on_btn_back_clicked);

#ifndef FIX_LAST_BLINK
	ivug_main_view_del_hide_timer(pMainView);

// Slider should be removed before mList is freed.
	if ( pMainView->slider )
	{
		evas_object_del(pMainView->slider);
		pMainView->slider = NULL;
	}

	if ( pMainView->pNameView )
	{
		MSG_MAIN_HIGH("Name View Destroy");
		ivug_name_view_destroy(pMainView->pNameView);
		pMainView->pNameView = NULL;
	}

	if ( pMainView->pSetAsView )
	{
		MSG_MAIN_HIGH("SetAS View Destroy");
		ivug_setas_view_destroy(pMainView->pSetAsView);
		pMainView->pSetAsView = NULL;
	}

	if ( pMainView->pDetailsView )
	{
		MSG_MAIN_HIGH("Details View Destroy");

		ivug_details_view_destroy(pMainView->pDetailsView);
		pMainView->pDetailsView = NULL;
	}


#if 1
	if(pMainView->layout)		// Remove topmost parent in main view.
	{
		evas_object_del(pMainView->layout);
		pMainView->layout = NULL;
	}

	if(pMainView->empty_layout)
	{
		evas_object_del(pMainView->empty_layout);
		pMainView->empty_layout = NULL;
	}
#else

#endif

	if ( pMainView->mList )
	{
		MSG_MAIN_HIGH("Remove media list");
		ivug_medialist_del(pMainView->mList);		// ivug_medialist_del() is not working on destroy cb.
		pMainView->mList = NULL;
	}

#endif

#if 0
	/* comment below, because t show main view and this back btn,
	   *firstly slide show view is mouse up, then ivug_ss_delete() will be invoked in taht mouse up cb
	*/
	/* free slideshow */
	if ( pMainView->ssHandle )
	{
		ivug_ss_delete(pMainView->ssHandle);
	}
#endif
	DESTROY_ME();
}

void on_btn_setas_clicked(void *data, Evas_Object *obj, void *event_info)
{
	Ivug_MainView *pMainView = (Ivug_MainView *)data;
	IV_ASSERT(pMainView != NULL);

	Media_Item *mitem = ivug_slider_get_item(pMainView->slider);
	Media_Data *mdata = ivug_medialist_get_data(mitem);

	IV_ASSERT(mdata != NULL);

	if ( mdata->filepath == NULL)
	{
		MSG_MAIN_ERROR("File path is NULL");
		return;
	}

	//ivug_main_view_del_hide_timer(pMainView);

	Evas_Object* popup = NULL;
	popup = ivug_listpopup_add(pMainView->area_obj);

	pMainView->ctx_popup2 = popup;

	evas_object_smart_callback_add(popup, "popup,dismissed", _dismissed_cb, pMainView);
	evas_object_smart_callback_add(popup, "popup,selected", _on_setas_selected, pMainView);

	if (mdata->bIsDRMContent)
	{
		Ivug_ListPopup_Item *it = NULL;

		bool bDisableWallpaper = !ivug_drm_is_possible_to_setas(mdata->filepath, IVUG_DRM_SETAS_WALLPAPER);
		bool bDisableCallerImage = !ivug_drm_is_possible_to_setas(mdata->filepath, IVUG_DRM_SETAS_CALLERID);

		it = ivug_listpopup_item_append(popup, NULL, IDS_HOME_SCREEN, IDS_HOME_SCREEN);

		if(bDisableWallpaper == true)
		{
			ivug_listpopup_item_disabled_set(it);
		}

		it = ivug_listpopup_item_append(popup, NULL, IDS_LOCK_SCREEN, IDS_LOCK_SCREEN);

		if(bDisableWallpaper == true)
		{
			ivug_listpopup_item_disabled_set(it);
		}

		it = ivug_listpopup_item_append(popup, NULL, IDS_HOME_AND_LOCK_SCREENS, IDS_HOME_AND_LOCK_SCREENS);
		if(bDisableWallpaper == true)
		{
			ivug_listpopup_item_disabled_set(it);
		}

		it = ivug_listpopup_item_append(popup, NULL, IDS_CALLER_IMAGE, IDS_CALLER_IMAGE);
		if(bDisableCallerImage == true)
		{
			ivug_listpopup_item_disabled_set(it);
		}
	}
	else
	{
		ivug_listpopup_item_append(popup, NULL, IDS_HOME_SCREEN, IDS_HOME_SCREEN);
		ivug_listpopup_item_append(popup, NULL, IDS_LOCK_SCREEN, IDS_LOCK_SCREEN);
		ivug_listpopup_item_append(popup, NULL, IDS_HOME_AND_LOCK_SCREENS, IDS_HOME_AND_LOCK_SCREENS);
		ivug_listpopup_item_append(popup, NULL, IDS_CALLER_IMAGE, IDS_CALLER_IMAGE);

	}

	Evas_Coord x, y;
	Evas_Coord w, h;

	Ivug_ListPopup_Item *item = (Ivug_ListPopup_Item *)event_info;
	Elm_Object_Item *eoi = ivug_listpopup_item_get_item(item);
	Evas_Object *button = elm_object_item_widget_get(eoi);
	evas_object_geometry_get(button, &x, &y, &w, &h);

	MSG_MAIN_MED("x = %d, y = %d, w = %d, h = %d", x, y, w, h);


	ivug_listpopup_context_show(popup, pMainView->area_obj, x+w/2, y+h/2);
	IVUG_FUNC_LEAVE();
	return;
}

static void _on_name_view_destroy(void *data, Evas_Object *obj, void *event_info)
{
	MSG_MAIN_HIGH("transition finished");
	Ivug_MainView *pMainView = (Ivug_MainView *)data;

	ivug_name_view_destroy(pMainView->pNameView);
	pMainView->pNameView = NULL;	// Will removed in add tag view.

	evas_object_smart_callback_del(pMainView->navi_bar, "transition,finished",
				_on_name_view_destroy);
}

static bool _rename(Media_Data *mdata, const char *str)
{
	IV_ASSERT(mdata!=NULL);

	if(mdata->filepath == NULL || mdata->thumbnail_path == NULL)
	{
		MSG_MAIN_ERROR("filepath is NULL");
		return false;
	}

	const char *old_fullpath = mdata->filepath;
	char *old_dir = ecore_file_dir_get(old_fullpath);
	char *ext = ivug_fileinfo_get_file_extension(old_fullpath);

	char new_fullpath[IVUG_MAX_FILE_PATH_LEN] = {0,};
	if(ext)
	{
		snprintf(new_fullpath, sizeof(new_fullpath),"%s/%s.%s", old_dir, str, ext);
	}
	else
	{
		snprintf(new_fullpath, sizeof(new_fullpath),"%s/%s", old_dir, str);
	}

	bool ret = ivug_rename_file(old_fullpath, new_fullpath);
	if(ret == false)
	{
		MSG_MAIN_ERROR("ivug_rename_file to %s failed", new_fullpath);
		return false;
	}

	if(ivug_db_rename(mdata->m_handle, new_fullpath) == false)
	{
		MSG_MAIN_ERROR("ivug_db_rename to %s failed", new_fullpath);
		return false;
	}

	ivug_db_destroy_file_handle(mdata->m_handle);
	mdata->m_handle = ivug_db_get_file_handle_from_media_id(mdata->mediaID);

	MSG_DETAIL_HIGH("Rename %s -> %s", old_fullpath, new_fullpath);
	free(mdata->filepath);
	mdata->filepath = strdup(new_fullpath);

	free(mdata->thumbnail_path);
	mdata->thumbnail_path = ivug_db_get_thumbnail_path(mdata->m_handle);

	return true;
}

static void _on_rename_view_response(ivug_name_response resp, const char *str, void *pClientData)
{
	Ivug_MainView *pMainView = (Ivug_MainView *)pClientData;

	if(resp != NAME_VIEW_RESPONSE_OK)
	{
		MSG_MAIN_ERROR("rename is canceled");

		evas_object_smart_callback_add(pMainView->navi_bar, "transition,finished",
				_on_name_view_destroy, pMainView);

		elm_naviframe_item_pop(pMainView->navi_bar);

		pMainView->navi_it = elm_naviframe_top_item_get(pMainView->navi_bar);

		Evas_Object *conformant = (Evas_Object *)ug_get_conformant();
		elm_object_signal_emit(conformant, "elm,state,indicator,overlap", "");
		return;
	}

	Ivug_DetailsView *pDetailsView = pMainView->pDetailsView;

	Media_Item *mitem = ivug_slider_get_item(pMainView->slider);
	Media_Data *mData = ivug_medialist_get_data(mitem);

	if(_rename(mData, str) == false)
	{
		MSG_MAIN_ERROR("_rename to %s failed", str);
		//ivug_notify_timeout_create(pMainView->layout, IDS_FAILED, 0.0, NULL, NULL); //it does not work
		ivug_selectioninfo_create(pMainView->layout, IDS_FAILED);
	}
	else
	{
		if(*(pDetailsView->filename) == NULL
			|| *(pDetailsView->path) == NULL)
		{
			MSG_MAIN_ERROR("Detail view data is NULL");
			return;
		}

		ivug_details_view_update(pDetailsView, mData->filepath);

		//update main view
		if(pMainView->view_by == IVUG_VIEW_BY_FILE)
		{
			const char *title = ecore_file_file_get(mData->filepath);
			elm_object_item_part_text_set(pMainView->navi_it, "elm.text.title", title);
		}

		ivug_slider_set_item(pMainView->slider, mitem, SLIDE_INSTANTLY);
	}

	evas_object_smart_callback_add(pMainView->navi_bar, "transition,finished",
			_on_name_view_destroy, pMainView);

	elm_naviframe_item_pop(pMainView->navi_bar);

	pMainView->navi_it = elm_naviframe_top_item_get(pMainView->navi_bar);

	Evas_Object *conformant = (Evas_Object *)ug_get_conformant();
	elm_object_signal_emit(conformant, "elm,state,indicator,overlap", "");
}


static void
_ivug_detail_view_rename_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
	MSG_MAIN_HIGH("ENTER : Btn rename clicked. obj=0x%08x data=0x%08x", obj, data);
	ivug_ret_if(!obj||!data);

	Ivug_MainView *pMainView = (Ivug_MainView *)data;

	Media_Item *mitem = ivug_slider_get_item(pMainView->slider);
	Media_Data *mdata = ivug_medialist_get_data(mitem);

	pMainView->pNameView = ivug_name_view_create(pMainView->layout, IDS_CHANGE_NAME);

	char *name = ecore_file_strip_ext(ecore_file_file_get(mdata->filepath));

	ivug_name_view_set_entry(pMainView->pNameView, name);

	// file name have to be smaller then MAX_BYTE_LEN include extension + '.'
	int limit_len = MAX_BYTE_LEN;

	char *ext = ivug_fileinfo_get_file_extension(mdata->filepath);
	if(ext)
	{
		limit_len -= (strlen(ext) + 1); // 1 is '.'
	}

	ivug_name_view_set_max_length(pMainView->pNameView, limit_len);

	ivug_name_view_set_response_callback(pMainView->pNameView, _on_rename_view_response, (void*)pMainView);

	if(ext)
	{
		free(ext);
	}

	free(name);

	Evas_Object *layout = ivug_name_view_object_get(pMainView->pNameView);

	pMainView->navi_it = elm_naviframe_item_push(pMainView->navi_bar, NULL, NULL, NULL, layout, NULL);

	Evas_Object *conformant = (Evas_Object *)ug_get_conformant();
	elm_object_signal_emit(conformant, "elm,state,indicator,nooverlap", "");

	elm_naviframe_item_title_visible_set(pMainView->navi_it, EINA_FALSE);
	elm_object_item_signal_emit(pMainView->navi_it, "elm,state,toolbar,close", "");

	ivug_name_view_set_focus(pMainView->pNameView);
}


void on_btn_details_clicked(void *data, Evas_Object *obj, void *event_info)
{
	MSG_MAIN_HIGH("ENTER : Btn detail view clicked. obj=0x%08x data=0x%08x", obj, data);
	ivug_ret_if(!obj||!data);

	Ivug_MainView *pMainView = (Ivug_MainView *)data;

	if(pMainView->pDetailsView)
	{
		MSG_MAIN_ERROR("details view already created");
		return;
	}
	if(pMainView->bShowMenu == false)
	{
		MSG_MAIN_ERROR("details view is hiding now");
		return;
	}

	ivug_main_view_del_hide_timer(pMainView);

	Media_Item *mitem = ivug_slider_get_item(pMainView->slider);
	Media_Data *mdata = ivug_medialist_get_data(mitem);
	IV_ASSERT(mdata != NULL);

	pMainView->pDetailsView = ivug_details_view_screen_create(pMainView->navi_bar, mitem, pMainView->view_by);

	if(!pMainView->pDetailsView)
	{
		MSG_MAIN_ERROR("create details view failed");
		ivug_main_view_set_hide_timer(pMainView);
		return;
	}

	Evas_Object *detail_layout = ivug_details_view_object_get(pMainView->pDetailsView);

	elm_object_item_part_content_set(pMainView->navi_it, "elm.swallow.detail", detail_layout);

	evas_object_smart_callback_add(detail_layout, "rename,clicked", _ivug_detail_view_rename_clicked_cb, pMainView);

	//_change_details_to_close(pMainView);

	MSG_MAIN_HIGH("LEAVE : Btn detail view clicked. obj=0x%08x data=0x%08x", obj, data);
}



void on_btn_delete_clicked(void *data, Evas_Object *obj, void *event_info)
{
	Ivug_MainView *pMainView = (Ivug_MainView *)data;
	IV_ASSERT(pMainView != NULL);

	ivug_main_view_del_hide_timer(pMainView);

	ivug_deletepopup_show(pMainView->layout, NULL, IDS_DELETE_MSG, _on_delete_selected, pMainView);

	return;
}

bool _share_pkg_cb(service_h service, const char *package, void *user_data)
{
	Evas_Object* popup = (Evas_Object*)user_data;

	char *name = NULL;

	app_manager_get_app_name(package, &name);

	ivug_listpopup_item_append(popup, NULL, name, (void *)strdup(package));

	free(name);

	return true;
}

#ifdef USE_DEFAULT_APP_SERVICE
void on_btn_share_clicked(void *data, Evas_Object *obj, void *event_info)
{
	Ivug_MainView *pMainView = (Ivug_MainView *)data;
	IV_ASSERT(pMainView != NULL);

	Media_Item *mitem = ivug_slider_get_item(pMainView->slider);
	Media_Data *mdata = ivug_medialist_get_data(mitem);
	IV_ASSERT(mdata != NULL);

	if ( mdata->filepath == NULL)
	{
		MSG_MAIN_ERROR("File path is NULL");
		return;
	}

	ivug_main_view_del_hide_timer(pMainView);

	Evas_Object* popup = NULL;
	popup = ivug_listpopup_add(pMainView->area_obj);

	pMainView->ctx_popup = popup;

	evas_object_smart_callback_add(popup, "popup,dismissed", _dismissed_cb, pMainView);
	evas_object_smart_callback_add(popup, "popup,selected", _on_share_selected, pMainView);

	service_h service;
	service_create(&service);
	service_set_operation(service, SERVICE_OPERATION_SEND);
	service_set_uri(service, mdata->filepath);
	service_foreach_app_matched(service, _share_pkg_cb, (void *)popup);

	ivug_listpopup_button_set(popup, IDS_CLOSE);
	ivug_listpopup_popup_show(popup);
}
#else
void on_btn_share_clicked(void *data, Evas_Object *obj, void *event_info)
{
	Ivug_MainView *pMainView = (Ivug_MainView *)data;
	IV_ASSERT(pMainView != NULL);

	Media_Item *mitem = ivug_slider_get_item(pMainView->slider);
	Media_Data *mdata = ivug_medialist_get_data(mitem);
	IV_ASSERT(mdata != NULL);

	if ( mdata->filepath == NULL)
	{
		MSG_MAIN_ERROR("File path is NULL");
		return;
	}

	ivug_main_view_del_hide_timer(pMainView);

	Evas_Object* popup = NULL;
	popup = ivug_listpopup_add(pMainView->area_obj);

	pMainView->ctx_popup = popup;

	evas_object_smart_callback_add(popup, "popup,dismissed", _dismissed_cb, pMainView);
	evas_object_smart_callback_add(popup, "popup,selected", _on_share_selected, pMainView);

	ivug_listpopup_item_append(popup, NULL, IDS_MESSAGE, IDS_MESSAGE);
	ivug_listpopup_item_append(popup, NULL, IDS_EMAIL, IDS_EMAIL);

	switch(mdata->slide_type)
	{
		case SLIDE_TYPE_IMAGE:
			ivug_listpopup_item_append(popup, NULL, IDS_WIFI_DIRECT, IDS_WIFI_DIRECT);
			ivug_listpopup_item_append(popup, NULL, IDS_BLUETOOTH, IDS_BLUETOOTH);

			if(mdata->bIsDRMContent == EINA_TRUE)
			{
				break;
			}

			ivug_listpopup_item_append(popup, NULL, IDS_FACEBOOK, IDS_FACEBOOK);
			ivug_listpopup_item_append(popup, NULL, IDS_PICASA, IDS_PICASA);

			break;
		case SLIDE_TYPE_VIDEO:
			ivug_listpopup_item_append(popup, NULL, IDS_WIFI_DIRECT, IDS_WIFI_DIRECT);
			ivug_listpopup_item_append(popup, NULL, IDS_BLUETOOTH, IDS_BLUETOOTH);

			if(mdata->bIsDRMContent == EINA_TRUE)
			{
				break;
			}
			ivug_listpopup_item_append(popup, NULL, IDS_FACEBOOK, IDS_FACEBOOK);
			ivug_listpopup_item_append(popup, NULL, IDS_YOUTUBE, IDS_YOUTUBE);

			break;
		case SLIDE_TYPE_UNKNOWN:
			ivug_listpopup_item_append(popup, NULL, IDS_BLUETOOTH, IDS_BLUETOOTH);
			break;
		case SLIDE_TYPE_NONE:
		default:
			MSG_MAIN_WARN("Unknown slide type=%d", mdata->slide_type);
			// Go through
	}

	ivug_listpopup_button_set(popup, IDS_CLOSE);
	ivug_listpopup_popup_show(popup);
}
#endif

void on_btn_save_clicked(void *data, Evas_Object *obj, void *event_info)
{
	Ivug_MainView *pMainView = (Ivug_MainView *)data;
	IV_ASSERT(pMainView != NULL);

	if(pMainView->popup)
	{
		MSG_MAIN_ERROR("popup already exist");
		return;
	}

	Media_Item *mitem = ivug_slider_get_item(pMainView->slider);
	Media_Data *mdata = ivug_medialist_get_data(mitem);
	IV_ASSERT(mdata != NULL);

	if ( mdata->filepath == NULL)
	{
		MSG_MAIN_ERROR("File path is NULL");
		return;
	}

	ivug_main_view_del_hide_timer(pMainView);

	MSG_MAIN_HIGH("Web image filepath %s, fileurl %s", mdata->filepath, mdata->fileurl);

#ifdef USE_DEFAULT_DOWNLOADS_FOLDER
	char dest_file[IVUG_MAX_FILE_PATH_LEN + 1] = {0,};
	const char *new_filename = ecore_file_file_get(mdata->filepath);
	char *temp_filename = NULL;

	if (new_filename == NULL)
	{
		MSG_MAIN_ERROR("File does not exist filepath=%s", mdata->filepath);
		ivug_main_view_set_hide_timer(pMainView);
		return;
	}

	if (ecore_file_dir_is_empty(DEFAULT_DOWNLOADS_FOLDER) == -1)
	{
		MSG_MAIN_WARN("Destination path doesn't exist. %s", DEFAULT_DOWNLOADS_FOLDER);
		mkdir(DEFAULT_DOWNLOADS_FOLDER, DIR_MASK_DEFAULT);
	}

	snprintf(dest_file, IVUG_MAX_FILE_PATH_LEN, "%s/%s", DEFAULT_DOWNLOADS_FOLDER, new_filename);
	if(ecore_file_exists(dest_file))
	{
		MSG_MAIN_WARN("file already exist");
		temp_filename = ivug_generate_file_name(dest_file, NULL, NULL);
		snprintf(dest_file, IVUG_MAX_FILE_PATH_LEN, "%s/%s", DEFAULT_DOWNLOADS_FOLDER, ecore_file_file_get(temp_filename));
		free(temp_filename);
	}

	if(ivug_copy_file(mdata->filepath, dest_file) == false)
	{
		_ivug_selectioninfo_create(pMainView, IDS_FAILED);
		ivug_main_view_set_hide_timer(pMainView);
		return;
	}
	// Add to album
	/*int ret = media_file_register(dest_file);
	if(ret < 0)
	{
		MSG_MAIN_ERROR("Cannot add to album file %s. ret=%d", dest_file, ret);
		_ivug_selectioninfo_create(pMainView, IDS_FAILED);
	}
	else
	{
		_ivug_selectioninfo_create(pMainView, IDS_SAVED);
	}*/
	_ivug_selectioninfo_create(pMainView, IDS_SAVED);
#else
	ivug_listpopup_itemlist items = ivug_listpopup_itemlist_new();

// Get local album name.

	bool ret = ivug_db_get_all_folder_list(_iter_album_list, items);
	if( ret == false )
	{
		MSG_SDATA_ERROR( "ivug_db_get_all_folder_list failed: %d", ret);
		return;
	}

	pMainView->popup = ivug_listpopup_show(pMainView->layout, IDS_SAVE, items, _on_save_selected, pMainView);

	ivug_listpopup_itemlist_free(items);
#endif

	return;
}


void on_btn_tag_clicked(void *data, Evas_Object *obj, void *event_info)
{
	Ivug_MainView *pMainView = (Ivug_MainView *)data;
	IV_ASSERT(pMainView != NULL);

	Media_Item *mitem = ivug_slider_get_item(pMainView->slider);
	Media_Data *mdata = ivug_medialist_get_data(mitem);
	ivug_ret_if(!mdata);

	//ivug_main_view_del_hide_timer(pMainView);

	Evas_Object* popup = NULL;
	popup = ivug_listpopup_add(pMainView->area_obj);

	pMainView->ctx_popup2 = popup;

	evas_object_smart_callback_add(popup, "popup,dismissed", _dismissed_cb, pMainView);
	evas_object_smart_callback_add(popup, "popup,selected", _on_addtag_selected, pMainView);

	ivug_listpopup_item_append(popup, NULL, IDS_CREATE_TAG, IDS_CREATE_TAG);
	ivug_listpopup_item_append(popup, NULL, IDS_FAVOURITES, IDS_FAVOURITES);

	bool ret = ivug_db_get_all_tag_list(_iter_tag_list, pMainView);
	if( ret == false )
	{
		MSG_SDATA_ERROR( "ivug_db_get_all_folder_list failed: %d", ret);
		return;
	}

	Evas_Coord x, y;
	Evas_Coord w, h;

	Ivug_ListPopup_Item *item = (Ivug_ListPopup_Item *)event_info;
	Elm_Object_Item *eoi = ivug_listpopup_item_get_item(item);
	Evas_Object *button = elm_object_item_widget_get(eoi);
	evas_object_geometry_get(button, &x, &y, &w, &h);
	MSG_MAIN_HIGH("x = %d, y = %d, w = %d, h = %d", x, y, w, h);

	ivug_listpopup_context_show(popup, pMainView->area_obj, x+w/2, y+h/2);
	return;
}

void on_btn_slideshow_clicked(void *data, Evas_Object *obj, void *event_info)
{
	Ivug_MainView *pMainView = (Ivug_MainView *)data;
	IV_ASSERT(pMainView != NULL);

	MSG_MAIN_HIGH("Start slide show");

/*
	Just for debugging.
*/
#if 0
	ivug_slider_dump_photocam(pMainView->slider);
	return;

#endif

	if(pMainView->pDetailsView)
	{
		MSG_MAIN_HIGH("Exit details view");
		//_change_close_to_details(pMainView);
		_delete_details_view(pMainView);
		ivug_main_view_set_hide_timer(pMainView);	// for hide_count
	}

	ivug_main_view_start_slideshow(pMainView, EINA_TRUE);

}

void on_btn_details_close_clicked(void *data, Evas_Object *obj, void *event_info)
{
	Ivug_MainView *pMainView = (Ivug_MainView *)data;
	IV_ASSERT(pMainView != NULL);

	MSG_MAIN_HIGH("ENTER : Btn detail close clicked. obj=0x%08x data=0x%08x", obj, data);

	//_change_close_to_details(pMainView);
	_delete_details_view(pMainView);

	ivug_main_view_set_hide_timer(pMainView);

	MSG_MAIN_HIGH("LEAVE : Btn detail close clicked. obj=0x%08x data=0x%08x", obj, data);
}



/*
	Used for only view type is select
*/
void on_btn_selectok_clicked(void *data, Evas_Object *obj, void *event_info)
{
	Ivug_MainView *pMainView = (Ivug_MainView *)data;
	IV_ASSERT(pMainView != NULL);

	MSG_MAIN_HIGH("Select OK.");

	service_h service;
	service_create(&service);

	service_add_extra_data(service, "Result", "Ok");
	ug_send_result(gGetUGHandle(), service);

	service_destroy(service);

	DESTROY_ME();

}
/*
	Used for only view type is select
*/
void on_btn_selectcancel_clicked(void *data, Evas_Object *obj, void *event_info)
{
	Ivug_MainView *pMainView = (Ivug_MainView *)data;
	IV_ASSERT(pMainView != NULL);

	MSG_MAIN_HIGH("Select Cancel");

//send result to caller
	service_h service;
	service_create(&service);

	service_add_extra_data(service, "Result", "Cancel");
	ug_send_result(gGetUGHandle(), service);

	service_destroy(service);

	DESTROY_ME();

}

void on_btn_rename_clicked(void *data, Evas_Object *obj, void *event_info)
{
	Ivug_MainView *pMainView = (Ivug_MainView *)data;
	IV_ASSERT(pMainView != NULL);

	Media_Item *mitem = ivug_slider_get_item(pMainView->slider);
	Media_Data *mdata = ivug_medialist_get_data(mitem);

	IV_ASSERT(mdata != NULL);

	ivug_main_view_del_hide_timer(pMainView);

	char *old_fullpath = mdata->filepath;
	char *old_name = ecore_file_strip_ext(ecore_file_file_get(old_fullpath));

	ivug_rename_popup_show(pMainView->layout, old_name, _on_rename_selected, pMainView);

	free(old_name);
}

static void
_on_copy_selected(void *data, Evas_Object *obj)
{
	Ivug_MainView *pMainView = (Ivug_MainView *)data;
	IV_ASSERT(pMainView != NULL);

	//get current file path.
	Media_Item *mitem = ivug_slider_get_item(pMainView->slider);
	Media_Data *mdata = ivug_medialist_get_data(mitem);
	if (mdata == NULL)
	{
		MSG_MAIN_ERROR("slider data is NULL");
		return;
	}

	char *path = NULL;
	int len = 0;

	if(mdata->slide_type == SLIDE_TYPE_VIDEO)
	{
		path = mdata->thumbnail_path;
	}
	else
	{
		path = mdata->filepath;
	}
	len = strlen(path)+1;

	if (len < IVUG_MAX_FILE_PATH_LEN)
	{
		MSG_MAIN_HIGH("file path = %s", path);
		elm_cnp_selection_set(pMainView->layout, ELM_SEL_TYPE_CLIPBOARD, ELM_SEL_FORMAT_IMAGE, path, len);
		_ivug_selectioninfo_create(pMainView, IDS_COPIED_TO_CLIPBOARD);
	}
	else
	{
		MSG_MAIN_ERROR("slider file path is too long");
		_ivug_selectioninfo_create(pMainView, IDS_FAILED);
	}
}

static void
_on_crop_selected(void *data, Evas_Object *obj)
{
	Ivug_MainView *pMainView = (Ivug_MainView *)data;
	IV_ASSERT(pMainView != NULL);

	//get current file path.
	Media_Item *WMitem = ivug_slider_get_item(pMainView->slider);
	Media_Data *mdata = ivug_medialist_get_data(WMitem);

	if (mdata == NULL)
	{
		MSG_MAIN_ERROR("slider data is NULL");
		return;
	}

	MSG_MAIN_HIGH("Create Crop view");
	IvugCropView *pCropView = ivug_crop_view_create(pMainView->layout);

	Evas_Object *layout = ivug_crop_view_get_object(pCropView);

	evas_object_smart_callback_add(layout, "ok,clicked", _ivug_crop_view_ok_clicked_cb, pMainView);
	evas_object_smart_callback_add(layout, "destroyed", _ivug_crop_view_destroyed_cb, pMainView);

	Evas_Object *btn_back = ivug_button_add(layout, "naviframe/end_btn/default",
		IDS_BACK, NULL, _ivug_crop_view_destroyed_cb, pMainView);

	pMainView->navi_it = elm_naviframe_item_push(pMainView->navi_bar, NULL, btn_back, NULL, pCropView->layout, NULL);

	//Evas_Object *conformant = (Evas_Object *)ug_get_conformant();
	//elm_object_signal_emit(conformant, "elm,state,indicator,nooverlap", "");

	ivug_crop_view_create_menu(pCropView, pMainView->navi_bar);

	ivug_crop_view_file_set(pCropView, mdata->filepath);

	pMainView->pCropView = pCropView;

	elm_naviframe_item_title_visible_set(pMainView->navi_it, EINA_FALSE);
}

#include <Ecore_File.h>

static void
_on_rotate_left_selected(void *data, Evas_Object *obj)
{
	Ivug_MainView *pMainView = (Ivug_MainView *)data;
	IV_ASSERT(pMainView != NULL);

	MSG_MAIN_HIGH("Rotate l button clicked");


#ifdef IVUG_DEFINE_EXIF_ROTATE
	Media_Item *mitem = NULL;
	Media_Data *mData = NULL;

	mitem = ivug_slider_get_item(pMainView->slider);
	mData = ivug_medialist_get_data(mitem);

	int degree = 0;

	if ( ivug_exif_get_rotate(mData->filepath, &degree) < 0 )
	{
		MSG_MAIN_ERROR("Cannot get degree : %s", mData->filepath);
		return;
	}


	degree = degree - 90 ;

	if ( ivug_exif_set_rotate(mData->filepath, degree) < 0 )
	{
		MSG_MAIN_ERROR("Cannot set degree : %s", mData->filepath);
		return;
	}

	ivug_db_refresh(mData->mediaID);

	ivug_slider_reload(pMainView->slider);		// Refresh image


// Remove old thumbnail. not working
//	ecore_file_remove(mData->thumbnail_path);

// Get thumbnail.
	char new_path[IVUG_MAX_FILE_PATH_LEN];

	int ret = -1;
	ret = thumbnail_request_from_db(mData->filepath, new_path, IVUG_MAX_FILE_PATH_LEN);
	if (ret < 0)
	{
		MSG_MAIN_ERROR("thumbnail_request_from_db failed! %s", mData->filepath);
		return;
	}

#if 0
	char dest[1024];

	snprintf(dest, sizeof(dest) - 1, "/opt/media/thm_l_%d.jpg", degree);

	ecore_file_cp(new_path, dest);
#endif

	if ( mData->thumbnail_path != NULL )
	{
		free(mData->thumbnail_path);
	}

	mData->thumbnail_path = strdup(new_path);


#else
	ivug_slider_image_rotate(pMainView->slider, -90);
#endif

	ivug_slider_zoom_reset(pMainView->slider);

	pMainView->zoom_level = 1;
	elm_slider_value_set(pMainView->zoom_slider, 1.0);
}

static void
_on_rotate_right_selected(void *data, Evas_Object *obj)
{
	Ivug_MainView *pMainView = (Ivug_MainView *)data;
	IV_ASSERT(pMainView != NULL);

	MSG_MAIN_HIGH("Rotate r button clicked");


#ifdef IVUG_DEFINE_EXIF_ROTATE
	Media_Item *mitem = NULL;
	Media_Data *mData = NULL;

	mitem = ivug_slider_get_item(pMainView->slider);
	mData = ivug_medialist_get_data(mitem);

	int degree = 0;

	if ( ivug_exif_get_rotate(mData->filepath, &degree) < 0 )
	{
		MSG_MAIN_ERROR("Cannot get degree : %s", mData->filepath);
		return;
	}


	degree = degree + 90 ;

	if ( ivug_exif_set_rotate(mData->filepath, degree) < 0 )
	{
		MSG_MAIN_ERROR("Cannot set degree : %s", mData->filepath);
		return;
	}

	ivug_db_refresh(mData->mediaID);

	ivug_slider_reload(pMainView->slider);		// Refresh image


// Remove old thumbnail. not working
//	ecore_file_remove(mData->thumbnail_path);

// Get thumbnail.
	char new_path[IVUG_MAX_FILE_PATH_LEN];

	int ret = -1;
	ret = thumbnail_request_from_db(mData->filepath, new_path,
					IVUG_MAX_FILE_PATH_LEN);
	if (ret < 0)
	{
		MSG_MAIN_ERROR("thumbnail_request_from_db failed! %s", mData->filepath);
		return;
	}

#if 0
	char dest[1024];

	snprintf(dest, sizeof(dest) - 1, "/opt/media/thm_r_%d.jpg", degree);

	ecore_file_cp(new_path, dest);
#endif

	if ( mData->thumbnail_path != NULL )
	{
		free(mData->thumbnail_path);
	}

	mData->thumbnail_path = strdup(new_path);


#else
	ivug_slider_image_rotate(pMainView->slider, +90);
#endif

	ivug_slider_zoom_reset(pMainView->slider);

	pMainView->zoom_level = 1;
	elm_slider_value_set(pMainView->zoom_slider, 1.0);
}


static void _on_more_selected(void *data, Evas_Object *obj, void *event_info)
{
// obj is ctxpopup!.
	Ivug_MainView *pMainView = (Ivug_MainView *)data;

	Ivug_ListPopup_Item *item = (Ivug_ListPopup_Item *)event_info;

	const char *label = (const char *)ivug_listpopup_item_get_data(item);
	if(label == NULL)
	{
		ivug_main_view_set_hide_timer(pMainView);
		return;
	}

	MSG_MAIN_HIGH("text(%s) is clicked", label);

	if(strncmp(label, IDS_COPY_TO_CLIPBOARD, strlen(label)) == 0)
	{
		_on_copy_selected(data, obj);
		ivug_main_view_set_hide_timer(pMainView);
	}
	else if(strncmp(label, IDS_CROP, strlen(label)) == 0)
	{
		_on_crop_selected(data, obj);
	}
	else if(strncmp(label, IDS_ROTATE_LEFT, strlen(label)) == 0)
	{
		_on_rotate_left_selected(data, obj);
		ivug_main_view_set_hide_timer(pMainView);
	}
	else if(strncmp(label, IDS_ROTATE_RIGHT, strlen(label)) == 0)
	{
		_on_rotate_right_selected(data, obj);
		ivug_main_view_set_hide_timer(pMainView);
	}
	else if(strncmp(label, IDS_SET_AS, strlen(label)) == 0)
	{
		on_btn_setas_clicked(data, obj, event_info);
	}
	else if(strncmp(label, IDS_SLIDE_SHOW, strlen(label)) == 0)
	{
		on_btn_slideshow_clicked(data, obj, event_info);
	}
	else if(strncmp(label, IDS_DETAILS, strlen(label)) == 0)
	{
		on_btn_details_clicked(data, obj, event_info);
	}
	else if(strncmp(label, IDS_ADD_TAG, strlen(label)) == 0)
	{
		on_btn_tag_clicked(data, obj, event_info);
	}
	else if(strncmp(label, IDS_PRINT, strlen(label)) == 0)
	{
		Media_Item *WMitem = ivug_slider_get_item(pMainView->slider);
		Media_Data *mdata = ivug_medialist_get_data(WMitem);

		if (mdata == NULL)
		{
			MSG_MAIN_ERROR("slider data is NULL");
			evas_object_del(pMainView->ctx_popup);
			pMainView->ctx_popup = NULL;
			ivug_main_view_set_hide_timer(pMainView);
			return;
		}
		ivug_ext_launch_print(mdata->filepath);
	}

	evas_object_del(pMainView->ctx_popup);
	pMainView->ctx_popup = NULL;
}

void on_btn_more_clicked(void *data, Evas_Object *obj, void *event_info)
{
	Ivug_MainView *pMainView = (Ivug_MainView *)data;
	IV_ASSERT(pMainView != NULL);

	slide_state_t state = ivug_slider_get_state(pMainView->slider);

	Media_Item *mitem = ivug_slider_get_item(pMainView->slider);
	Media_Data *mdata = ivug_medialist_get_data(mitem);

	IV_ASSERT(mdata != NULL);

	ivug_main_view_del_hide_timer(pMainView);

	Evas_Object* popup = NULL;
	popup = ivug_listpopup_add(pMainView->area_obj);

	pMainView->ctx_popup = popup;

	evas_object_smart_callback_add(popup, "popup,dismissed", _dismissed_cb, pMainView);
	evas_object_smart_callback_add(popup, "popup,selected", _on_more_selected, pMainView);

	if (state == SLIDE_STATE_ERROR || state == SLIDE_STATE_DOWNLOAD_FAILED)
	{
		MSG_MAIN_HIGH("slide state is invalid %d", state);
		ivug_listpopup_item_append(popup, NULL, IDS_DETAILS, IDS_DETAILS);
		goto MORE_BTN_END;
	}

	if(mdata->slide_type == SLIDE_TYPE_IMAGE)
	{
		ivug_listpopup_item_append(popup, NULL, IDS_SET_AS, IDS_SET_AS);
		ivug_listpopup_item_append(popup, NULL, IDS_SLIDE_SHOW, IDS_SLIDE_SHOW);
		if(pMainView->pDetailsView == NULL)
			ivug_listpopup_item_append(popup, NULL, IDS_DETAILS, IDS_DETAILS);
		ivug_listpopup_item_append(popup, NULL, IDS_ADD_TAG, IDS_ADD_TAG);

		if(mdata->bIsDRMContent == EINA_FALSE)
		{
			ivug_listpopup_item_append(popup, NULL, IDS_CROP, IDS_CROP);

			int degree = 0;
			if ( ivug_exif_get_rotate(mdata->filepath, &degree) == 0 )
			{
				ivug_listpopup_item_append(popup, NULL, IDS_ROTATE_LEFT, IDS_ROTATE_LEFT);
				ivug_listpopup_item_append(popup, NULL, IDS_ROTATE_RIGHT, IDS_ROTATE_RIGHT);
			}
		}

		ivug_listpopup_item_append(popup, NULL, IDS_PRINT, IDS_PRINT);
	}
	else if(mdata->slide_type == SLIDE_TYPE_VIDEO)
	{
		if(ivug_is_editable_video_file(mdata->filepath) == true)
		{
			ivug_listpopup_item_append(popup, NULL, IDS_TRIM, IDS_TRIM);
		}
		ivug_listpopup_item_append(popup, NULL, IDS_SLIDE_SHOW, IDS_SLIDE_SHOW);
		if(pMainView->pDetailsView == NULL)
			ivug_listpopup_item_append(popup, NULL, IDS_DETAILS, IDS_DETAILS);
		ivug_listpopup_item_append(popup, NULL, IDS_ADD_TAG, IDS_ADD_TAG);
		//ivug_listpopup_item_append(popup, NULL, IDS_CROP, IDS_CROP);
	}

MORE_BTN_END:
	Evas_Coord x, y;
	Evas_Coord w, h;

	evas_object_geometry_get(obj, &x, &y, &w, &h);
	MSG_MAIN_MED("x = %d, y = %d, w = %d, h = %d", x, y, w, h);

	ivug_listpopup_context_show(popup, pMainView->area_obj, x+w/2, y+h/2);
}

