/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "ivug-common.h"
#include "ivug-main-view.h"
#include "ivug-main-view-menu.h"
#include "ivug-main-view-priv.h"

#include "ivug-slider.h"

#define MIME_TYPE_LEN			(255)
#define MIME_TYPE_3GPP			"video/3gpp"
#define PATH_CAMERA_LOCAL		"/opt/usr/media/Camera"
#define PATH_CAMERA_SDCARD		"/opt/storage/sdcard/Camera"

/* Video editor can start when video exist in camera folder and 3gp file format */

bool ivug_is_editable_video_file(char *filepath)
{
	MSG_UTIL_MED("path = %s", filepath);
	if(strncmp(filepath, PATH_CAMERA_LOCAL, strlen(PATH_CAMERA_LOCAL)) == 0
		|| strncmp(filepath, PATH_CAMERA_SDCARD, strlen(PATH_CAMERA_SDCARD)) == 0)
	{
		efreet_mime_init();
		const char *type = NULL;
		type = efreet_mime_type_get(ivug_get_filename(filepath));

		efreet_mime_shutdown();

		if ( type == NULL )
		{
			MSG_UTIL_ERROR("Cannot get mime type for %s", filepath);
			return false;
		}

		MSG_UTIL_MED("mime type = %s", type);
		if(strcmp(type, MIME_TYPE_3GPP) == 0)
		{
			return true;
		}
	}
	return false;
}

ivug_toolbar
_get_tool_bar_type(ivug_mode mode, Media_Type slide_type )
{
	ivug_toolbar tool_bar_type = NAVI_BAR_TYPE_NONE;

	//calc navi bar style , temporary code.
	static  const char *szType[] = {
		"[ INVALID  ]",
		"[          ]",
		"[Set,  Info]",
		"[Ok, Cancel]",
	};


	switch (mode)
	{
		case IVUG_MODE_NORMAL:
		case IVUG_MODE_CAMERA:
			tool_bar_type = NAVI_BAR_TYPE_EMPTY;
			if (slide_type == SLIDE_TYPE_IMAGE)
			{
				tool_bar_type = NAVI_BAR_TYPE_IMAGE;
			}
			else if (slide_type == SLIDE_TYPE_VIDEO)
			{
				tool_bar_type = NAVI_BAR_TYPE_VIDEO;
			}
			else
			{
				tool_bar_type = NAVI_BAR_TYPE_EMPTY;
			}
		break;
		case IVUG_MODE_SINGLE:
		case IVUG_MODE_FILE:
			{
				tool_bar_type = NAVI_BAR_TYPE_FILE;
			}

		break;
		case IVUG_MODE_DISPLAY:
			tool_bar_type = NAVI_BAR_TYPE_SELECT;

		break;
		default:
			MSG_MAIN_ERROR("Unhandled mode : %d", mode);
			tool_bar_type = NAVI_BAR_TYPE_EMPTY;

			break;
	}

	MSG_MAIN_MED("Mode=%d Slide=%d ToolBar=%s", mode, slide_type, szType[tool_bar_type]);

	return tool_bar_type;
}


ivug_ctrlbar
_get_ctrl_bar_type(ivug_mode mode, Media_Type slide_type)
{
	// this is temporary code.
	ivug_ctrlbar type = CTRL_BAR_TYPE_FILE;

	static const char *szType[] = {
		"[ Invalid ]",
		"[ None ]",
		"[Delete,Share,Set,Info]",
		"[Delete,Share,Set,Info, Editor]",
		"[Delete,Share,Set,Info, Trim]",
		"[Share, Set, Save, Info]",
		"[Delete,Share, Add tag, Slide show, Editor]",
		"[Delete,Share, Add tag, Slide show, Trim]",
	};

	switch (mode)
	{
		case IVUG_MODE_NORMAL:
		case IVUG_MODE_CAMERA:
			if (slide_type == SLIDE_TYPE_IMAGE)
			{
				type = CTRL_BAR_TYPE_IMAGE;
			}
			else if (slide_type == SLIDE_TYPE_VIDEO)
			{
				type = CTRL_BAR_TYPE_VIDEO;
			}
			else if (slide_type == SLIDE_TYPE_UNKNOWN)
			{
				type = CTRL_BAR_TYPE_IMAGE;
			}
			else
			{
				MSG_MAIN_ERROR("Unhandled slide type : %d", slide_type);
				type = CTRL_BAR_TYPE_FILE;
			}
		break;
		case IVUG_MODE_SINGLE:
		case IVUG_MODE_FILE:
			{
				type = CTRL_BAR_TYPE_FILE;
			}
		break;
		case IVUG_MODE_DISPLAY:
			type = CTRL_BAR_TYPE_EMPTY;
		break;
		case IVUG_MODE_SETAS:
			type = CTRL_BAR_TYPE_EMPTY;
		break;
		default:
			MSG_MAIN_ERROR("Unhandled mode : %d", mode);
			type = CTRL_BAR_TYPE_FILE;
	}

	MSG_MAIN_MED("Mode=%d Slide=%d CtrlBar=%s", mode, slide_type, szType[type]);

	return type;
}

void _create_main_menu(Ivug_MainView *pMainView)
{
	if(pMainView->mode == IVUG_MODE_DISPLAY)
	{
		Evas_Object *btn_save = ivug_button_add(pMainView->navi_bar, "naviframe/toolbar/default",
									IDS_SAVE, NULL, on_btn_selectok_clicked, pMainView);
		elm_object_item_part_content_set(pMainView->navi_it, "toolbar_button1", btn_save);
	}
	else
	{
		pMainView->btn_more = ivug_button_add(pMainView->navi_bar, "naviframe/more/default",
									NULL, NULL, on_btn_more_clicked, pMainView);
		elm_object_item_part_content_set(pMainView->navi_it, "toolbar_more_btn", pMainView->btn_more);

		pMainView->btn_share = ivug_button_add(pMainView->navi_bar, "naviframe/toolbar/left",
									IDS_SHARE, NULL, on_btn_share_clicked, pMainView);
		elm_object_item_part_content_set(pMainView->navi_it, "toolbar_button1", pMainView->btn_share);

		Evas_Object *btn_del = ivug_button_add(pMainView->navi_bar, "naviframe/toolbar/right",
									IDS_DELETE, NULL, on_btn_delete_clicked, pMainView);
		elm_object_item_part_content_set(pMainView->navi_it, "toolbar_button2", btn_del);
	}
}

