/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "ivug-common.h"
#include "ivug-parameter.h"

#include "ivug-slideshow-view.h"

#include <vconf.h>
#include <vconf-keys.h>
#include <utilX.h>			// Key event
#include <ui-gadget-module.h>	// ug_send_result

#include "ivug-filter.h"

#include "ivug-popup.h"

#include "EFLUtil.h"
#undef LOG_LVL
#define LOG_LVL DBG_MSG_LVL_MED

#undef LOG_CAT
#define LOG_CAT "IV-SLIDESHOW"


#define IVUG_SS_EDJ 	EDJ_PATH"/ivug-slideshow-view.edj"


static void
_on_slideshow_finished(void *data, Evas_Object *obj, void *event_info);

static void
_send_result(ui_gadget_h ug, const char *key1, const char *val1, const char *key2, const char *val2)
{
	ivug_ret_if(!ug);

	service_h service;
	service_create(&service);

	if(key1 && val1)
	{
		MSG_HIGH("Bundle 1 : [%s = %s]", key1, val1);
		service_add_extra_data(service, key1, val1);
	}
	if(key2 && val2)
	{
		MSG_HIGH("Bundle 2 : [%s = %s]", key2, val2);
		service_add_extra_data(service, key2, val2);
	}
	ug_send_result(gGetUGHandle(), service);

	service_destroy(service);
}


static bool _destory_slideshow_and_ug(Ivug_SlideShowView *pSSView,
	int state,
	bool bMmc_out)
{
	IV_ASSERT(pSSView != NULL);
	evas_object_smart_callback_del_full(ivug_ss_object_get(pSSView->ssHandle),
		"slideshow,finished", _on_slideshow_finished, pSSView);

	ivug_allow_lcd_off();
	/* send msg to caller */
	if(state == SLIDE_SHOW_STOPPED)
	{
		_send_result(gGetUGHandle(), "EXIT", "NORMAL", NULL, NULL);
	}
	/*from gallery ablum*/

	if (pSSView->ssHandle )
	{
		MSG_HIGH("image viewer end cause slide show ended");
		ivug_ss_delete(pSSView->ssHandle);
	}
	pSSView->ssHandle = NULL;

	DESTROY_ME();
	return true;
}

static Eina_Bool __stop_slideshow_cb(void *data)
{
	Ivug_SlideShowView *pSSView = (Ivug_SlideShowView *)data;
	if (pSSView && (pSSView->ssHandle)) {
		if(pSSView->homekey_idler)
		{
			ecore_idler_del(pSSView->homekey_idler);
			pSSView->homekey_idler = NULL;
		}
		ivug_ss_stop(pSSView->ssHandle);
	}
	return ECORE_CALLBACK_CANCEL;
}

static Eina_Bool _on_key_down(void *user_data, int type, void *event)
{
	if (!user_data) {
		MSG_IMAGEVIEW_ERROR("user data is NULL");
		return ECORE_CALLBACK_PASS_ON;
	}

	Ivug_SlideShowView *pSSView = static_cast<Ivug_SlideShowView *>(user_data);

	Ecore_Event_Key *key_event = (Ecore_Event_Key *) event;

	if (!strcmp(key_event->keyname, KEY_END))
	{
		MSG_IMAGEVIEW_HIGH("Bakc(End) key");
	}
	else if (!strcmp(key_event->keyname, KEY_SELECT))
	{
		MSG_IMAGEVIEW_HIGH("Home key");
		/*home key,we need to stop slideshow also*/
		if (pSSView->homekey_idler == NULL) {
			pSSView->homekey_idler = ecore_idler_add(__stop_slideshow_cb, pSSView);
		}
	}
	else if (!strcmp(key_event->keyname, KEY_POWER))
	{
		MSG_IMAGEVIEW_HIGH("Power key");
		if ( pSSView->ssHandle )
		{
			ivug_ss_stop(pSSView->ssHandle);
		}
	}

	MSG_IMAGEVIEW_LOW("Key down : %s", key_event->keyname);

	return ECORE_CALLBACK_PASS_ON;
}

static void
_on_slideshow_finished(void *data, Evas_Object *obj, void *event_info)
{
	MSG_HIGH("_on_slideshow_finished");
	IV_ASSERT(data != NULL);

	Ivug_SlideShowView *pSSView = static_cast<Ivug_SlideShowView *>(data);

	int ss_state = (int)event_info;
	bool bDestoryed = false;
	bDestoryed = _destory_slideshow_and_ug(pSSView, ss_state, false);
}

static void _on_resize(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	Ivug_SlideShowView *pSSView = static_cast<Ivug_SlideShowView *>(data);
	IV_ASSERT(pSSView != NULL);

	Evas_Coord x,y,w,h;
	evas_object_geometry_get(obj, &x, &y, &w, &h);

/**
*  during slideshow, after rotation, resize cb of slideshow obj cannot be invoked,
    so invoke it manually
*/
	if(pSSView->ssHandle)
		ivug_ss_resize(pSSView->ssHandle);

	MSG_HIGH("SlideshowView resized geomtery XYWH(%d,%d,%d,%d) Rotate=%d", x, y, w, h, gGetRotationDegree());
}

void _ivug_slideshow_view_on_mmc_state_changed(keynode_t* node, void *data)
{
	Ivug_SlideShowView *pSSView = static_cast<Ivug_SlideShowView *>(data);
	IV_ASSERT(pSSView != NULL);

	//ivug_timeout_popup_show(ivug_ss_object_get(pSSView->ssHandle), NULL, NULL, IDS_ERROR, IDS_SD_CARD_REMOVED);
	ivug_ss_set_stop(pSSView->ssHandle);
	_destory_slideshow_and_ug(pSSView, SLIDE_SHOW_STOPPED, true);
}

static char * _ivug_get_folder_name(char *filepath)
{
	char *folder = ecore_file_dir_get(filepath);

	char *name = NULL;

	name = strrchr(folder, '/');
	if ( (name != NULL) && ((name+1) != NULL) )
	{
		return strdup(name + 1);
	}

	return NULL;
}


bool
ivug_slideshow_view_set_list(Ivug_SlideShowView *pSSView, ivug_parameter *ivug_param)
{
	MSG_HIGH("Load media list.");

	IV_ASSERT(pSSView != NULL);

	PERF_CHECK_BEGIN(LVL2, "create media list");

	Media_List *mlist = ivug_medialist_create();
	if (mlist == NULL)
	{
		MSG_ERROR("Creating media list failed");
		return false;
	}
	PERF_CHECK_END(LVL2, "create media list");

	PERF_CHECK_BEGIN(LVL2, "create filter");

	Filter_struct *filter = ivug_param_create_filter(ivug_param);
	if (filter == NULL)
	{
		MSG_ERROR("Creating filter failed");
		ivug_medialist_del(mlist);
		return false;
	}

	PERF_CHECK_END(LVL2, "create filter");

	Media_Item *current = NULL;
	Media_Data *pData = NULL;

	{
		PERF_CHECK_BEGIN(LVL2, "media list load");
		current = ivug_medialist_load(mlist, filter);
		PERF_CHECK_END(LVL2, "media list load");
	}

	if(current == NULL)
	{
		MSG_ERROR("Media list load failed");
		goto LOAD_LIST_FAILED;
	}

	pData = ivug_medialist_get_data(current);
	if(pData == NULL)
	{
		MSG_ERROR("current data is NULL");
		goto LOAD_LIST_FAILED;
	}

	if(pData->fileurl == NULL)
	{
		MSG_ERROR("current fileurl is NULL");
		goto LOAD_LIST_FAILED;
	}

#ifdef USE_RESCAN_FILE_PATH_AT_LIST
	if(strncmp(pData->fileurl, ivug_param->filepath, strlen(pData->fileurl)) != 0)
	{
		current = ivug_medialist_find_item_by_filename(mlist, ivug_param->filepath);
		if(current == NULL)
		{
			MSG_ERROR("Media list load failed, %s is not exist at list", ivug_param->filepath);
			goto LOAD_LIST_FAILED;
		}
	}
#endif

	if(ivug_param->view_by == IVUG_VIEW_BY_ALL)
	{
		pSSView->album_name = strdup(IDS_ALL_ALBUMS);
	}
	else if(ivug_param->view_by == IVUG_VIEW_BY_FOLDER)
	{
		/*
		media_handle m_handle = ivug_db_get_folder_handle(ecore_file_dir_get(ivug_param->filepath));
		if(m_handle == NULL)
		{
			MSG_IVUG_FATAL("View by Folder. but media handle is NULL" );
			return NULL;
		}

		pMainView->album_name = ivug_db_get_folder_name(m_handle);
		*/
		pSSView->album_name = _ivug_get_folder_name(ivug_param->filepath);
		if(pSSView->album_name == NULL)
		{
			pSSView->album_name = strdup(IDS_NO_NAME);
		}
	}

	pSSView->mList = mlist;
	PERF_CHECK_BEGIN(LVL2, "media list set");

	pSSView->ss_curItem = current;

	PERF_CHECK_END(LVL2, "media list set");

	ivug_data_filter_delete(filter);

	return true;

LOAD_LIST_FAILED:

	if(filter)
		ivug_data_filter_delete(filter);

	if(mlist)
		ivug_medialist_del(mlist);

	pSSView->mList = NULL;
	return false;
}

Ivug_SlideShowView *
ivug_slideshow_view_create(Evas_Object* parent, ivug_mode mode, ivug_view_by view_by)
{
	IV_ASSERT(parent != NULL);

	MSG_HIGH("Creating slideshow view. Parent=0x%08x Mode=%d ViewBy=%d", parent, mode, view_by);

	PERF_CHECK_BEGIN(LVL2, "Create layout");

//create main view layout
	Ivug_SlideShowView *pSSView = (Ivug_SlideShowView *)calloc(1, sizeof(Ivug_SlideShowView) );
	IV_ASSERT(pSSView != NULL);

// Set default value
	pSSView->parent = parent;
	pSSView->view_by = view_by;

	Evas_Object *layout;
	layout = elm_layout_add(parent);
	if ( layout == NULL)	//if failed
	{
		MSG_ERROR("main layout create failed");
		free(pSSView);
		return NULL;
	}

	pSSView->layout = layout;
	evas_object_name_set(pSSView->layout, "SlideShow Layout");

#if 0
	if (elm_layout_file_set(layout, IVUG_SS_EDJ, "ssview") == EINA_FALSE)
	{
		MSG_MAIN_ERROR("edj loading fail, file=%s group=%s", IVUG_SS_EDJ, IVUG_SS_EDJ);
		evas_object_del(layout);
		free(pSSView);
		return NULL;
	}
#endif
	evas_object_event_callback_add(pSSView->layout, EVAS_CALLBACK_RESIZE, _on_resize, pSSView);

	pSSView->keydown_handler = ecore_event_handler_add(ECORE_EVENT_KEY_DOWN, _on_key_down, (void *)pSSView);

	MSG_HIGH("Main View Created");

	return pSSView;
}


Evas_Object *
ivug_slideshow_view_object_get(Ivug_SlideShowView *pSSView)
{
	IV_ASSERT(pSSView != NULL);

	return pSSView->layout;
}


void
ivug_slideshow_view_start(Ivug_SlideShowView *pSSView)
{
	IV_ASSERT(pSSView != NULL);

	MSG_HIGH("Slide show running");

	Media_Item *current = pSSView->ss_curItem;

	ivug_prohibit_lcd_off();
	//ivug_main_view_hide_menu_bar(pMainView);

	pSSView->ssHandle = ivug_ss_create(pSSView->layout);

// Register callback
	evas_object_smart_callback_add(ivug_ss_object_get(pSSView->ssHandle),  "slideshow,finished", _on_slideshow_finished, pSSView);

	elm_win_indicator_mode_set((Evas_Object *)ug_get_window(), ELM_WIN_INDICATOR_HIDE);

	ivug_ss_start(pSSView->ssHandle, current, pSSView->mList, EINA_FALSE);

}



void
ivug_slideshow_view_resume(Ivug_SlideShowView *pSSView)
{
	IV_ASSERT(pSSView != NULL);

	MSG_HIGH("Slideshow View Resumed");

	Media_Item *mitem = pSSView->ss_curItem;
	Media_Data *mdata = ivug_medialist_get_data(mitem);

	IV_ASSERT(mdata != NULL);

	if (mdata->slide_type == SLIDE_TYPE_IMAGE || mdata->slide_type == SLIDE_TYPE_VIDEO )
	{
		if(mdata->filepath && ecore_file_exists(mdata->filepath)) // Current file is not deleted
		{
			MSG_HIGH("Current filepath : %s", mdata->filepath);
		}
		else
		{
			/*if file is not exist ug terminated temporally */
			MSG_ERROR("Center file is not exist. stype=%d name=%s", mdata->slide_type, mdata->filepath);
			DESTROY_ME();
			return ;
		}
	}
	else
	{
		MSG_ERROR("Unhandled slide type : %d", mdata->slide_type);
	}
}


void
ivug_slideshow_view_pause(Ivug_SlideShowView *pSSView)
{
	IV_ASSERT(pSSView != NULL);

	if ( pSSView->ssHandle )
	{
		ivug_ss_stop(pSSView->ssHandle);
	}

	// Stop photocam If AGIF
}


void
ivug_slideshow_view_destroy(Ivug_SlideShowView *pSSView)
{
	IV_ASSERT(pSSView != NULL);

	MSG_HIGH("ENTER : SlideShow View Destroy. pMainView=0x%08x", pSSView);

	MSG_HIGH("Unregister system notifications");
	if (pSSView->layout) {
		evas_object_del(pSSView->layout);
		pSSView->layout = NULL;
	}
	if(pSSView->keydown_handler)
	{
		ecore_event_handler_del(pSSView->keydown_handler);
		pSSView->keydown_handler = NULL;
	}
	if(pSSView->homekey_idler)
	{
		ecore_idler_del(pSSView->homekey_idler);
		pSSView->homekey_idler = NULL;
	}

	if ( pSSView->mList )
	{
		MSG_MAIN_HIGH("Remove media list");
		ivug_medialist_del(pSSView->mList);		// ivug_medialist_del() is not working on destroy cb.
		pSSView->mList = NULL;
	}
	if(pSSView->album_name)
	{
		free(pSSView->album_name);
		pSSView->album_name = NULL;
	}
	EFL::dump_obj(pSSView->parent, 0);
	free(pSSView);
	pSSView = NULL;
	MSG_HIGH("LEAVE : SlideShow View Destroy.");

	return ;
}


