/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "ivug-define.h"
#include "ivug-datatypes.h"
#include "ivug-uuid.h"
#include "ivug-debug.h"
#include "ivug-mediadata.h"
#include "ivug-util.h"
#include "ivug-drm-common.h"
#include "ivug-file-info.h"

#include "ivug-db.h"

#include <media_content.h>

#define IMAGE_PATH		PREFIX"/res/images/"PACKAGE
#define DEFAULT_THUMBNAIL_PATH		IMAGE_PATH"/T01_Nocontents_broken.png"

static char *_strerror_db(int error)
{
	switch(error)
	{
	case MEDIA_CONTENT_ERROR_INVALID_PARAMETER:
		return "Invalid parameter";
	case MEDIA_CONTENT_ERROR_OUT_OF_MEMORY :
		return "Out of memory";
	case MEDIA_CONTENT_ERROR_DB_FAILED :
		return "DB operation failed";
	default:
		{
			static char buf[40];
			sprintf(buf, "Error Code=%d", error);
			return buf;
		}

	}
}

static void _ivug_mediadata_thumbnail_cb(media_handle media, const char *path, void *data)
{
	MSG_SDATA_ERROR("_ivug_mediadata_thumbnail_cb, path = %s", path);

	Media_Data *mdata = (Media_Data *)data;

	mdata->thumbnail_path = strdup(path);

	ivug_db_cancel_thumbnail(mdata->thumb_handle);

	mdata->thumb_handle = NULL;
}

Media_Data *ivug_alloc_mediadata_from_media_handle(media_handle media)
{
	IV_ASSERT(media != NULL);

	Media_Data * mdata = NULL;
	mdata = (Media_Data *) calloc(1, sizeof(Media_Data));
	if (mdata == NULL)
	{
		MSG_SDATA_ERROR("Cannot allocate memory");
		return NULL;
	}

	media_info_h item = (media_info_h)media;

	int ret = media_info_clone((media_info_h*)&(mdata->m_handle), item);
	if(ret != MEDIA_CONTENT_ERROR_NONE)
	{
		MSG_SDATA_ERROR("media_info_foreach_media_from_db is failed, err = %s", _strerror_db(ret));
		goto ALLOC_MHANDLE_ERROR;
	}

	char *uuid = NULL;

	ret = media_info_get_media_id(item, &uuid);
	if(ret != MEDIA_CONTENT_ERROR_NONE)
	{
		MSG_SDATA_ERROR("media_info_foreach_media_from_db is failed, err = %s", _strerror_db(ret));
		goto ALLOC_MHANDLE_ERROR;
	}
	mdata->mediaID = uuid_assign(uuid);
	free(uuid);

	//file url
	ret = media_info_get_file_path(item, &(mdata->fileurl));
	if(ret != MEDIA_CONTENT_ERROR_NONE)
	{
		MSG_SDATA_ERROR("media_info_foreach_media_from_db is failed, err = %s", _strerror_db(ret));
		goto ALLOC_MHANDLE_ERROR;
	}
	if (mdata->fileurl == NULL)
	{
		goto ALLOC_MHANDLE_ERROR;
	}

	ret = media_info_get_thumbnail_path(item, &(mdata->thumbnail_path));
	if(ret != MEDIA_CONTENT_ERROR_NONE)
	{
		MSG_SDATA_ERROR("media_info_foreach_media_from_db is failed, err = %s", _strerror_db(ret));
		goto ALLOC_MHANDLE_ERROR;
	}
	if(mdata->thumbnail_path == NULL)
	{
		MSG_SDATA_ERROR("thumbnail is NULL, request to DB");
		mdata->thumb_handle = ivug_db_create_thumbnail(mdata->m_handle, _ivug_mediadata_thumbnail_cb, (void *)mdata);
	}

#if 0
	// Useless codes because media_info_get_thumbnail_path() never failed.
	if(mdata->thumbnail_path == NULL)
	{
		MSG_SDATA_ERROR("Thumbnail path is NULL");
		mdata->thumbnail_path = strdup(DEFAULT_THUMBNAIL_PATH);
	}
#endif

	ret = media_info_is_drm(item, &(mdata->bIsDRMContent));
	if(ret != MEDIA_CONTENT_ERROR_NONE)
	{
		MSG_SDATA_ERROR("media_info_foreach_media_from_db is failed, err = %s", _strerror_db(ret));
		goto ALLOC_MHANDLE_ERROR;
	}

	media_content_type_e media_type = 0;
	ret = media_info_get_media_type(item, &media_type);
	if(ret != MEDIA_CONTENT_ERROR_NONE)
	{
		MSG_SDATA_ERROR("media_info_foreach_media_from_db is failed, err = %s", _strerror_db(ret));
		goto ALLOC_MHANDLE_ERROR;
	}

	if(media_type == MEDIA_CONTENT_TYPE_IMAGE)
	{
		MSG_SDATA_MED("Local image : %s. %s", uuid_getchar( mdata->mediaID ), mdata->fileurl);

		mdata->slide_type = SLIDE_TYPE_IMAGE;
		mdata->filepath = strdup(mdata->fileurl);
	}
	else if(media_type == MEDIA_CONTENT_TYPE_VIDEO)
	{
		MSG_SDATA_MED("Local video : %s. %s", uuid_getchar(mdata->mediaID), mdata->fileurl);

		mdata->slide_type = SLIDE_TYPE_VIDEO;
		mdata->filepath = strdup(mdata->fileurl);
	}

	return mdata;

ALLOC_MHANDLE_ERROR:
	if(mdata)
	{
		ivug_free_mediadata(mdata);
	}
	return NULL;
}

Media_Data *ivug_alloc_mediadata_from_filepath(const char *filepath)
{
	IV_ASSERT(filepath != NULL);

	Media_Data * mdata = NULL;

	media_handle m_handle = ivug_db_get_file_handle(filepath);
	if(m_handle)
	{
		mdata = ivug_alloc_mediadata_from_media_handle(m_handle);
		if(mdata != NULL)
		{
			MSG_SDATA_HIGH("File path founded in DB.");
			return mdata;
		}
		media_info_destroy(m_handle);
	}
	// Some file does not have media handle ex) USB host, Not inserted in DB yet

	MSG_SDATA_WARN("Cannot find in DB.");

	mdata = (Media_Data *) calloc(1, sizeof(Media_Data));
	if (mdata == NULL)
	{
		MSG_SDATA_ERROR("Cannot allocate memory");
		return NULL;
	}
	mdata->mediaID = INVALID_UUID;	//copy id WMitem

	if (ivug_drm_is_drm_file(filepath) == EINA_TRUE)
	{
		mdata->bIsDRMContent = true;
	}
	char *mime_type = NULL;
	mime_type = ivug_fileinfo_get_mime_type(filepath);
	if (mime_type == NULL)
	{
		MSG_SDATA_WARN("file path is not vaild = %s", filepath);
		mdata->slide_type = SLIDE_TYPE_UNKNOWN;
		mdata->fileurl = NULL;
		mdata->filepath = NULL;
		return mdata;
	}

	Media_Type slide_type = SLIDE_TYPE_NONE;
	//image
	if (strncmp(mime_type, "image/", strlen("image/")) == 0)
	{
		slide_type = SLIDE_TYPE_IMAGE;
	}
	else if (strncmp(mime_type, "video/", strlen("video/")) == 0)
	{
		slide_type = SLIDE_TYPE_VIDEO;
	}
	else
	{
		slide_type = SLIDE_TYPE_NONE;
	}
	MSG_SDATA_HIGH("File=%s Mime=%s", filepath, mime_type);
	free(mime_type);		//free strdup
	if (slide_type == SLIDE_TYPE_IMAGE)
	{
		mdata->slide_type = SLIDE_TYPE_IMAGE;
		mdata->fileurl = strdup(filepath);
		if (mdata->fileurl == NULL)
		{
			MSG_SDATA_ERROR("strdup return NULL");
			goto ERROR;
		}
		mdata->filepath = strdup(filepath);
		if (mdata->filepath == NULL)
		{
			MSG_SDATA_ERROR("strdup return NULL");
			free(mdata->fileurl);
			goto ERROR;
		}
	}
	else if (slide_type == SLIDE_TYPE_VIDEO)
	{
		mdata->slide_type = SLIDE_TYPE_VIDEO;
		mdata->fileurl = strdup(filepath);
		if (mdata->fileurl == NULL)
		{
			MSG_SDATA_ERROR("strdup return NULL");
			goto ERROR;
		}
		mdata->filepath = strdup(filepath);
		if (mdata->filepath == NULL)
		{
			MSG_SDATA_ERROR("strdup return NULL");
			free(mdata->fileurl);
			goto ERROR;
		}
	}
	else
	{
		MSG_SDATA_WARN("file path is not vaild = %s", filepath);
		mdata->slide_type = SLIDE_TYPE_UNKNOWN;
		mdata->fileurl = NULL;
		mdata->filepath = NULL;
	}
	return mdata;
ERROR:
	if (mdata)
	{
		ivug_free_mediadata(mdata);
	}
	return NULL;
}



Media_Data *ivug_alloc_mediadata_from_url(const char *url)
{
	IV_ASSERT(url != NULL);

	Media_Data * mdata = NULL;
	mdata = (Media_Data *) calloc(1, sizeof(Media_Data));
	if (mdata == NULL)
	{
		MSG_SDATA_ERROR("Cannot allocate memory");
		return NULL;
	}

	//file url
	mdata->fileurl = strdup(url);
	if (mdata->fileurl == NULL)
	{
		MSG_SDATA_ERROR("strdup return NULL");
		free(mdata);
		return NULL;
	}

	return mdata;
}

void ivug_free_mediadata(Media_Data * mdata)
{
	IV_ASSERT(mdata != NULL);

#ifdef USE_NEW_DB_API
	if (mdata->thumb_handle)
	{
		ivug_db_cancel_thumbnail(mdata->thumb_handle);
		mdata->thumb_handle = NULL;
	}

	if(mdata->m_handle)
	{
		media_info_destroy(mdata->m_handle);
		mdata->m_handle = NULL;
	}
#endif

	uuid_free(mdata->mediaID);


	//file path
	if (mdata->filepath)
	{
		//IVUG_DEBUG_MSG("filepath =%s", sd->filepath);
		free(mdata->filepath);
		mdata->filepath = NULL;
	}

	//file url
	if (mdata->fileurl)
	{
		MSG_SDATA_MED("Remove media data. %s", mdata->fileurl);
		free(mdata->fileurl);
		mdata->fileurl = NULL;
	}

	if (mdata->thumbnail_path)
	{
		free(mdata->thumbnail_path);
		mdata->thumbnail_path = NULL;
	}

	if (mdata->drm_filepath)
	{
		if ( ivug_remove_file(mdata->drm_filepath) == false)
		{
			MSG_SDATA_ERROR("Cannot delete drm file : %s", mdata->drm_filepath);
			// Go though
		}

		free(mdata->drm_filepath);
		mdata->drm_filepath = NULL;
	}

	free(mdata);
}

bool ivug_mediadata_set_tag(Media_Data *data, const char *newtag)
{
	ivug_retv_if(!newtag, false);

	MSG_UTIL_HIGH("Add Tag : uuid(%s), tag_name=%s", uuid_getchar(data->mediaID), newtag);

	tag_handle tag;
	bool ret = ivug_db_create_tag(&tag, newtag);
	if(ret == false)
	{
		MSG_SDATA_WARN("tag already created");
		tag = ivug_db_get_tag_handle(newtag);
		if(tag == NULL)
		{
			MSG_SDATA_ERROR("ivug_db_get_tag_handle ERROR");
			return false;
		}
	}

	ret = ivug_db_set_tag(data->m_handle, tag);
	if(ret == false)
	{
		MSG_SDATA_ERROR("ivug_db_set_tag ERROR");
	}

	ivug_db_destroy_tag(tag);

	return ret;
}

bool ivug_mediadata_set_favorite(Media_Data *data, bool bFavorite)
{
	MSG_UTIL_HIGH("Add Favorite : uuid(%s), bFavorite=%d", uuid_getchar(data->mediaID), bFavorite);

	return ivug_db_set_favorite(data->m_handle, bFavorite);
}



bool ivug_mediadata_delete(Media_Data * mdata)
{
	IV_ASSERT(mdata != NULL);

	if (uuid_is_valid(mdata->mediaID) == true )
	{
		int ret = media_info_delete_from_db(mdata->mediaID);
		if (ret == MEDIA_CONTENT_ERROR_NONE)
		{
			if (mdata->filepath)
			{
				MSG_SDATA_HIGH("File removed. %s", mdata->filepath);
				if (ivug_remove_file(mdata->filepath) == false)
				{
					MSG_SDATA_ERROR("file remove error : %s", mdata->filepath);
				}
			}
			else
			{
				MSG_SDATA_ERROR("File path is NULL", mdata->filepath);
			}
		}
		else
		{
			MSG_SDATA_ERROR("media_info_delete_from_db faild=%d uuid=%s file=%s", ret, uuid_getchar(mdata->mediaID), mdata->filepath);
			return false;
		}
	}
	else
	{
		MSG_SDATA_WARN("Invalid UUID. Path=%s", mdata->filepath);

		if (mdata->filepath)
		{
			MSG_SDATA_HIGH("File removed. %s", mdata->filepath);

			if (ivug_remove_file(mdata->filepath) == false)
			{
				MSG_SDATA_ERROR("file remove error : %s", mdata->filepath);
			}
		}
		else
		{
			MSG_SDATA_ERROR("File path is NULL", mdata->filepath);
		}

	}

	if (mdata->drm_filepath)
	{
		if ( ivug_remove_file(mdata->drm_filepath) == false)
		{
			MSG_SDATA_ERROR("Cannot delete drm file : %s", mdata->drm_filepath);
			// Go though
		}
	}

	return true;
}


