/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "ivug-define.h"

#include "ivug-debug.h"
#include "ivug-medialist.h"

#include "ivug-mediadata.h"

#include "ivug-data-list.h"


#include "ivug-db.h"

#include <Ecore.h>

#undef DEBUG_DUMP_LIST

#define LOAD_BLOCK_COUNT (100)

#undef UNIT_TEST
#define UT_USE_DB
#define UT_TOTAL (50)
#define UT_INDEX (49)

typedef struct {
	Eina_List *header; 	 // Start pointer for media list
	int count; 	 // Total count

	Ecore_Thread *thread;	// Loading thread
	Eina_Lock mutex;
	Eina_Condition condition;

	Eina_Bool bTerminate;	//

	bool bLoading;			// Is thread loading on progress?
	bool bStarted;			// Is thread started?
	Filter_struct *filter_str;

	Eina_List *shufflelist;	// Shuffle liste

	Ecore_Job *callback_job;
	ivug_medialist_cb cb;	// loaded callback
	void *data;
} _Media_List;


typedef struct {
	_Media_List *_mList;
	Filter_struct *filter_str;

	int center_block;
	int max_block;
} ThreadParam;

typedef enum {
	BLOCK_LEFT,
	BLOCK_RIGHT,
} BlockType;

typedef struct {
	_Media_List *_mList;

	BlockType direction;
	Eina_List *list;
} ThreadMSG;

static int _get_block_num(int index)
{
	return (index / LOAD_BLOCK_COUNT);
}

static Eina_List *_load_partial(const Filter_struct *filter, int stp, int endp)
{
	Eina_List *header = NULL;

#ifdef UT_USE_DB
	{
		if(filter->selected_list)
		{
			header = ivug_list_load_DB_items_list(filter, filter->selected_list);
		}
		else
		{
			header = ivug_list_load_DB_items(filter, stp, endp);
		}
	}

#else
	Eina_List *header = NULL;

	int i;
	Media_Data *data;

	for (i = stp; i <= endp; i++)
	{
		data = malloc(sizeof(Media_Data));
		data->index = i;

		header = eina_list_append(header, data);
	}
#endif

	MSG_SDATA_HIGH("Loaded : %d ~ %d", stp, endp);

	return header;
}

static Eina_List *_load_block(const Filter_struct *filter, int block_num)
{
	int stp = block_num * LOAD_BLOCK_COUNT;
	int endp = ((block_num + 1) * LOAD_BLOCK_COUNT) - 1;

	Eina_List *header = _load_partial(filter, stp, endp);

	MSG_SDATA_HIGH("Loaded block : %d", block_num);

	return header;
}

static void _job_send_cb(void *data)
{
	IV_ASSERT(data != NULL);

	_Media_List *_mList = (_Media_List *)data;
	if(_mList->cb)
	{
		_mList->cb(_mList, _mList->data);
	}
}

static void _call_loaded_callback(_Media_List *_mList)
{
	IV_ASSERT(_mList != NULL);

	if(_mList->callback_job)
		ecore_job_del(_mList->callback_job);

	_mList->callback_job = ecore_job_add(_job_send_cb, _mList);
}

static void _doLoad(Ecore_Thread *thread, _Media_List *_mList, const Filter_struct *filter, int center, int max_block)
{
	bool bRightEnd = false;
	bool bLeftEnd = false;
	int iter = 1;

	Eina_List *left;
	Eina_List *right;

	while((bLeftEnd && bRightEnd) == false)
	{
		if ( center + iter <= max_block )
		{
			right = _load_block(filter, center + iter);

#if 0
			if ( ecore_thread_check(thread) == EINA_TRUE )
			{
				MSG_SDATA_HIGH("Check True");
				ivug_list_delete_items(right);
				break;
			}
#else
			if ( _mList->bTerminate == EINA_TRUE )
			{
				MSG_SDATA_HIGH("Check True");
				ivug_list_delete_items(right);
				return;
			}
#endif
			IV_ASSERT(right != NULL);
			_mList->header = eina_list_merge (_mList->header, right);
		}
		else
		{
			bRightEnd = true;
		}

		if ( center - iter >= 0)
		{
			left = _load_block(filter , center - iter);

#if 0
			if ( ecore_thread_check(thread) == EINA_TRUE )
			{
				MSG_SDATA_HIGH("Check True");
				ivug_list_delete_items(left);
				break;
			}
#else
			if ( _mList->bTerminate == EINA_TRUE )
			{
				MSG_SDATA_HIGH("Check True");
				ivug_list_delete_items(left);
				return;
			}
#endif
			IV_ASSERT(left != NULL);

			_mList->header = eina_list_merge (left, _mList->header);

		}
		else
		{
			bLeftEnd = true;
		}

		iter++;
		usleep(100);
	}

	MSG_SDATA_HIGH("EinaCount=%d Count=%d", eina_list_count(_mList->header), _mList->count);
}


static void loader_heavy(void *data, Ecore_Thread *thread)
{
	MSG_SDATA_HIGH("Thread started. tID=0x%08x", thread);
	ThreadParam *pParam = data;

#if 0
	if ( ecore_thread_check(thread) == EINA_TRUE )
	{
		MSG_SDATA_HIGH("Check True");
		return;
	}
#else

	if ( pParam->_mList->bTerminate == EINA_TRUE )
	{
		MSG_SDATA_HIGH("Check True");
		return;
	}
#endif

	Eina_Bool ret = EINA_FALSE;

	ret = eina_lock_new(&pParam->_mList->mutex);
	if(ret == EINA_FALSE)
	{
		MSG_SDATA_FATAL("eina_lock_new failed");
	}
	else
	{
		ret = eina_condition_new(&pParam->_mList->condition, &pParam->_mList->mutex);
		if(ret == EINA_FALSE)
		{
			MSG_SDATA_FATAL("eina_condition_new failed");
		}
	}

	pParam->_mList->bStarted = true;

	MSG_SDATA_HIGH("Load : Begin");

	_doLoad(thread, pParam->_mList, pParam->filter_str, pParam->center_block, pParam->max_block);

	MSG_SDATA_HIGH("Load : End");

	pParam->_mList->thread = NULL;

	eina_lock_take(&pParam->_mList->mutex);
	eina_condition_signal(&pParam->_mList->condition);
	eina_lock_release(&pParam->_mList->mutex);
}


static void loader_end(void *data, Ecore_Thread *thread)
{
	ThreadParam *pParam = data;

	//  MSG_SDATA_HIGH("Thread Ended. EinaCount=%d Count=%d", eina_list_count(pParam->_mList), pParam->_mList->count);
	MSG_SDATA_HIGH("Thread Ended. tID=0x%08x", thread);

	PERF_CHECK_END(LVL3, "Deffered loading");

	ivug_data_filter_delete(pParam->filter_str);

	_call_loaded_callback(pParam->_mList);
}

static void loader_cancel(void *data, Ecore_Thread *thread)
{
	ThreadParam *pParam = data;

	MSG_SDATA_HIGH("Thread canceled. tID=0x%08x", thread);

	PERF_CHECK_END(LVL3, "Deffered loading");

	ivug_data_filter_delete(pParam->filter_str);
}

static int _sort_cb(const void *d1, const void *d2)
{
	return (rand() % 4 - 2) ;
}

static Media_Item *_find_item(Media_List *mList, int index)
{
	IV_ASSERT(mList != NULL);
	_Media_List *_mList = (_Media_List *)mList;

	Eina_List *l;
	Media_Data *data;

	EINA_LIST_FOREACH(_mList->header, l, data)
	{
		if ( data->index == index)
		{
			return (Media_Item *)l;
		}
	}

	return NULL;
}


#ifdef DEBUG_DUMP_LIST
static void _print_shuffle(void *data)
{
	MSG_SDATA_HIGH("Item : %d", (int)data);
}

static void _dump_list(Eina_List *list, void (*func)(void *))
{
	Eina_List *l;
	void *data;

	EINA_LIST_FOREACH(list, l, data)
	{
		func(data);
	}

}
#endif

static void _create_shuffle_list(_Media_List *_mList)
{
	int i;
	_mList->shufflelist = NULL;
	for ( i = 0; i < _mList->count; i++)
	{
		_mList->shufflelist = eina_list_append(_mList->shufflelist, (void *)i);
	}

	_mList->shufflelist = eina_list_sort(_mList->shufflelist, eina_list_count(_mList->shufflelist), _sort_cb);
}

Media_List *ivug_medialist_create()
{
	_Media_List *_mList = calloc(1, sizeof(_Media_List));
	IV_ASSERT(_mList != NULL);

	srand((unsigned)time(NULL));

	MSG_SDATA_HIGH("Create media list : 0x%08x", _mList);

	return (Media_List *)_mList;
}

bool ivug_medialist_set_callback(Media_List *mList, ivug_medialist_cb callback, void *data)
{
	IV_ASSERT(mList != NULL);

	_Media_List *_mList = (_Media_List *)mList;
	_mList->cb = callback;
	_mList->data = data;

	return true;
}

Media_Item * ivug_medialist_find_item_by_filename(Media_List *mList, const char* filepath)
{
	IV_ASSERT(mList != NULL);
	IV_ASSERT(filepath != NULL);
	_Media_List *_mList = (_Media_List *)mList;

	Eina_List *l;
	Media_Data *data;

	EINA_LIST_FOREACH(_mList->header, l, data)
	{
		if(strncmp(data->fileurl, filepath, strlen(data->fileurl)) == 0)
		{
			return (Media_Item *)l;
		}
	}
	MSG_SDATA_ERROR("Cannot find file path %s at list", filepath);
	return NULL;
}


Media_Item *ivug_medialist_load(Media_List *mList, const Filter_struct *filter)
{
	IV_ASSERT(mList != NULL);
	_Media_List *_mList = (_Media_List *)mList;

	MSG_SDATA_HIGH("Create slide data list. index=%d", filter->index);

	if ( _mList->header != NULL )
	{
		MSG_SDATA_FATAL("Header is not NULL");
		return NULL;
	}
//	_mList->filter = ivug_data_filter_copy(filter);

	int db_idx = 0;

#ifdef UNIT_TEST
	_mList->count = UT_TOTAL;
	db_idx = UT_INDEX;

#else
	PERF_CHECK_BEGIN(LVL3, "MediaList - Get list count");
	 if(filter->view_by == IVUG_VIEW_BY_DIRECTORY)
	{
		_mList->count = ivug_list_get_dir_cnt(filter->dir_filter->basedir);

		if ( _mList->count == 0 )
		{
			MSG_SDATA_ERROR("No file founded");
			return NULL;
		}

		_mList->header = ivug_list_load_dir_items(filter->dir_filter->basedir);

// Find Current.
		Eina_List *l = NULL;
		void *data;

		EINA_LIST_FOREACH(_mList->header, l, data)
		{
			Media_Data *pData = data;

			if (strcmp(pData->fileurl, filter->dir_filter->current) == 0)
			{
				return (Media_Item *)l;
			}
		}

		MSG_SDATA_ERROR("Not found current");
		return NULL;
	}
	else if(filter->view_by == IVUG_VIEW_BY_FOLDER && filter->index == IVUG_INVALID_INDEX)
	{
		_mList->count = ivug_list_get_item_cnt(filter);
		if ( _mList->count == 0 )
		{
			MSG_SDATA_ERROR("No file founded");
			return NULL;
		}
		PERF_CHECK_BEGIN(LVL3, "MediaList - load all block");
		_mList->header = _load_partial(filter, 0, _mList->count-1);	 // Load all
		PERF_CHECK_END(LVL3, "MediaList - load all block");
		if(_mList->header == NULL)
		{
			MSG_SDATA_ERROR("MediaList is NULL");
			return NULL;
		}
		MSG_SDATA_HIGH("%d items loaded", eina_list_count(_mList->header));
		_create_shuffle_list(_mList);
		Media_Item *cur = ivug_medialist_find_item_by_filename(mList, filter->filepath);
		_call_loaded_callback(_mList);
		return cur;
	}
	else
	{
		_mList->count = ivug_list_get_item_cnt(filter);
	}

	PERF_CHECK_END(LVL3, "MediaList - Get list count");

	MSG_SDATA_HIGH("Total item count=%d", _mList->count);
	db_idx = filter->index - 1;
#endif

	if ( _mList->count < db_idx)
	{
		MSG_SDATA_ERROR("Invalid param");
		return NULL;
	}

	int block_num = _get_block_num(db_idx);
	int block_max = _get_block_num(_mList->count - 1);

	MSG_SDATA_HIGH("Total=%d Current=%d Block Count=%d", _mList->count, db_idx, block_max);

	PERF_CHECK_BEGIN(LVL3, "MediaList - load first block");

	// Load Center
	_mList->header = _load_block(filter, block_num);	 // Load current block

	PERF_CHECK_END(LVL3, "MediaList - load first block");

	if(_mList->header == NULL)
	{
		//MSG_SDATA_FATAL("MediaList is NULL");
		MSG_SDATA_ERROR("MediaList is NULL");
		return NULL;
	}

// find current data;
	MSG_SDATA_HIGH("Header=0x%08x Current nTh=%d", _mList->header, db_idx - (block_num * LOAD_BLOCK_COUNT));

	PERF_CHECK_BEGIN(LVL3, "MediaList - Find current");

	Eina_List *current = eina_list_nth_list(_mList->header, db_idx - (block_num * LOAD_BLOCK_COUNT));

	PERF_CHECK_END(LVL3, "MediaList - Find current");

	if(current == NULL)
	{
		MSG_SDATA_HIGH("current is NULL");
		return NULL;
	}

	PERF_CHECK_BEGIN(LVL3, "MediaList - shuffle");

	_create_shuffle_list(_mList);

	PERF_CHECK_END(LVL3, "MediaList - shuffle");

	// _dump_list(_mList->shufflelist, _print_shuffle);
	//  MSG_SDATA_HIGH("ParamPath=%s CurrentPath=%s", param->filepath, _mList->current->filepath);

	if ( block_max == 0 && eina_list_count(_mList->header) < LOAD_BLOCK_COUNT )
	{
		MSG_SDATA_HIGH("Deffered loading is not needed. BolckMax=%d FirstBlockCount=%d", block_max, eina_list_count(_mList->header));
		_call_loaded_callback(_mList);
		return (Media_Item *)current;
	}

	ThreadParam *pParam = NULL;

	pParam = malloc(sizeof(ThreadParam));

	pParam->filter_str = ivug_data_filter_copy(filter);
	pParam->_mList = _mList;
	pParam->center_block = block_num;
	pParam->max_block = block_max;

	_mList->bLoading = true;

	MSG_SDATA_HIGH("Starting thread");

	PERF_CHECK_BEGIN(LVL3, "Deffered loading");

	_mList->thread = ecore_thread_run(loader_heavy, loader_end, loader_cancel, pParam);
	MSG_SDATA_HIGH("Thread ID= 0x%08x", _mList->thread);
	MSG_SDATA_HIGH("Create slide data list END");

	return (Media_Item *)current;
}


void
ivug_medialist_del(Media_List *mList)
{
	IV_ASSERT(mList != NULL);
	_Media_List *_mList = (_Media_List *)mList;
	Eina_Bool ret = EINA_FALSE;

	MSG_SDATA_HIGH("Removing all media data");

	if(_mList->callback_job)
	{
		ecore_job_del(_mList->callback_job);
	}

	if ( _mList->thread )
	{
		MSG_SDATA_HIGH("1. Thread cancel");
#if 0
		if ( ecore_thread_cancel(_mList->thread) == EINA_TRUE )
		{
			MSG_SDATA_HIGH("Thread canceled");
		}
#endif
		_mList->bTerminate = EINA_TRUE;

		if(_mList->bStarted)
		{
			ret = eina_condition_wait(&_mList->condition);
			if(ret == EINA_FALSE)
			{
				MSG_SDATA_ERROR("eina_condition_wait error");
			}
			ret = eina_lock_release(&_mList->mutex);
			if(ret == EINA_FALSE)
			{
				MSG_SDATA_ERROR("eina_lock_release error");
			}
			eina_condition_free(&_mList->condition);
			eina_lock_free(&_mList->mutex);
		}

		_mList->thread = NULL;
		MSG_SDATA_HIGH("2. Thread cancel");
	}

	ivug_list_delete_items(_mList->header);
	_mList->header = NULL;

	eina_list_free(_mList->shufflelist);
	_mList->shufflelist = NULL;

	_mList->bStarted = false;

	free(_mList);

}


int ivug_medialist_get_count(Media_List *mList)
{
	IV_ASSERT(mList != NULL);
	_Media_List *_mList = (_Media_List *)mList;

	return _mList->count;
}

int ivug_medialist_get_index(Media_List *mList, Media_Item *item)
{
	IV_ASSERT(mList != NULL);

	Media_Data *pData;

	MSG_SDATA_HIGH("Loading is not finished");
	pData = (Media_Data *)eina_list_data_get((Eina_List *)item);

	return pData->index + 1;


#if 0
	PERF_CHECK_BEGIN("Get Index");

	for (tmp = _mList->header, i= 0;  tmp != NULL ; tmp = eina_list_next(tmp), i++)
	{
	if ( item == tmp )
	{
	PERF_CHECK_END("Get Index");
	return i+1; // This index starts from 1
	}
	}

	PERF_CHECK_END("Get Index");


	return 0;		 // Cannot find.
#endif

}

Media_Item *ivug_medialist_get_first(Media_List *mList)
{
	IV_ASSERT(mList != NULL);
	_Media_List *_mList = (_Media_List *)mList;

	return (Media_Item *)_mList->header;
}


Media_Item *ivug_medialist_get_last(Media_List *mList)
{
	IV_ASSERT(mList != NULL);
	_Media_List *_mList = (_Media_List *)mList;

	return (Media_Item *)eina_list_last(_mList->header);
}

Media_Item *ivug_medialist_get_next(Media_List *mList, Media_Item *item)
{
	IV_ASSERT(mList != NULL);

	IV_ASSERT(item != NULL);
	Eina_List *next = eina_list_next((Eina_List *)item);

	return (Media_Item *)next;
}


Media_Item *ivug_medialist_get_prev(Media_List *mList, Media_Item *item)
{
	IV_ASSERT(mList != NULL);

	IV_ASSERT(item != NULL);
	Eina_List *prev = eina_list_prev((Eina_List *)item);

	return (Media_Item *)prev;

}

Media_Data *ivug_medialist_get_data(const Media_Item *item)
{
	IV_ASSERT(item != NULL);

	return (Media_Data *)eina_list_data_get((Eina_List *)item);
}


void ivug_medialist_delete_item(Media_List *mList, Media_Item *item)
{
	PERF_CHECK_BEGIN(LVL1, "MediaList - delete");

	IV_ASSERT(mList != NULL);
	_Media_List *_mList = (_Media_List *)mList;

	Media_Data *pData = (Media_Data *)eina_list_data_get((Eina_List *)item);

	Eina_List *list = eina_list_next((Eina_List *)item);
	Eina_List *l;
	Media_Data *data;

	EINA_LIST_FOREACH(list, l, data)
	{
		data->index = data->index - 1;
	}

	_mList->header = eina_list_remove_list(_mList->header, (Eina_List *)item);

	// Change index
	if ( ivug_mediadata_delete(pData) == false )
	{
		MSG_SDATA_HIGH("Cannot delete mediadata");
	}
	ivug_free_mediadata(pData);

	// Shuffle list?
	_mList->count--;

	Eina_List *found = eina_list_data_find_list(_mList->shufflelist, (void *)_mList->count);

	_mList->shufflelist = eina_list_remove(_mList->shufflelist, found);

	MSG_SDATA_HIGH("Item removed. Total=%d", _mList->count);

	PERF_CHECK_END(LVL1, "MediaList - delete");
}


Media_Item *ivug_medialist_get_random_item(Media_List *mList)
{
	IV_ASSERT(mList != NULL);
	_Media_List *_mList = (_Media_List *)mList;

	int count = eina_list_count(_mList->header);

	return (Media_Item *)eina_list_nth_list(_mList->header, random() % count);
}

Media_Item *ivug_medialist_get_shuffle_item(Media_List *mList, Media_Item *item)
{
	IV_ASSERT(mList != NULL);
	_Media_List *_mList = (_Media_List *)mList;

	Media_Data *pData = (Media_Data *)eina_list_data_get((Eina_List *)item);

	Eina_List *found = eina_list_data_find_list(_mList->shufflelist, (void *)pData->index);

	if ( found != NULL )
	{
		Media_Item *next;
		Eina_List *iter;

		iter = eina_list_next(found);

		if ( iter == NULL )
		{
			int nFirstIdx = (int)eina_list_data_get(_mList->shufflelist);

			next = _find_item(mList, nFirstIdx);

			MSG_SDATA_HIGH("End reached. rewind to first. Index=%d", nFirstIdx);

			return next;
		}

		do {
			next = _find_item(mList, (int)eina_list_data_get(iter));

			if ( next == NULL )
			{
				MSG_SDATA_HIGH("Index : %d is not loaded", (int)eina_list_data_get(iter));
				_mList->shufflelist = eina_list_demote_list(_mList->shufflelist, iter);
			}

			iter = eina_list_next(found);

		} while(next == NULL);

		Media_Data *nData = (Media_Data *)eina_list_data_get((Eina_List *)next);

		MSG_SDATA_HIGH("Shuffle : %d->%d", pData->index, nData->index );

//		_dump_list(_mList->shufflelist, _print_shuffle);
		return next;
	}

	MSG_SDATA_ERROR("Cannot find data. Index=%d", pData->index);
	return NULL;
}

Media_Item *
ivug_medialist_find_item_by_index(Media_List *mList, int index)
{
	IV_ASSERT(mList != NULL);

	_Media_List *_mList = (_Media_List *)mList;

	Media_Data *current = NULL;
	Eina_List *l = NULL;

	EINA_LIST_FOREACH(_mList->header, l, current)
	{
		if(current == NULL)
		{
			MSG_IVUG_ERROR("album list is NULL");
			break;
		}
		if(current->index == index)
		{
			return (Media_Item *)l;
		}
	}
	return NULL;
}

Media_Item * ivug_medialist_append_item(Media_List *mList, const char *filepath)
{
	IV_ASSERT(mList != NULL);

	_Media_List *_mList = (_Media_List *)mList;

	_mList->header = ivug_list_append_item(_mList->header, filepath);

	_mList->count++;

	int index = 0;
	Eina_List *l;
	Media_Data *data;
	EINA_LIST_FOREACH(_mList->header, l, data)
	{
		data->index = index++;
	}

	return (Media_Item *)eina_list_last(_mList->header);
}

Media_Item * ivug_medialist_prepend_item(Media_List *mList, const char *filepath)
{
	IV_ASSERT(mList != NULL);

	_Media_List *_mList = (_Media_List *)mList;

	_mList->header = ivug_list_prepend_item(_mList->header, filepath);

	_mList->count++;

	int index = 0;
	Eina_List *l;
	Media_Data *data;
	EINA_LIST_FOREACH(_mList->header, l, data)
	{
		data->index = index++;
	}

	return (Media_Item *)_mList->header;
}

