/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdlib.h>
#include <ui-gadget-module.h>
#include "ivug-anim.h"
#include "ivug-slideshow.h"
#include "ivug-slideshow-priv.h"

#include "ivug-debug.h"

#include "ivug-config.h"
#include "ivug-define.h"
#include "ivug-datatypes.h"
#include "ivug-uuid.h"
#include "ivug-util.h"

#include "ivug-image.h"

#include <Elementary.h>
#include <assert.h>

#include "statistics.h"

#include "EFLUtil.h"

#undef USE_IVUG_IMAGE

#undef LOG_LVL
#define LOG_LVL DBG_MSG_LVL_MED

#undef LOG_CAT
#define LOG_CAT "IV-SLIDESHOW"

#define EDJ_PATH PREFIX"/res/edje/"PACKAGE
#define IVUG_SS_LY_EDJ_PATH 	EDJ_PATH"/ivug-ss-ly.edj"

#define IVUG_IMAGE_BETWEEN_MARGIN 	(30)
#define IVUG_IMAGE_MOVE_MARGIN 			((int)IVUG_IMAGE_BETWEEN_MARGIN*0.8)
#define IVUG_IMAGE_SCROLL_MARGIN 		(5)

#define _EDJ(o)			elm_layout_edje_get(o)

void ivug_ss_get_screen_size(int *width, int *height)
{
	int rotation = elm_win_rotation_get((Evas_Object *)ug_get_window());

	int screen_w = 0;
	int screen_h = 0;
	ecore_x_window_size_get(ecore_x_window_root_first_get(), &screen_w, &screen_h);
	if(rotation == 0 || rotation == 180)
	{
		*width = screen_w;
		*height = screen_h;
	}
	else if(rotation == 90 || rotation == 270)
	{
		*width = screen_h;
		*height = screen_w;
	}
}

Media_Item *ivug_ss_get_next_item(Media_List *mList,
			Media_Item *header,
			Media_Item *current,
			slide_show_mode mode)
{
	Media_Item *item = NULL;

	MSG_HIGH("Get Next Item : Header=0x%08x Current=0x%08x", header, current);

	MSG_HIGH("Get Next : Header=0x%08x Current=0x%08x",
			 header, current);
	switch(mode)
	{
	case SLIDE_SHOW_MODE_REPEAT:
		item = ivug_medialist_get_next(mList, current);
		if ( item == NULL )
		{
			item = ivug_medialist_get_first(mList);
		}
		break;

	case SLIDE_SHOW_MODE_NORMAL:
		item = ivug_medialist_get_next(mList, current);
		break;

	case SLIDE_SHOW_MODE_SHUFFLE_REPEAT:
		item = ivug_medialist_get_shuffle_item(mList, current);
		if ( item == NULL )
		{
			MSG_ERROR("Never touch here");
		}
		break;

	case SLIDE_SHOW_MODE_SHUFFLE:
		item = ivug_medialist_get_shuffle_item(mList, current);
		if ( item == header )
		{
			MSG_ERROR("Reach end");
			return NULL;
		}
		break;

	default:
		MSG_ERROR("Unknown mode : %d", mode);
		item = NULL;
		break;
	}

	return item;

}


static void
_moved(void *data, Evas *evas, Evas_Object *obj, void *event_info)
{
	SlideShow *pSlideShow = (SlideShow *)data;

	Evas_Coord ox, oy, ow, oh;
	evas_object_geometry_get(obj, &ox, &oy, &ow, &oh);

	MSG_HIGH("Moved (%d,%d,%d,%d)", ox, oy, ow, oh);

	evas_object_move(pSlideShow->event, ox, oy);
}

static void
_resized(void *data, Evas *evas, Evas_Object *obj, void *event_info)
{
	SlideShow *pSlideShow = (SlideShow *)data;

	Evas_Coord ox, oy, ow, oh;
	evas_object_geometry_get(obj, &ox, &oy, &ow, &oh);

	MSG_HIGH("Resized (%d,%d,%d,%d)", ox, oy, ow, oh);

	evas_object_resize(pSlideShow->event, ow, oh);

	evas_object_resize(pSlideShow->sLayout[pSlideShow->sCurrent].layout, ow, oh);
	evas_object_resize(pSlideShow->sLayout[(pSlideShow->sCurrent + 1) % 2].layout, ow, oh);
}

static void
_shown(void *data, Evas *evas, Evas_Object *obj, void *event_info)
{
	MSG_HIGH("SlideShow is Shown");
}

static void
_hidden(void *data, Evas *evas, Evas_Object *obj, void *event_info)
{
	MSG_HIGH("SlideShow is Hidden");
}


static void
_shown1(void *data, Evas *evas, Evas_Object *obj, void *event_info)
{
	MSG_HIGH("%s is Shown", evas_object_name_get(obj));
}

static void
_hidden1(void *data, Evas *evas, Evas_Object *obj, void *event_info)
{
	MSG_HIGH("%s is Hidden", evas_object_name_get(obj));
}

static bool _ivug_ss_set_content(Slide_Layout *pSlide, Media_Item *item)
{
	Media_Data *mdata;
	int ret = EVAS_LOAD_ERROR_NONE;

	pSlide->mitem = item;

	if(pSlide->mitem == NULL)
	{
		MSG_ERROR("Mitem is NULL");
		return false;
	}

	mdata = ivug_medialist_get_data(item);
	IV_ASSERT(mdata != NULL);

	MSG_HIGH("Start Loading : %s", mdata->filepath);

	if(mdata->slide_type == SLIDE_TYPE_VIDEO)
	{
		/* Display play icon */
		edje_object_signal_emit(elm_layout_edje_get(pSlide->layout),
					"elm,state,show_play_icon",
					"slideshow");


#ifdef USE_IVUG_IMAGE
		ret = ivug_image_file_set(pSlide->photocam, mdata->thumbnail_path, NULL);
#else
		ret = elm_photocam_file_set(pSlide->photocam, mdata->thumbnail_path);
#endif
		if(EVAS_LOAD_ERROR_NONE != ret) {
			MSG_HIGH("elm_photocam_file_set failed");
			return false;
		}
	}
	else
	{
		/* Hide play icon */
		edje_object_signal_emit(elm_layout_edje_get(pSlide->layout),
					"elm,state,hide_play_icon",
					"slideshow");
#ifdef USE_IVUG_IMAGE
		ret = ivug_image_file_set(pSlide->photocam, mdata->filepath, NULL);
#else
		ret = elm_photocam_file_set(pSlide->photocam, mdata->filepath);
#endif
		if(EVAS_LOAD_ERROR_NONE != ret) {
			MSG_HIGH("elm_photocam_file_set failed");
			return false;
		}

		if (elm_image_file_set(pSlide->thumbnail, mdata->thumbnail_path, NULL) == EINA_FALSE)
		{
			MSG_ERROR("Cannot load thumbnail : %s", mdata->thumbnail_path);
		}
		else
		{
			edje_object_signal_emit(_EDJ(pSlide->layout), "elm,state,show_thumbnail", "slideshow");
		}
	}
	MSG_HIGH("Load : %s", mdata->filepath);
	elm_photocam_zoom_mode_set(pSlide->photocam, ELM_PHOTOCAM_ZOOM_MODE_AUTO_FIT);
	elm_photocam_paused_set(pSlide->photocam, true);
	evas_object_size_hint_weight_set(pSlide->photocam, EVAS_HINT_EXPAND,
									 EVAS_HINT_EXPAND);
	/* use for test
	int w_p = 0;
	int h_p = 0;
	float ff = elm_photocam_zoom_get(pSlide->photocam);
	char * cc = (char *)elm_photocam_file_get(pSlide->photocam);
	elm_photocam_image_size_get(pSlide->photocam, &w_p, &h_p);
	MSG_IVUG_HIGH("photocam_zoom:%f, %s ,%d,%d", ff,cc, w_p, h_p);
	*/
	return true;
}


static void _ivug_ss_update_pos(SlideShow *pSlideShow, Evas_Coord x, Evas_Coord y)
{
	IV_ASSERT(pSlideShow != NULL);

	MSG_HIGH("Update Pos(%d,%d) sCurrent=%d", x, y, pSlideShow->sCurrent);

//	x = x - 360;

	Slide_Layout *sLyCurrent = &pSlideShow->sLayout[pSlideShow->sCurrent];
	Slide_Layout *sLyNext = &pSlideShow->sLayout[(pSlideShow->sCurrent + 1) % 2];

	evas_object_move(sLyCurrent->layout, x, y);
	evas_object_move(sLyNext->layout,
				x + pSlideShow->screen_w + IVUG_IMAGE_BETWEEN_MARGIN ,
				y);
}
static bool _ivug_ss_load_next_image(SlideShow *pSlideShow)
{
	ivug_retv_if(!pSlideShow, false);
	MSG_HIGH("");

	Slide_Layout* sLyCurrent = &pSlideShow->sLayout[pSlideShow->sCurrent];
	Slide_Layout* sLy = &pSlideShow->sLayout[(pSlideShow->sCurrent + 1) % 2];

	Media_Item *next = NULL;
	Media_Item *current = sLyCurrent->mitem;
	do {
		next = ivug_ss_get_next_item(pSlideShow->media_list,
				pSlideShow->ss_Header,
				current,
				pSlideShow->ss_mode);

		if(next == NULL) {
			sLy->mitem = NULL;
			return false;
		}
		current = next;
	}while(!_ivug_ss_set_content(sLy, next));

	if(next) {
		evas_object_show(sLy->layout);
	}

	return true;
}

void _ivug_ss_effect_finished(void *data)
{
	ivug_ret_if(!data);
	SlideShow *pSlideShow = (SlideShow *) data;
	MSG_HIGH("slideshow Effect ended");

	if(pSlideShow->effect_engine)
	{
		MSG_HIGH("ivug_effect_finalize");
		ivug_effect_finalize(pSlideShow->effect_engine);
		pSlideShow->effect_engine = NULL;
	}

	/* Increse current index */
	pSlideShow->sCurrent = (pSlideShow->sCurrent + 1) % 2;
	_ivug_ss_update_pos(pSlideShow, 0, 0);		// Reset position

	Slide_Layout *sLyCurrent = &pSlideShow->sLayout[pSlideShow->sCurrent];
	Slide_Layout *sLyNext = &pSlideShow->sLayout[(pSlideShow->sCurrent + 1) % 2];

	Media_Item *next = NULL;

	next = ivug_ss_get_next_item(pSlideShow->media_list,
			pSlideShow->ss_Header,
			sLyCurrent->mitem,
			pSlideShow->ss_mode);

	if(next == NULL) {
		sLyNext->mitem = NULL;
		MSG_ERROR("Cannot find next item");
		return;
	}

	_ivug_ss_set_content(sLyNext, next);

// Reset visibilaty because effect can hide layout
	evas_object_show(sLyCurrent->layout);
	evas_object_show(sLyNext->layout);

	MSG_HIGH("pSlideShow->bSS_StopFlag = %d", pSlideShow->bSS_StopFlag);

	if(pSlideShow->bSS_StopFlag == EINA_TRUE)
	{
		pSlideShow->state = SLIDE_SHOW_STOPPED;
		pSlideShow->ss_Header = NULL;
		evas_object_smart_callback_call(ivug_ss_object_get(pSlideShow), "slideshow,finished", (void *)SLIDE_SHOW_STOPPED);
//		ivug_ss_delete(pSlideShow);

		pSlideShow->bSS_StopFlag = EINA_FALSE;
	}
	//EFL::dump_obj(pSlideShow->obj, 0);
}

Evas_Object *_ivug_ss_create_layout(Evas_Object *parent, const char *edj_path, const char *group)
{
	MSG_ASSERT(parent != NULL);
	MSG_ASSERT(edj_path != NULL);
	MSG_ASSERT(group != NULL);

	Evas_Object *ly = NULL;
	ly = elm_layout_add(parent);

	ivug_retv_if(!ly, NULL);

	if (elm_layout_file_set(ly , edj_path, group ) == EINA_FALSE)
	{
		MSG_ERROR("Cannot create layout. %s %s", edj_path, group);
		evas_object_del(ly);
		return NULL;
	}
	evas_object_size_hint_weight_set(ly, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(ly, EVAS_HINT_FILL, EVAS_HINT_FILL);

	evas_object_show(ly);
	return ly;
}

// Slide Show related functions.
static Eina_Bool _ivug_ss_on_slide_interval(void *data)
{
	MSG_ASSERT(data != NULL);

	SlideShow *pSlideShow = (SlideShow*)data;

	MSG_HIGH("On Slide Interval");
	/* stopped by other operation */
	if (pSlideShow->state == SLIDE_SHOW_STOPPED)
	{
		pSlideShow->ss_timer = NULL;
		MSG_ERROR("Slide show already stopped");
		return ECORE_CALLBACK_CANCEL;
	}

	Slide_Layout *sLyCurrent = &pSlideShow->sLayout[pSlideShow->sCurrent];
	Slide_Layout *sLyNext = &pSlideShow->sLayout[(pSlideShow->sCurrent + 1) % 2];

	/* Next item is NULL */
	if(sLyNext->mitem == NULL)
	{
		MSG_HIGH("Next item is NULL");
		pSlideShow->ss_timer = NULL;
		pSlideShow->ss_Header = NULL;
		/* exit slide show after whole animation is over */
		ivug_ss_stop(pSlideShow);

		return ECORE_CALLBACK_CANCEL;
	}

	/* Slideshow Effect */
	Effect_Engine *eng = ivug_effect_add(pSlideShow->effect_type);
	MSG_ASSERT(eng != NULL);

	pSlideShow->effect_engine = eng;

	ivug_effect_init(pSlideShow->effect_engine, sLyCurrent->layout, sLyNext->layout);

	int rotation = elm_win_rotation_get((Evas_Object *)ug_get_window());
	ivug_ss_get_screen_size(&pSlideShow->screen_w, &pSlideShow->screen_h);

	if(ivug_effect_set_size(pSlideShow->effect_engine, pSlideShow->screen_w, pSlideShow->screen_h, rotation) == false)
	{
		pSlideShow->ss_timer = NULL;
		ivug_effect_finalize(pSlideShow->effect_engine);
		return ECORE_CALLBACK_CANCEL;
	}

//	EFL::dump_obj(pSlideShow->obj, 0);

	ivug_effect_start(pSlideShow->effect_engine, _ivug_ss_effect_finished, pSlideShow);

	return ECORE_CALLBACK_RENEW;
}



static Effect_Type _ivug_ss_get_trans_effect(ivug_effect_type type)
{
	MSG_MED("type %d", type);
	switch(type)
	{
		case IVUG_EFFECT_TYPE_SLIDE:
			return EFFECT_SLIDE;
		case IVUG_EFFECT_TYPE_IRIS:
			return EFFECT_IRIS;
		case IVUG_EFFECT_TYPE_DISSOLVE:
			return EFFECT_DISSOLVE;
		case IVUG_EFFECT_TYPE_DISSOLVE_FADE:
			return EFFECT_DISSOLVE_FADE;
		case IVUG_EFFECT_TYPE_SPLIT:
			return EFFECT_SPLIT;
		default:
			return EFFECT_SLIDE;
			//break;
	}
	return EFFECT_NONE;
}




void _ivug_ss_on_mouse_down(void *data, Evas *e,
	Evas_Object *obj, void *event_info)
{
	SlideShow *pSlideShow = (SlideShow *) data;

	MSG_HIGH("_ivug_ss_on_mouse_down, state is %d", pSlideShow->state);

	if (pSlideShow->click_timer) {
		ecore_timer_del(pSlideShow->click_timer);
		pSlideShow->click_timer = NULL;
	}

	if(pSlideShow->ss_timer) {
		ecore_timer_del(pSlideShow->ss_timer);
		pSlideShow->ss_timer = NULL;
	}
}

void _ivug_ss_on_mouse_move(void *data, Evas *e,
	Evas_Object *obj, void *event_info)
{
	MSG_LOW("_ivug_ss_on_mouse_move");
	return;
}

static Eina_Bool _ivug_ss_clicked_timer_cb(void *data)
{
	SlideShow *pSlideShow = (SlideShow *)data;
	if (NULL == pSlideShow) {
		return ECORE_CALLBACK_CANCEL;
	}

	pSlideShow->click_timer = NULL;

	if(pSlideShow->state == SLIDE_SHOW_RUNNING) {
		ivug_ss_stop(pSlideShow);
		//EFL::dump_obj(pSlideShow->obj, 0);
		return ECORE_CALLBACK_CANCEL;
	}

	MSG_HIGH("-------------_ivug_ss_clicked_timer_cb--------------");

	pSlideShow->state = SLIDE_SHOW_INTERRUPTED;
	evas_object_smart_callback_call(ivug_ss_object_get(pSlideShow), "slideshow,finished", (void *)SLIDE_SHOW_INTERRUPTED);

	return ECORE_CALLBACK_CANCEL;
}
void _ivug_ss_on_mouse_up(void *data, Evas *e,
	Evas_Object *obj, void *event_info)
{
	SlideShow *pSlideShow = (SlideShow *)data;
	if (NULL == pSlideShow) {
		return;
	}
	MSG_HIGH("_ivug_ss_on_mouse_up, pSlideShow->v is %d", pSlideShow->state);

	if (pSlideShow->bMouse_event) {
		if(pSlideShow->click_timer)
		{
			ecore_timer_del(pSlideShow->click_timer);
		}
		pSlideShow->click_timer = ecore_timer_add(0.3f, _ivug_ss_clicked_timer_cb, data);
	}
}

static Eina_Bool
_ivug_ss_auto_finish_timer_cb(void *data)
{
	ivug_retv_if(!data, ECORE_CALLBACK_CANCEL);

	SlideShow *pSlideShow = (SlideShow*)data;
	/* exit slide show after whole animation is over */
	ivug_ss_stop(pSlideShow);
	return ECORE_CALLBACK_CANCEL;
}


static void
_ivug_ss_photocam_loaded_cb(void *data, Evas_Object *obj, void *event_info)
{
	IV_ASSERT(data != NULL);

	Slide_Layout* sLy = static_cast<Slide_Layout*>(data);
	SlideShow *pSlideShow = (SlideShow *)sLy->pSlideshow;

	if ( sLy->mitem == NULL)
	{
		MSG_ERROR("Debug Me! Data item is NULL.");
		return;
	}

	Media_Data* mdata = ivug_medialist_get_data(sLy->mitem);
	if(mdata == NULL)
	{
		MSG_ERROR("ivug_medialist_get_data failed.");
		return;
	}

	Evas_Load_Error error = static_cast<Evas_Load_Error>(reinterpret_cast<int>(event_info));
	if ( error != EVAS_LOAD_ERROR_NONE )
	{
		MSG_HIGH("Photocam Pre-loaded. File=%s", ivug_get_filename(mdata->filepath));
		MSG_SITEM_ERROR("Image loading failed. Error=%d File=%s",
				error, mdata->filepath);
		_ivug_ss_load_next_image((SlideShow *)sLy->pSlideshow);
		return;
	}
	edje_object_signal_emit(_EDJ(pSlideShow->sLayout[pSlideShow->sCurrent].layout), "elm,state,hide_thumbnail", "slideshow");

	MSG_SITEM_HIGH("Photocam Pre-loaded. File=%s",
			ivug_get_filename(mdata->filepath));

	return;
}

static bool _ivug_ss_create_image_layout(Evas_Object *parent, Slide_Layout *sLayout)
{
	/* Create Layout for the current item */
	sLayout->layout = _ivug_ss_create_layout(parent, IVUG_SS_LY_EDJ_PATH, "slayout");

	if(sLayout->layout == NULL) {
		MSG_ERROR("Cannot create current layout");
		return false;
	}

#ifdef USE_IVUG_IMAGE
	sLayout->photocam = ivug_image_create(sLayout->layout);
#else
	sLayout->photocam = elm_photocam_add(sLayout->layout);
#endif
	evas_object_size_hint_expand_set(sLayout->photocam, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(sLayout->photocam, EVAS_HINT_FILL, EVAS_HINT_FILL);

	elm_object_part_content_set(sLayout->layout, "content", sLayout->photocam);

	sLayout->thumbnail = elm_icon_add(sLayout->layout);
	evas_object_name_set(sLayout->thumbnail, "Thumbnail");
	elm_object_part_content_set(sLayout->layout, "thumbnail", sLayout->thumbnail);

#ifdef USE_IVUG_IMAGE
#else
	elm_photocam_zoom_mode_set(sLayout->photocam, ELM_PHOTOCAM_ZOOM_MODE_AUTO_FIT);
	elm_photocam_paused_set(sLayout->photocam, EINA_TRUE);
#endif

	evas_object_smart_callback_add(sLayout->photocam,
					"loaded",
					_ivug_ss_photocam_loaded_cb,
					sLayout);

	evas_object_show(sLayout->photocam);
	evas_object_show(sLayout->layout);

	return true;
}

SlideShow *ivug_ss_create(Evas_Object *parent)
{
	SlideShow *pSlideShow = NULL;
	MSG_IVUG_HIGH("ivug_ss_create");
	pSlideShow = (SlideShow*)calloc(1,sizeof(SlideShow));
	if (pSlideShow == NULL) {
		return NULL;
	}

	pSlideShow->state = SLIDE_SHOW_STOPPED;
	pSlideShow->bMouse_event = false;
	ivug_effect_type ivug_effect = IVUG_EFFECT_TYPE_SLIDE;
	ivug_config_get_slideshow_setting(&(pSlideShow->ss_mode),
			&(pSlideShow->ss_interval_time), &ivug_effect);
	if(ivug_effect == IVUG_EFFECT_TYPE_UNKNOWN) {
		free(pSlideShow);
		pSlideShow = NULL;
		return NULL;
	}

	pSlideShow->effect_type = _ivug_ss_get_trans_effect(ivug_effect);

	pSlideShow->obj = _ivug_ss_create_layout(parent, IVUG_SS_LY_EDJ_PATH, "view.slideshow");
	MSG_ASSERT(pSlideShow->obj != NULL);
	evas_object_name_set(pSlideShow->obj, "slideshow");

	/* Create Layout for the current item */
	if ( _ivug_ss_create_image_layout(pSlideShow->obj, &pSlideShow->sLayout[0]) == false)
	{
		MSG_ERROR("Cannot create current layout");
		free(pSlideShow);
		return NULL;
	}
	pSlideShow->sLayout[0].pSlideshow = pSlideShow;
	evas_object_name_set(pSlideShow->sLayout[0].layout, "Layout 0");

	/* Create Layout for the next item */
	if ( _ivug_ss_create_image_layout(pSlideShow->obj, &pSlideShow->sLayout[1]) == false)
	{
		MSG_ERROR("Cannot create current layout");
		elm_object_part_content_unset(pSlideShow->sLayout[0].layout, "content");
		evas_object_del(pSlideShow->sLayout[0].photocam);
		evas_object_del(pSlideShow->sLayout[0].layout);
		free(pSlideShow);
		return NULL;
	}

	pSlideShow->sLayout[1].pSlideshow = pSlideShow;
	evas_object_name_set(pSlideShow->sLayout[1].layout, "Layout 1");

	evas_object_event_callback_add(pSlideShow->sLayout[0].layout, EVAS_CALLBACK_SHOW, _shown1, NULL);
	evas_object_event_callback_add(pSlideShow->sLayout[0].layout, EVAS_CALLBACK_HIDE, _hidden1, NULL);

	evas_object_event_callback_add(pSlideShow->sLayout[1].layout, EVAS_CALLBACK_SHOW, _shown1, NULL);
	evas_object_event_callback_add(pSlideShow->sLayout[1].layout, EVAS_CALLBACK_HIDE, _hidden1, NULL);

	/* Event rect */
	pSlideShow->event = evas_object_rectangle_add(evas_object_evas_get(parent));
	evas_object_name_set(pSlideShow->event, "ss_event");
	evas_object_color_set(pSlideShow->event, 0,0,0,0);

	evas_object_show(pSlideShow->event);

	//EFL::dump_obj(pSlideShow->obj, 0);

	evas_object_event_callback_add(pSlideShow->obj, EVAS_CALLBACK_MOVE, _moved, pSlideShow);
	evas_object_event_callback_add(pSlideShow->obj, EVAS_CALLBACK_RESIZE, _resized, pSlideShow);
	evas_object_event_callback_add(pSlideShow->obj, EVAS_CALLBACK_SHOW, _shown, pSlideShow);
	evas_object_event_callback_add(pSlideShow->obj, EVAS_CALLBACK_HIDE, _hidden, pSlideShow);

// Event
	evas_object_event_callback_add(pSlideShow->event, EVAS_CALLBACK_MOUSE_DOWN, _ivug_ss_on_mouse_down, pSlideShow);
	evas_object_event_callback_add(pSlideShow->event, EVAS_CALLBACK_MOUSE_MOVE, _ivug_ss_on_mouse_move, pSlideShow);
	evas_object_event_callback_add(pSlideShow->event, EVAS_CALLBACK_MOUSE_UP, _ivug_ss_on_mouse_up, pSlideShow);

	return pSlideShow;
}


bool ivug_ss_start(SlideShow *pSlideShow , Media_Item *current, Media_List *list, Eina_Bool bSlideFirst)
{
	MSG_ASSERT(pSlideShow != NULL);

	ivug_ss_get_screen_size(&pSlideShow->screen_w, &pSlideShow->screen_h);

	if(pSlideShow->ss_interval_time < 0)
	{
		MSG_ERROR("slide show interval time is invalid !!!");
		return false;
	}

	if (pSlideShow->state == SLIDE_SHOW_RUNNING )
	{
		MSG_ERROR("Debug me!!! Slide show is running. remove previous one.");
		ivug_ss_stop(pSlideShow);
	}

	if (pSlideShow->effect_engine != NULL )
	{
		MSG_FATAL("Debug Me!!!");
		return false;
	}

	pSlideShow->sCurrent = 0;
	pSlideShow->bSS_StopFlag = EINA_FALSE;
	pSlideShow->ss_Header = current;
	pSlideShow->media_list = list;

	MSG_ASSERT(pSlideShow->obj != NULL);
	pSlideShow->bMouse_event = true;
	evas_object_move(pSlideShow->obj, 0,0);
	evas_object_resize(pSlideShow->obj, pSlideShow->screen_w, pSlideShow->screen_h);

	_ivug_ss_update_pos(pSlideShow, 0, 0);

	Slide_Layout *sLyCurrent = &pSlideShow->sLayout[pSlideShow->sCurrent];
	Slide_Layout *sLyNext = &pSlideShow->sLayout[(pSlideShow->sCurrent + 1) % 2];

	_ivug_ss_set_content(sLyCurrent, current);

	Media_Item *next = NULL;

	next = ivug_ss_get_next_item(pSlideShow->media_list,
			pSlideShow->ss_Header,
			sLyCurrent->mitem,
			pSlideShow->ss_mode);

	if(next == NULL)
	{
		/* if last image is tapped, then after some time, back to main view directly */
		pSlideShow->ss_timer = NULL;
		pSlideShow->ss_Header = NULL;
		pSlideShow->state = SLIDE_SHOW_RUNNING;

		pSlideShow->ss_timer = ecore_timer_add(pSlideShow->ss_interval_time, _ivug_ss_auto_finish_timer_cb, pSlideShow);
		return false;
	}

	_ivug_ss_set_content(sLyNext, next);

	pSlideShow->state = SLIDE_SHOW_RUNNING;

#if 0
	if(_ivug_ss_on_slide_interval(pSlideShow) == false)
	{
		pSlideShow->ss_timer = NULL;
		pSlideShow->ss_Header = NULL;

		/* exit slide show after whole animation is over */
		ivug_ss_stop(pSlideShow);
		return false;
	}
#endif
//	EFL::dump_obj(pSlideShow->obj, 0);

	if(pSlideShow->ss_timer)
	{
		ecore_timer_del(pSlideShow->ss_timer);
	}
	pSlideShow->ss_timer = ecore_timer_add(pSlideShow->ss_interval_time, _ivug_ss_on_slide_interval, pSlideShow);

	MSG_HIGH("Slide show started!");

	return true;
}


bool ivug_ss_stop(SlideShow *pSlideShow)
{
	MSG_ASSERT(pSlideShow != NULL);

	MSG_HIGH("Slide show stopping");

	if (pSlideShow->state != SLIDE_SHOW_RUNNING)
	{
		MSG_HIGH("pSlideShow->state != SLIDE_SHOW_RUNNING");
		return true;
	}

	pSlideShow->bSS_StopFlag = EINA_TRUE;

	//delete timmer
	if(pSlideShow->ss_timer) {
		ecore_timer_del(pSlideShow->ss_timer);
		pSlideShow->ss_timer = NULL;
	}

	if(pSlideShow->click_timer) {
		ecore_timer_del(pSlideShow->click_timer);
		pSlideShow->click_timer = NULL;
	}
	if(pSlideShow->event) {
		evas_object_del(pSlideShow->event);
		pSlideShow->event = NULL;
	}

	if(pSlideShow->effect_engine == NULL)
	{
		MSG_HIGH("Slide effect_engine = NULL");
		pSlideShow->state = SLIDE_SHOW_STOPPED;
		pSlideShow->ss_Header = NULL;
		pSlideShow->bSS_StopFlag = EINA_FALSE;
		evas_object_smart_callback_call(ivug_ss_object_get(pSlideShow), "slideshow,finished", (void *)SLIDE_SHOW_STOPPED);
	}
	//MSG_HIGH("Stop slide show. but not run state");

	return true;
}

bool ivug_ss_resume(SlideShow *pSlideShow)
{
	MSG_HIGH("Slide show Resume");
	MSG_ASSERT(pSlideShow != NULL);

	if(pSlideShow->ss_timer)
	{
		ecore_timer_thaw(pSlideShow->ss_timer);
	}

	pSlideShow->state = SLIDE_SHOW_RUNNING;
	//MSG_HIGH("ivug_resume_slideshow");

	return true;
}

bool ivug_ss_pause(SlideShow *pSlideShow)
{
	MSG_HIGH("Slide show Pause");

	MSG_ASSERT(pSlideShow != NULL);

	if(pSlideShow->ss_timer)
	{
		ecore_timer_freeze(pSlideShow->ss_timer);
	}

	pSlideShow->state = SLIDE_SHOW_PAUSE;

	return true;
}

Media_Item * ivug_ss_item_get(SlideShow *pSlideShow)
{
	MSG_ASSERT(pSlideShow != NULL);

	return pSlideShow->sLayout[(pSlideShow->sCurrent) % 2].mitem;
}

void ivug_ss_delete(SlideShow *pSlideShow)
{
	MSG_HIGH("Slide show Delete");

	if(!pSlideShow)
	{
		MSG_ERROR("Already deleted!!");
		return;
	}
	evas_object_event_callback_del(pSlideShow->event, EVAS_CALLBACK_MOUSE_DOWN, _ivug_ss_on_mouse_down);
	evas_object_event_callback_del(pSlideShow->event, EVAS_CALLBACK_MOUSE_MOVE, _ivug_ss_on_mouse_move);
	evas_object_event_callback_del(pSlideShow->event, EVAS_CALLBACK_MOUSE_UP, _ivug_ss_on_mouse_up);

	if(pSlideShow->effect_engine)
		ivug_effect_finalize(pSlideShow->effect_engine);

	if(pSlideShow->ss_timer) {
		ecore_timer_del(pSlideShow->ss_timer);
		pSlideShow->ss_timer = NULL;
	}
	if(pSlideShow->click_timer) {
		ecore_timer_del(pSlideShow->click_timer);
		pSlideShow->click_timer = NULL;
	}

	for ( int i = 0 ; i < 2; i++)
	{
		if(pSlideShow->sLayout[i].photocam) {
			#ifdef USE_IVUG_IMAGE
			ivug_image_unset(pSlideShow->sLayout[i].photocam);
			#endif
			evas_object_del(pSlideShow->sLayout[i].photocam);
			pSlideShow->sLayout[i].photocam = NULL;
		}
		if(pSlideShow->sLayout[(pSlideShow->sCurrent) % 2].layout)
			pSlideShow->sLayout[(pSlideShow->sCurrent) % 2].photocam =
				elm_object_part_content_unset(pSlideShow->sLayout[(pSlideShow->sCurrent) % 2].layout,
				"content");

		if(pSlideShow->sLayout[(pSlideShow->sCurrent) % 2].photocam) {
			evas_object_del(pSlideShow->sLayout[(pSlideShow->sCurrent) % 2].photocam);
			pSlideShow->sLayout[(pSlideShow->sCurrent) % 2].photocam = NULL;
		}
		if(pSlideShow->sLayout[(pSlideShow->sCurrent + 1) % 2].layout)
			pSlideShow->sLayout[(pSlideShow->sCurrent + 1) % 2].photocam =
				elm_object_part_content_unset(pSlideShow->sLayout[(pSlideShow->sCurrent + 1) % 2].layout,
				"content");

		if(pSlideShow->sLayout[i].layout) {
			evas_object_del(pSlideShow->sLayout[i].layout);
			pSlideShow->sLayout[i].layout = NULL;
		}

	}

	if ( pSlideShow->event )
	{
		evas_object_del(pSlideShow->event);
		pSlideShow->event = NULL;
	}

	if ( pSlideShow->obj )
	{
		evas_object_del(pSlideShow->obj);
		pSlideShow->obj = NULL;
	}

	if(pSlideShow)
	{
		free(pSlideShow);
		pSlideShow = NULL;
	}
	return;
}

void ivug_ss_resize(SlideShow *pSlideShow)
{
	IVUG_FUNC_ENTER();
	/*This interface only used for 2d slideshow,
	  *So disable it when current is dali-slideshow.
	  *resize slide show layout and move them to right pos after rotation
	  */
	if (!pSlideShow) {
		return;
	}
	if((pSlideShow->obj) && (pSlideShow->event)) {
		ivug_ss_get_screen_size(&pSlideShow->screen_w, &pSlideShow->screen_h);
		evas_object_resize(pSlideShow->obj, pSlideShow->screen_w, pSlideShow->screen_h);
		Evas_Coord ox, oy, ow, oh;
		evas_object_geometry_get(pSlideShow->obj, &ox, &oy, &ow, &oh);

		//MSG_HIGH("Moved (%d,%d,%d,%d)", ox, oy, ow, oh);
		if(pSlideShow->event)
		{
			evas_object_move(pSlideShow->event, ox, oy);
		}

		if(pSlideShow->sLayout[pSlideShow->sCurrent].layout)
		{
			evas_object_move(pSlideShow->sLayout[pSlideShow->sCurrent].layout, ox, oy);
		}
		if(pSlideShow->sLayout[(pSlideShow->sCurrent + 1) % 2].layout)
		{
			evas_object_move(pSlideShow->sLayout[(pSlideShow->sCurrent + 1) % 2].layout, ox + pSlideShow->screen_w + IVUG_IMAGE_BETWEEN_MARGIN, oy);
		}
		//MSG_HIGH("_ivug_ss_resize_obj, Moved (%d,%d,%d,%d)", ox, oy, ow, oh);
	}

	IVUG_FUNC_LEAVE();
}



Evas_Object *ivug_ss_object_get(SlideShow *pSlideShow)
{
	MSG_ASSERT(pSlideShow != NULL);

	return pSlideShow->obj;
}

void
ivug_ss_set_stop(SlideShow *pSlideShow)
{
	MSG_ASSERT(pSlideShow != NULL);

	pSlideShow->state = SLIDE_SHOW_STOPPED;
}

