/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "ivug-debug.h"
#include "ivug-xcapture.h"
#include "ivug-effect.h"

/* Currently only 4 is supported */
#define DEFAULT_MAP_COUNT		(4)
#define DEFAULT_FOCUL_DISTANCE		(10000)

#define DEFAULT_START_ANGLE		(0.0f)
#define DEFAULT_END_ANGLE		(90.0f)

#define MAX_BLIND_NUM	8

typedef struct _Blind_Rotation Blind_Rotation;

struct _Blind_Rotation {
	bool cw;		/* Clockwise rotation */
	float from, to;		/* rotate angle: 0 ~ 360 */
};

typedef struct _Blind_Data Blind_Data;

struct _Blind_Data {
	int bx;
	int by;
	int bw;
	int bh;

	Evas_Object * img[MAX_BLIND_NUM];

	Evas_Object * dst;

	Evas * evas;

	Blind_Rotation rotation;
};

static bool
__ivug_blind_image_create(Evas *e, Blind_Data *engine,
				int *src_data)
{
	ivug_retvm_if(e == NULL, false, "Evas is NULL");
	ivug_retvm_if(engine == NULL, false, "engine is NULL");
	ivug_retvm_if(src_data == NULL, false, "src_data is NULL");

	Blind_Data *eng_data = engine;

	ivug_retvm_if((eng_data->bw <= 0 || eng_data->bh <= 0), false,
		      "bw or bh is 0: [%d, %d]", eng_data->bw, eng_data->bh);

	int index;
	int row = 0;
	int col = 0;

	int *psrc = src_data;
	int *dst_data = NULL;

	for (index = 0; index < MAX_BLIND_NUM; index++) {
		eng_data->img[index] = evas_object_image_filled_add(e);
		evas_object_image_size_set(eng_data->img[index], eng_data->bw,
					   eng_data->bh);
		evas_object_image_fill_set(eng_data->img[index], 0, 0,
					   eng_data->bw, eng_data->bh);
		evas_object_image_data_update_add(eng_data->img[index], 0, 0,
						  eng_data->bw, eng_data->bh);

		dst_data =
			evas_object_image_data_get(eng_data->img[index],
						   EINA_TRUE);

		row = eng_data->bw;
		col = eng_data->bh;
		while (col--) {
			while (row--) {
				*dst_data++ = *psrc++;
			}

			row = eng_data->bw;
		}

		evas_object_resize(eng_data->img[index], eng_data->bw,
				   eng_data->bh);
		evas_object_move(eng_data->img[index], eng_data->bx,
				 eng_data->by + eng_data->bh * index);
		evas_object_show(eng_data->img[index]);
	}

	return true;
}

static Effect_Data __ivug_blind_init(Evas_Object *src, Evas_Object *dst)
{
	IV_ASSERT(src != NULL);

	Blind_Data *eng_data = calloc(1, sizeof(Blind_Data));
	if (eng_data == NULL) {
		MSG_EFFECT_ERROR("Memory calloc failed");
		return NULL;
	}

	eng_data->evas = evas_object_evas_get(src);

	Evas_Coord x, y, w, h;
	evas_object_geometry_get(src, &x, &y, &w, &h);

	eng_data->bx = x;
	eng_data->by = y;

	eng_data->dst = dst;

	evas_object_stack_below(dst, src);
	evas_object_move(dst, 0, 0);

	eng_data->rotation.cw = true;
	eng_data->rotation.from = DEFAULT_START_ANGLE;
	eng_data->rotation.to = DEFAULT_END_ANGLE;

	/* Need hide the img_object here */
	evas_object_hide(src);

	return (Effect_Data)eng_data;
}

static bool __ivug_blind_set_size(void *data, int screen_w, int screen_h, int rotation)
{
	IVUG_FUNC_ENTER();

	ivug_retvm_if((screen_w <= 0
		       || screen_h <= 0), false,
		      "screen_w or screen_h <= 0: [%d, %d]", screen_w,
		      screen_h);

	Blind_Data *eng_data = (Blind_Data *) data;

	/* Get the data from ecore evas */
	int size = 0;
	int *src_data = (int *)ivug_xcapture_screen_data_get(&size, rotation);
	ivug_retvm_if(src_data == NULL, false, "screen data is NULL");

	if (size != screen_h * EFFECT_LBYTES_ARGB8888(screen_w)) {
		MSG_EFFECT_ERROR("Screen data size is invalid: %d", size);

		free(src_data);
		src_data = NULL;

		return false;
	}

	eng_data->bw = screen_w;
	eng_data->bh = screen_h / MAX_BLIND_NUM;

	MSG_EFFECT_HIGH("eng_data->bw = %d, eng_data->bh = %d", eng_data->bw,
			eng_data->bh);

	int index;
	for (index = 0; index < MAX_BLIND_NUM; index++) {
		evas_object_del(eng_data->img[index]);
		eng_data->img[index] = NULL;
	}

	bool ret = __ivug_blind_image_create(eng_data->evas,
					     eng_data, src_data);
	if (ret == false) {
		MSG_EFFECT_ERROR("Create effect image failed");

		free(src_data);
		src_data = NULL;

		return false;
	}

	free(src_data);
	src_data = NULL;

	evas_object_move(eng_data->dst, 0, 0);

	IVUG_FUNC_LEAVE();

	return true;
}

static void _ivug_blind_rotation_op(void *data, double frame)
{
	IVUG_FUNC_ENTER();

	Blind_Data *eng_data = (Blind_Data *) data;
	ivug_retm_if(eng_data == NULL, "eng_data is NULL");

	Blind_Rotation *rotation = &(eng_data->rotation);

	float degree;

	degree = rotation->from + (float)(frame * rotation->to);

	if (!rotation->cw) {
		degree *= -1;
	}

	MSG_EFFECT_MED("frame = %f, degree = %f", frame, degree);

	int offset = eng_data->bh / 2;
	int y;

	Evas_Map *map = NULL;
	int index;
	for (index = 0; index < MAX_BLIND_NUM; index++) {
		map = evas_map_new(DEFAULT_MAP_COUNT);
		if (map == NULL) {
			MSG_EFFECT_ERROR("Create map failed");
			break;
		}

		evas_map_smooth_set(map, EINA_TRUE);
		evas_map_util_points_populate_from_object_full(map,
							       eng_data->img
							       [index], 0);

		y = index * eng_data->bh + offset;

		evas_map_util_3d_rotate(map, degree, 0, 0, eng_data->bw / 2, y, 0);
		evas_map_util_3d_perspective(map, eng_data->bw / 2, y, 0,
					     DEFAULT_FOCUL_DISTANCE);

		evas_object_map_enable_set(eng_data->img[index], EINA_TRUE);
		evas_object_map_set(eng_data->img[index], map);
		evas_map_free(map);
	}

	IVUG_FUNC_LEAVE();
}

static void __ivug_blind_anim(void *data, double percent)
{
/*      IVUG_FUNC_ENTER(); */

	ivug_retm_if(data == NULL, "data is NULL");

	Blind_Data *eng_data = (Blind_Data *) data;

	ivug_retm_if((percent < MIN_PERCENT
		      || percent > MAX_PERCENT),
		     "Invalide value: percent = [%f]", percent);

	double frame = percent / MAX_PERCENT;
	_ivug_blind_rotation_op(eng_data, frame);

/*      IVUG_FUNC_LEAVE(); */
}

static void __ivug_blind_pause(void *data)
{
	IVUG_FUNC_ENTER();

	IVUG_FUNC_LEAVE();
}

static void __ivug_blind_resume(void *data)
{
	IVUG_FUNC_ENTER();

	IVUG_FUNC_LEAVE();
}

static void __ivug_blind_finialize(void *data)
{
	IVUG_FUNC_ENTER();

	ivug_retm_if(data == NULL, "data is NULL");

	Blind_Data *eng_data = (Blind_Data *) data;

	int index;
	for (index = 0; index < MAX_BLIND_NUM; index++) {
		evas_object_del(eng_data->img[index]);
		eng_data->img[index] = NULL;
	}

	free(eng_data);
	eng_data = NULL;

	IVUG_FUNC_LEAVE();
}

Effect_Engine *ivug_blind_add(void)
{
	Effect_Engine *eng_blind = calloc(1, sizeof(Effect_Engine));
	ivug_retvm_if(eng_blind == NULL, NULL, "calloc failed");

	eng_blind->func.init = &__ivug_blind_init;
	eng_blind->func.set_size = &__ivug_blind_set_size;
	eng_blind->func.animate = &__ivug_blind_anim;
	eng_blind->func.pause = &__ivug_blind_pause;
	eng_blind->func.resume = &__ivug_blind_resume;
	eng_blind->func.finalize = &__ivug_blind_finialize;

	return eng_blind;
}
