/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#include "ivug-debug.h"

#include "ivug-xcapture.h"
#include "ivug-effect.h"

#define BLOCK_ROW_NUM	20
#define BLOCK_COL_NUM	12

#define DISSOLVE_NUM	240		// 20 * 12

const int const DISSOLVE_TABLE[DISSOLVE_NUM] = {
	201, 8, 82, 163, 181, 215, 0, 126, 30, 106,
	162, 122, 216, 18, 99, 70, 223, 89, 22, 191,
	103, 235, 154, 230, 50, 231, 14, 135, 60, 10,
	114, 210, 53, 113, 17, 199, 42, 165, 229, 100,
	97, 45, 125, 34, 15, 64, 48, 202, 150, 80,
	130, 2, 61, 86, 159, 239, 207, 95, 164, 168,
	55, 75, 169, 6, 145, 149, 173, 208, 205, 4,
	124, 13, 116, 194, 198, 175, 94, 176, 81, 144,
	184, 180, 187, 66, 72, 206, 24, 225, 220, 59,
	3, 182, 93, 237, 16, 227, 54, 119, 183, 38,
	101, 107, 21, 23, 143, 186, 233, 28, 213, 52,
	136, 31, 179, 228, 7, 129, 185, 123, 147, 131,
	111, 234, 192, 33, 139, 203, 160, 137, 47, 40,
	141, 172, 65, 157, 43, 88, 218, 44, 83, 62,
	217, 178, 193, 109, 189, 151, 115, 39, 91, 19,
	138, 76, 190, 209, 79, 63, 158, 196, 85, 214,
	56, 152, 108, 27, 222, 46, 156, 177, 105, 37,
	71, 96, 26, 87, 221, 68, 110, 117, 78, 112,
	188, 74, 77, 92, 69, 170, 238, 120, 35, 51,
	211, 146, 11, 148, 58, 5, 174, 41, 161, 84,
	12, 121, 171, 25, 1, 226, 166, 195, 134, 153,
	212, 142, 29, 9, 155, 132, 67, 200, 73, 127,
	197, 232, 140, 224, 104, 167, 90, 102, 57, 204,
	133, 128, 49, 118, 32, 219, 20, 236, 36, 98
};

typedef struct _Dissolve_Data Dissolve_Data;

struct _Dissolve_Data {
	int screen_w;
	int screen_h;

	int screen_x;
	int screen_y;

	int block_w;		/*  block width*/
	int block_h;		/*  block height*/

	int n_row;		/*  row number*/
	int n_col;		/*  col number*/

	Evas_Object **diss_img;
	Evas * evas;
	Evas_Object *src;
	Evas_Object *dst;
};

static bool
__ivug_dissolve_image_create(Evas *evas, Dissolve_Data * eng_data,
			     int *src_data)
{
	ivug_retvm_if(evas == NULL, false, "Evas is NULL");
	ivug_retvm_if(eng_data == NULL, false, "eng_data is NULL");
	ivug_retvm_if(src_data == NULL, false, "src_data is NULL");

	ivug_retvm_if((eng_data->block_w <= 0 || eng_data->block_h <= 0), false,
		      "screen_w or screen_h is: [%d, %d]", eng_data->block_w,
		      eng_data->block_h);

	ivug_retvm_if((eng_data->n_row <= 0 || eng_data->n_col <= 0), false,
		      "n_row or n_col is: [%d, %d]", eng_data->n_row,
		      eng_data->n_col);

	int *psrc = NULL;
	Evas_Object *img_obj = NULL;
	int *dst_data = NULL;

	int i, j;
	int start_pos = 0;
	int index;
	for (i = 0; i < eng_data->n_row; i++) {
		for (j = 0; j < eng_data->n_col; j++) {
			img_obj = evas_object_image_filled_add(evas);
			evas_object_image_size_set(img_obj, eng_data->block_w,
						   eng_data->block_h);

			/* Locate at the left-top corner*/
			start_pos =
				i * eng_data->screen_w * eng_data->block_h +
				j * eng_data->block_w;
			psrc = src_data + start_pos;
//			MSG_EFFECT_HIGH("start_pos is %d", start_pos);

			/* Set the image data*/
			dst_data = evas_object_image_data_get(img_obj,
							      EINA_TRUE);
			int row = eng_data->block_w;
			int col = eng_data->block_h;

			while (col--) {
				while (row--) {
					*dst_data++ = *psrc++;
				}

				row = eng_data->block_w;
				psrc += (eng_data->n_col - 1) * row;
			}

			evas_object_image_alpha_set(img_obj, EINA_TRUE);
			evas_object_resize(img_obj, eng_data->block_w,
					   eng_data->block_h);

			/* Change the position*/
			int pos_x, pos_y;
			pos_x = eng_data->screen_x + j * eng_data->block_w;
			pos_y = eng_data->screen_y + i * eng_data->block_h;

//			MSG_EFFECT_HIGH("[pos_x, pos_y] is [%d, %d]\n", pos_x,
//					pos_y);
			evas_object_move(img_obj, pos_x, pos_y);
			evas_object_show(img_obj);

			index = i * eng_data->n_col + j;
			eng_data->diss_img[index] = img_obj;
		}
	}

	return true;
}

static Effect_Data __ivug_dissolve_init(Evas_Object *src, Evas_Object *dst)
{
	IVUG_FUNC_ENTER();

	IV_ASSERT(src != NULL);

	Dissolve_Data *eng_data = calloc(1, sizeof(Dissolve_Data));
	if (eng_data == NULL) {
		MSG_EFFECT_ERROR("calloc for eng_data failed");
		return NULL;
	}

	eng_data->evas = evas_object_evas_get(src);

	eng_data->src = src;
	eng_data->dst = dst;

	evas_object_stack_below(dst, src);
	evas_object_move(dst, 0, 0);

	/* Need hide the img_object here*/
	evas_object_hide(src);

	IVUG_FUNC_LEAVE();

	return (Effect_Data)eng_data;
}

static bool __ivug_dissolve_set_size(void *data, int screen_w, int screen_h, int rotation)
{
	IVUG_FUNC_ENTER();

	IV_ASSERT(data != NULL);

	ivug_retvm_if((screen_w <= 0
		       || screen_h <= 0), false,
		      "screen_w or screen_h <= 0: [%d, %d]", screen_w,
		      screen_h);

	MSG_EFFECT_HIGH("[screen_w, screen_h] = [%d, %d]", screen_w, screen_h);

	Dissolve_Data *eng_data = (Dissolve_Data *) data;

	/* Get the data from ecore evas*/
	int size = 0;
	char *src_data = ivug_xcapture_screen_data_get(&size, rotation);
	ivug_retvm_if(src_data == NULL, false, "screen data is NULL");

	if (size != screen_h * EFFECT_LBYTES_ARGB8888(screen_w)) {
		MSG_EFFECT_ERROR("Screen data size is invalid: %d", size);

		free(src_data);
		src_data = NULL;

		return false;
	}

	Evas_Coord x, y, w, h;
	evas_object_geometry_get(eng_data->src, &x, &y, &w, &h);
	eng_data->screen_x = x;
	eng_data->screen_y = y;
	eng_data->screen_w = w;
	eng_data->screen_h = h;

	if(rotation == 0 || rotation == 180)
	{
		eng_data->n_row = BLOCK_ROW_NUM;
		eng_data->n_col = BLOCK_COL_NUM;
	}
	else
	{
		eng_data->n_row = BLOCK_COL_NUM;
		eng_data->n_col = BLOCK_ROW_NUM;
	}
	eng_data->block_h = eng_data->screen_h / eng_data->n_row;
	eng_data->block_w = eng_data->screen_w / eng_data->n_col;

	MSG_EFFECT_HIGH("[n_row, n_col] is [%d, %d]", eng_data->n_row,
			eng_data->n_col);
	MSG_EFFECT_HIGH("[block_h, block_w] is [%d, %d]", eng_data->block_h,
			eng_data->block_w);

	int block_num = eng_data->n_row * eng_data->n_col;
	MSG_EFFECT_HIGH("block_num is %d", block_num);

	int i;
	if(eng_data->diss_img != NULL)
	{
		for (i = 0; i < block_num; i++) {
			if (eng_data->diss_img[i]) {
				evas_object_del(eng_data->diss_img[i]);
			}
			eng_data->diss_img[i] = NULL;
		}
		free(eng_data->diss_img);
	}

	eng_data->diss_img = (Evas_Object **)calloc(1, block_num *
						    sizeof(Evas_Object *));

	bool ret = __ivug_dissolve_image_create(eng_data->evas, eng_data,
						(int *)src_data);
	if (!ret) {
		MSG_EFFECT_ERROR("Create images failed");

		free(src_data);
		src_data = NULL;

		return false;
	}

	free(src_data);
	src_data = NULL;

	evas_object_move(eng_data->dst, 0, 0);

	IVUG_FUNC_LEAVE();

	return true;
}

static void __ivug_dissolve_image_data_clear(Evas_Object *image)
{
	ivug_retm_if(!image, "image object is NULL");

	int w, h;
	evas_object_image_size_get(image, &w, &h);
	ivug_retm_if(w <= 0, "image width is less than 0");
	ivug_retm_if(h <= 0, "image height is less than  0");

	int *data = evas_object_image_data_get(image, EINA_TRUE);
	ivug_retm_if(!data, "data is NULL");

	int size = w * h;

	memset(data, 0x0, size * sizeof(int));
	evas_object_image_data_update_add(image, 0, 0, w, h);
}

static void __ivug_dissolve_anim(void *data, double percent)
{
/*       IVUG_FUNC_ENTER();*/

	Dissolve_Data *eng_data = (Dissolve_Data *) data;

	ivug_retm_if(eng_data == NULL, "data is NULL");
	ivug_retm_if(eng_data->diss_img == NULL, "dissolve image is NULL");

	int count = (eng_data->n_row * eng_data->n_col) * percent / MAX_PERCENT;
	MSG_EFFECT_MED("percent = %f, count = %d", percent, count);

	int i;
	for (i = 0; i < count; i++) {
		MSG_EFFECT_MED("diss_array[%d] = %d", i, DISSOLVE_TABLE[i]);
		__ivug_dissolve_image_data_clear(eng_data->
						 diss_img[DISSOLVE_TABLE[i]]);
	}

/*       IVUG_FUNC_LEAVE();*/
}

static void __ivug_dissolve_pause(void *data)
{
	IVUG_FUNC_ENTER();

	IVUG_FUNC_LEAVE();
}

static void __ivug_dissolve_resume(void *data)
{
	IVUG_FUNC_ENTER();

	IVUG_FUNC_LEAVE();
}

static void __ivug_dissolve_finialize(void *data)
{
	IVUG_FUNC_ENTER();

	ivug_retm_if(data == NULL, "data is NULL");

	Dissolve_Data *eng_data = (Dissolve_Data *) data;

	if (eng_data->diss_img) {
		int block_num = eng_data->n_row * eng_data->n_col;
		int i;

		for (i = 0; i < block_num; i++) {
			if (eng_data->diss_img[i]) {
				evas_object_del(eng_data->diss_img[i]);
			}

			eng_data->diss_img[i] = NULL;
		}

		free(eng_data->diss_img);
		eng_data->diss_img = NULL;
	}

	free(eng_data);
	eng_data = NULL;

	IVUG_FUNC_LEAVE();
}

Effect_Engine *ivug_dissolve_add(void)
{
	Effect_Engine *eng_dissolve = calloc(1, sizeof(Effect_Engine));
	ivug_retvm_if(eng_dissolve == NULL, NULL, "calloc failed");

	eng_dissolve->func.init = &__ivug_dissolve_init;
	eng_dissolve->func.set_size = &__ivug_dissolve_set_size;
	eng_dissolve->func.animate = &__ivug_dissolve_anim;
	eng_dissolve->func.pause = &__ivug_dissolve_pause;
	eng_dissolve->func.resume = &__ivug_dissolve_resume;
	eng_dissolve->func.finalize = &__ivug_dissolve_finialize;

	return eng_dissolve;
}
