/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "ivug-debug.h"
#include "ivug-effect.h"


typedef struct _Iris_Data Iris_Data;

struct _Iris_Data {
	Evas_Object *src;
	Evas_Object *dst;

	Evas_Object *orig_cliper;

	Evas_Object *cliper;

	Evas * evas;

	int screen_w;
	int screen_h;
};

static Effect_Data __ivug_iris_init(Evas_Object *src, Evas_Object *dst)
{
	IVUG_FUNC_ENTER();

	Iris_Data *eng_data = calloc(1, sizeof(Iris_Data));
	ivug_retvm_if(eng_data == NULL, NULL, "calloc return NULL");

	eng_data->evas = evas_object_evas_get(src);
	//ivug_retvm_if(eng_data->evas == NULL, NULL, "Get evas is NULL");
	if (eng_data->evas == NULL) {
		MSG_IVUG_HIGH("evas is null");
		if (eng_data) {
			free(eng_data);
			eng_data = NULL;
		}
		return NULL;
	}
	eng_data->src = src;
	eng_data->dst = dst;

	Evas_Object *orig_cliper = evas_object_clip_get(src);
	eng_data->orig_cliper = orig_cliper;

	evas_object_stack_below(dst, src);
	evas_object_move(dst, 0, 0);

	IVUG_FUNC_LEAVE();

	return (Effect_Data)eng_data;
}

static bool __ivug_iris_set_size(void *data, int screen_w, int screen_h, int rotation)
{
	IVUG_FUNC_ENTER();

	ivug_retvm_if((screen_w <= 0
			   || screen_h <= 0), false,
			  "screen_w or screen_h <= 0: [%d, %d]", screen_w,
			  screen_h);

	Iris_Data *eng_data = (Iris_Data *) data;

	Evas_Coord x, y, w, h;
	evas_object_geometry_get(eng_data->src, &x, &y, &w, &h);
	eng_data->screen_w = screen_w;
	eng_data->screen_h = screen_h;

	if (eng_data->cliper) {
		evas_object_del(eng_data->cliper);
		eng_data->cliper = NULL;
	}

	Evas_Object *clip = evas_object_rectangle_add(eng_data->evas);
	evas_object_resize(clip, screen_w, screen_h);
	evas_object_move(clip, x, y);
	evas_object_show(clip);
	evas_object_clip_set(eng_data->src, clip);
	eng_data->cliper = clip;

	evas_object_move(eng_data->dst, 0, 0);

	IVUG_FUNC_LEAVE();

	return true;

}

static void __ivug_iris_anim(void *data, double percent)
{
/*      IVUG_FUNC_ENTER(); */

	Iris_Data *eng_data = (Iris_Data *) data;

	ivug_retm_if(eng_data == NULL, "data is NULL");
	ivug_retm_if(eng_data->cliper == NULL, "cliper is NULL");

	int w = eng_data->screen_w;
	int h = eng_data->screen_h;
	int x, y;

	w = w * (MAX_PERCENT - percent) / MAX_PERCENT;
	h = h * (MAX_PERCENT - percent) / MAX_PERCENT;

	x = (eng_data->screen_w - w) / 2;
	y = (eng_data->screen_h - h) / 2;

	MSG_EFFECT_MED("percent=%f, [x, y, w, h] = [%d, %d, %d, %d]", percent, x, y, w, h);

	evas_object_resize(eng_data->cliper, w, h);
	evas_object_move(eng_data->cliper, x, y);

/*      IVUG_FUNC_LEAVE(); */
}

static void __ivug_iris_pause(void *data)
{
	IVUG_FUNC_ENTER();

	IVUG_FUNC_LEAVE();
}

static void __ivug_iris_resume(void *data)
{
	IVUG_FUNC_ENTER();

	IVUG_FUNC_LEAVE();
}

static void __ivug_iris_finialize(void *data)
{
	IVUG_FUNC_ENTER();

	ivug_retm_if(data == NULL, "data is NULL");

	Iris_Data *eng_data = (Iris_Data *) data;

	if (eng_data->cliper) {
		evas_object_del(eng_data->cliper);
		eng_data->cliper = NULL;
	}

	if (eng_data->src && eng_data->orig_cliper) {
		evas_object_clip_set(eng_data->src, eng_data->orig_cliper);
	}

	free(eng_data);
	eng_data = NULL;

	IVUG_FUNC_LEAVE();
}

Effect_Engine *ivug_iris_add(void)
{
	IVUG_FUNC_ENTER();

	Effect_Engine *eng_iris = calloc(1, sizeof(Effect_Engine));
	ivug_retvm_if(eng_iris == NULL, NULL, "calloc failed");

	eng_iris->func.init = &__ivug_iris_init;
	eng_iris->func.set_size = &__ivug_iris_set_size;
	eng_iris->func.animate = &__ivug_iris_anim;
	eng_iris->func.pause = &__ivug_iris_pause;
	eng_iris->func.resume = &__ivug_iris_resume;
	eng_iris->func.finalize = &__ivug_iris_finialize;

	IVUG_FUNC_LEAVE();

	return eng_iris;
}
