/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <Elementary.h>
#include <Ecore_X.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <assert.h>
#include <errno.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/extensions/Xcomposite.h>
#include <X11/extensions/XShm.h>
#include <X11/Xatom.h>

#include "ivug-debug.h"

#define MAX_ERROR_MSG_LEN	256

struct xcapture_t {
	XImage *xim;
	XShmSegmentInfo si;
	int size;
	Pixmap pix;
};

static int __ivug_xcapture_get_window_attribute(Window id,
						int *depth, Visual **visual,
						int *width, int *height)
{
	IV_ASSERT(id);
	XWindowAttributes attr;

	if (!XGetWindowAttributes(ecore_x_display_get(), id, &attr)) {
		MSG_EFFECT_ERROR("XGetWindowAttributes failed\n");
		return -1;
	}

	if (attr.map_state == IsViewable && attr.class == InputOutput) {
		*depth = attr.depth;
		*width = attr.width;
		*height = attr.height;
		*visual = attr.visual;
	}

	return 0;
}

static struct xcapture_t *__ivug_xcapture(unsigned long id)
{
	IV_ASSERT(id);

	struct xcapture_t *handle = NULL;
	int depth = 0;
	int width = 0;
	int height = 0;
	Visual *visual = NULL;
	char error_msg[MAX_ERROR_MSG_LEN] = { 0, };

	handle = malloc(sizeof(struct xcapture_t));
	if (!handle) {
		MSG_EFFECT_ERROR
			("Failed to allocate memory for __ivug_xcapture\n");
		return NULL;
	}

	MSG_EFFECT_MED("Window : 0x%lX\n", id);
	if (__ivug_xcapture_get_window_attribute
	    (id, &depth, &visual, &width, &height) < 0) {
		MSG_EFFECT_ERROR("Failed to get the attributes from 0x%x\n",
				 (unsigned int)id);
		free(handle);
		return NULL;
	}

	MSG_EFFECT_MED("WxH : %dx%d\n", width, height);
	MSG_EFFECT_MED("Depth : %d\n", depth >> 3);

	/* NOTE: just add one more depth.... */
	handle->si.shmid = shmget(IPC_PRIVATE,
				  width * height * ((depth >> 3) + 1),
				  IPC_CREAT | 0666);
	if (handle->si.shmid < 0) {
		MSG_EFFECT_ERROR("Failed to get SHM %s\n",
				 strerror_r(errno, error_msg,
					    sizeof(error_msg)));
		free(handle);
		return NULL;
	}

	handle->si.readOnly = False;
	handle->si.shmaddr = shmat(handle->si.shmid, NULL, 0);
	if (handle->si.shmaddr == (char *)-1) {
		MSG_EFFECT_ERROR("Failed to attach shm %s\n",
				 strerror_r(errno, error_msg,
					    sizeof(error_msg)));

		shmctl(handle->si.shmid, IPC_RMID, 0);
		free(handle);
		return NULL;
	}

	handle->xim = XShmCreateImage(ecore_x_display_get(), visual, depth,
				      ZPixmap, NULL, &handle->si, width,
				      height);
	if (!handle->xim) {
		MSG_EFFECT_ERROR("Failed create an image %s\n",
				 strerror_r(errno, error_msg,
					    sizeof(error_msg)));

		shmdt(handle->si.shmaddr);
		shmctl(handle->si.shmid, IPC_RMID, 0);
		free(handle);
		return NULL;
	}

	handle->size = handle->xim->bytes_per_line * handle->xim->height;
	handle->xim->data = handle->si.shmaddr;

	handle->pix = XCompositeNameWindowPixmap(ecore_x_display_get(), id);

	if (XShmAttach(ecore_x_display_get(), &handle->si) == False) {
		MSG_EFFECT_ERROR("Failed to attach\n");
	}

	if (XShmGetImage(ecore_x_display_get(), id, handle->xim, 0, 0,
			 0xFFFFFFFF) == False) {
		MSG_EFFECT_ERROR("Failed to get image\n");
	}

	XSync(ecore_x_display_get(), False);
	return handle;
}

static void __ivug_xcapture_destroy(struct xcapture_t *handle)
{
	ivug_retm_if(!handle, "handle is NULL");

	XShmDetach(ecore_x_display_get(), &handle->si);
	XFreePixmap(ecore_x_display_get(), handle->pix);
	XDestroyImage(handle->xim);

	shmdt(handle->si.shmaddr);
	shmctl(handle->si.shmid, IPC_RMID, 0);

	free(handle);
}

static inline void *__ivug_xcapture_get_data(struct xcapture_t *handle,
					     int *size)
{
	ivug_retvm_if(!handle, NULL, "handle is NULL");

	if (size) {
		*size = handle->size;
	}

	return handle->xim->data;
}


static void __ivug_xcapture_data_rotate_180cw(unsigned int **dst,
					      unsigned int *src, int width,
					      int height)
{
	ivug_retm_if(!src, "src is NULL");

	unsigned int *p1, *p2, tmp;
	int x, hw;

	/* memcpy */
	*dst = malloc(width * height * sizeof(unsigned int));
	memcpy(*dst, src, width * height * sizeof(unsigned int));

	/* set width, height */
	hw = width * height;
	x = (hw / 2);
	p1 = *dst;
	p2 = *dst + hw - 1;
	for (; --x > 0;) {
		tmp = *p1;
		*p1 = *p2;
		*p2 = tmp;
		p1++;
		p2--;
	}
}

static void __ivug_xcapture_data_rotate_90cw(unsigned int **dst,
					     unsigned int *src, int width,
					     int height)
{
	ivug_retm_if(!src, "src is NULL");

	unsigned int *to = NULL, *from = NULL;
	int x, y, w, hw;

	/* memcpy */
	*dst = malloc(width * height * sizeof(unsigned int));
	memcpy(*dst, src, width * height * sizeof(unsigned int));

	/* set width, height */
	w = height;
	height = width;
	width = w;
	hw = width * height;

	to = *dst + w - 1;
	hw = -hw - 1;
	from = src;

	for (x = width; --x >= 0;) {
		for (y = height; --y >= 0;) {
			*to = *from;
			from++;
			to += w;
		}

		to += hw;
	}
}

static void __ivug_xcapture_data_rotate_270cw(unsigned int **dst,
					      unsigned int *src, int width,
					      int height)
{
	ivug_retm_if(!src, "src is NULL");

	unsigned int *to, *from;
	int x, y, w, hw;

	/* memcpy */
	*dst = malloc(width * height * sizeof(unsigned int));
	memcpy(*dst, src, width * height * sizeof(unsigned int));

	/* set width, height */
	w = height;
	height = width;
	width = w;

	hw = width * height;

	to = *dst + hw - w;
	w = -w;
	hw = hw + 1;
	from = src;

	for (x = width; --x >= 0;) {
		for (y = height; --y >= 0;) {
			*to = *from;
			from++;
			to += w;
		}

		to += hw;
	}
}

static char *__ivug_xcapture_data_get(Ecore_X_Window win, int *size,
				      int rotation)
{
	struct xcapture_t *handle = NULL;

	handle = __ivug_xcapture(win);
	if (!handle) {
		MSG_EFFECT_ERROR("__ivug_xcapture failed");
		return NULL;
	}

	int data_size = 0;
	char *image = __ivug_xcapture_get_data(handle, &data_size);
	if (!image || data_size == 0) {
		MSG_EFFECT_ERROR("__ivug_xcapture_get_data failed");

		__ivug_xcapture_destroy(handle);
		return NULL;
	}

	MSG_EFFECT_MED("handle->xim: width = %d, height = %d\n",
			handle->xim->width, handle->xim->height);

	/* Free the data when unused */
	int width = 0;
	int height = 0;
	char *data = NULL;

	width = handle->xim->bytes_per_line / sizeof(int);
	height = handle->xim->height;

	if (rotation == 180) {
		__ivug_xcapture_data_rotate_180cw((unsigned int **)&data,
						  (unsigned int *)image, width,
						  height);
	} else if (rotation == 90) {
		__ivug_xcapture_data_rotate_90cw((unsigned int **)&data,
						 (unsigned int *)image, width,
						 height);
	} else if (rotation == 270) {
		__ivug_xcapture_data_rotate_270cw((unsigned int **)&data,
						  (unsigned int *)image, width,
						  height);
	} else {		/* all other cases treated as angle 0 */

		data = (char *)calloc(1, data_size);
		memcpy(data, image, data_size);
	}

	__ivug_xcapture_destroy(handle);

	if (size) {
		*size = data_size;
	}

	return data;
}

char *ivug_xcapture_screen_data_get(int *size, int rotation)
{
	Ecore_X_Window root;
	root = ecore_x_window_focus_get();

	/* Free the memory allocated when unused */
	return __ivug_xcapture_data_get(root, size, rotation);
}
