/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "ivug-define.h"

#include <Elementary.h>
#include <Ecore_Evas.h>

#include "ivug-debug.h"
#include "ivug-image.h"
#include "errno.h"

#include "debug.h"


static Evas_Object *_crop_image(const char *file, int /* image coord */crop_x, int crop_y, int crop_w, int crop_h, int final_w, int final_h)
{
	MSG_HIGH("Crop image XYWH(%d,%d,%d,%d) Dimension(%d,%d)", crop_x, crop_y, crop_w, crop_h, final_w, final_h);

// 1. Load source image into buffer canvas.
	Ecore_Evas *m_ee;
	m_ee = ecore_evas_buffer_new(final_w, final_h);
	MSG_ASSERT(m_ee != NULL);

	Evas *evas = ecore_evas_get(m_ee);

	Evas_Object *bg;

	bg = evas_object_rectangle_add(evas);
	evas_object_color_set(bg, 0, 0, 0, 255); /* black bg */
	evas_object_move(bg, 0, 0); /* at origin */
	evas_object_resize(bg, final_w, final_h); /* covers full canvas */
	evas_object_show(bg);

	Evas_Object *img;

	img = evas_object_image_filled_add(evas);

	evas_object_image_file_set(img, file, NULL);

	Evas_Load_Error err = evas_object_image_load_error_get(img);
	if (err != EVAS_LOAD_ERROR_NONE)
	{
		MSG_ERROR("could not load image '%s'. error string is \"%s\"", file, evas_load_error_str(err));
		return NULL;
	}

	Eina_Bool bRegion = evas_object_image_region_support_get(img);

	evas_object_image_file_set(img, NULL, NULL);

	if ( bRegion == EINA_TRUE)
	{
		MSG_HIGH("Region supported");
		evas_object_image_load_orientation_set(img, EINA_TRUE);
		evas_object_image_load_region_set(img, crop_x, crop_y, crop_w, crop_h);

		evas_object_image_file_set(img, file, NULL);

		Evas_Load_Error err = evas_object_image_load_error_get(img);
		if (err != EVAS_LOAD_ERROR_NONE)
		{
			MSG_ERROR("could not load image '%s'. error string is \"%s\"", file, evas_load_error_str(err));
			return NULL;
		}
	}
	else
	{
		MSG_HIGH("Region not supported.");

// Load full image
		Evas_Object *src;

		src = evas_object_image_add(evas);

		evas_object_image_file_set(src, file, NULL);

		Evas_Load_Error err = evas_object_image_load_error_get(src);
		if (err != EVAS_LOAD_ERROR_NONE)
		{
			MSG_ERROR("could not load image '%s'. error string is \"%s\"", file, evas_load_error_str(err));
			return NULL;
		}

		int w,h;
		evas_object_image_size_get(src, &w, &h);

		MSG_HIGH("Image WH(%d,%d)", w, h);

// Setup cropped buffer
		evas_object_image_size_set(img, crop_w, crop_h);
		evas_object_image_fill_set(img, 0, 0, crop_w, crop_h);

// Copy image from src to img.
		int *Src = (int *)evas_object_image_data_get(src, EINA_FALSE);		// ImgBuffer's size is same as original image
		MSG_ASSERT(Src != NULL);

		int *Dest = (int *)evas_object_image_data_get(img, EINA_TRUE);
		MSG_ASSERT(Dest != NULL);

		int *Tmp = Dest;

		int ImgW = w;
		int *pRow = Src + (crop_y * ImgW /* stride */ + crop_x) ;		// Src begin buffer

		int nRow = crop_h;
		int nCol = crop_w;

		MSG_HIGH("Crop(%d,%d,%d,%d) Stride=%d nRow=%d nCol=%d", crop_x, crop_y, crop_w, crop_h, ImgW, nRow, nCol);

		while(nRow--)
		{
			while(nCol--)
			{
				// ELM_MSG_HIGH("nRow=%d, nCol=%d, Src=0x%08x Dst=0x%08x", nRow, nCol,	pRow , Dest);
				*Tmp++ = *pRow++;
			}

			nCol = crop_w;
			pRow = pRow + (ImgW - crop_w /* stride */ );
		}
		evas_object_image_data_set(src, (void *)Src); 	// Unref
		evas_object_del(src);

		evas_object_image_data_set(img, (void *)Dest);
	}

	int cw, ch;

	cw = crop_w;
	ch = crop_h;

	int ph, pw;
	int px, py;

// Auto fit. why???????. Used when set wallpaper(image size is smaller than LCD)
	ph = (ch * final_w) / cw;

	if (ph > final_h)
	{
		pw = (cw * final_h) / ch;
		ph = final_h;
	}
	else
	{
		pw = final_w;
	}

// Calculate XY
	px = (final_w - pw ) / 2;
	py = (final_h - ph ) / 2;

	MSG_HIGH("CenterPut XYWH(%d,%d,%d,%d) in Buf(%d,%d)", px, py, pw, ph, final_w, final_h);

	evas_object_move(img, px, py);
	evas_object_resize(img, pw, ph); // set object size which you want to scale

	evas_object_show(img);

//	evas_object_image_save(img, "/opt/ug/data/ug-image-viewer-efl/tt.jpg", NULL, NULL);

//  Copy Buffer canvas data to image object in current evas
	const void *data = ecore_evas_buffer_pixels_get(m_ee);
	MSG_ASSERT(data != NULL);

	Evas_Object *result;
	result = evas_object_image_filled_add(evas);

	evas_object_image_size_set(result, final_w, final_h);
	evas_object_image_data_set(result, const_cast<void *>(data));
	evas_object_image_data_update_add(result, 0, 0, final_w, final_h);

	MSG_HIGH("Cropped image is created. obj=0x%08x", result);
	return result;
};



// APIs
EAPI Evas_Object *ivug_image_create(Evas_Object *parent)
{
	Evas_Object *obj;

	obj = elm_photocam_add(parent);

	elm_photocam_paused_set(obj, EINA_TRUE);
	elm_photocam_zoom_mode_set(obj, ELM_PHOTOCAM_ZOOM_MODE_AUTO_FIT);

	elm_scroller_policy_set(obj, ELM_SCROLLER_POLICY_OFF, ELM_SCROLLER_POLICY_OFF);

	MSG_HIGH("Create photocam object. obj=0x%08x", obj);
	return obj;
}

EAPI Evas_Load_Error ivug_image_file_set(Evas_Object *obj, const char *file, const char *key)
{
	Evas_Load_Error err = EVAS_LOAD_ERROR_NONE;

	err = elm_photocam_file_set(obj, file);

	MSG_HIGH("photocam file set. %s error=%d", file, err);

	if ( err != EVAS_LOAD_ERROR_NONE)
	{
		MSG_ERROR("Photocam file set error! err=%d", err);
	}

	return err;
}

EAPI Evas_Load_Error ivug_image_mem_set(Evas_Object *obj, const unsigned char *buffer, unsigned int size, const char *key)
{
	MSG_ERROR("%s is not supported", __func__);

	return EVAS_LOAD_ERROR_GENERIC;
}


EAPI Evas_Load_Error ivug_image_unset(Evas_Object *obj)
{
	Evas_Load_Error err = EVAS_LOAD_ERROR_NONE;

	MSG_HIGH("photocam file unset. %s", elm_photocam_file_get(obj));

	err = elm_photocam_file_set(obj, NULL);
	return err;
}

EAPI void ivug_image_zoom_set(Evas_Object *obj, double zoom, Evas_Point *pCenter)
{
	//elm_photocam_paused_set(obj, EINA_FALSE); //stop photocam animation
	elm_photocam_zoom_mode_set(obj, ELM_PHOTOCAM_ZOOM_MODE_MANUAL);
	elm_photocam_zoom_set(obj, zoom);
}

EAPI double ivug_image_zoom_get(const Evas_Object *obj)
{

	return elm_photocam_zoom_get(obj);
}

EAPI void ivug_image_image_size_get(const Evas_Object *obj, int *w, int *h)
{
// Return original image size
	elm_photocam_image_size_get(obj, w, h);

	MSG_HIGH("Image size XYWH(%d,%d)", *w, *h);

}

EAPI void ivug_image_region_get(const Evas_Object *obj, int *x, int *y, int *w, int *h)
{
	Evas_Object *image;

	image = elm_photocam_internal_image_get(obj);

	if ( image == NULL )
	{
		if(x) *x = 0;
		if(y) *y = 0;
		if(w) *w = 0;
		if(h) *h = 0;

		return;
	}

	evas_object_geometry_get(image, x ,y, w, h);
//	elm_photocam_image_region_get(obj, x ,y, w, h);

	MSG_HIGH("Region size XYWH(%d,%d,%d,%d)", *x, *y, *w, *h);
}

EAPI void ivug_image_decoded_size_get(const Evas_Object *obj, int *w, int *h)
{
	Evas_Object *image = NULL;

	image = elm_photocam_internal_image_get(obj);

	if ( image == NULL )
	{
		if(w) *w = 0;
		if(h) *h = 0;

		return;
	}

	int iw, ih;

	elm_photocam_image_size_get(obj, &iw, &ih);

	int scale = evas_object_image_load_scale_down_get(obj);

	if(w) *w = iw / scale;
	if(h) *h = ih / scale;
};


EAPI unsigned char *ivug_image_decoded_buffer_get(const Evas_Object *obj)
{
	Evas_Object *image = NULL;

	image = elm_photocam_internal_image_get(obj);

	if ( image == NULL )
	{
		return NULL;
	}

	return (unsigned char *)evas_object_image_data_get(image, EINA_FALSE /* Get for reading Only*/ );
}

EAPI void ivug_image_hold_set(const Evas_Object *obj, Eina_Bool hold)
{
	MSG_ERROR("%s is not supported", __func__);
}

EAPI void ivug_image_zoom_reset(Evas_Object *obj, Evas_Point *pCenter)
{
	Eina_Bool bPaused = elm_photocam_paused_get(obj);

	elm_photocam_paused_set(obj, EINA_TRUE);
	elm_photocam_zoom_mode_set(obj, ELM_PHOTOCAM_ZOOM_MODE_AUTO_FIT);
	elm_photocam_paused_set(obj, bPaused);

}

EAPI Evas_Object *ivug_image_region_image_get(Evas_Object *obj, int /* LCD coord */ x, int y, int w, int h )
{
	MSG_HIGH("Region (%d,%d,%d,%d)", x, y, w, h);

	Evas_Object *image;

	image = elm_photocam_internal_image_get(obj);

	int ix, iy, iw, ih;

	evas_object_geometry_get(image, &ix, &iy, &iw, &ih);

	MSG_HIGH("Image geometry get : (%d,%d,%d,%d)", ix, iy, iw, ih);

	int ow, oh;
	evas_object_image_size_get(image, &ow, &oh);

	ow = ow * evas_object_image_load_scale_down_get(image);
	oh = oh * evas_object_image_load_scale_down_get(image);

	MSG_HIGH("Original image size(%d,%d)", ow, oh);

	int cx, cy, cw, ch;

// Get intersect rect between xywh and ixywh
	cx = ix;
	if ( x > ix ) cx = x;

	cy = iy;
	if ( y > iy ) cy = y;

	cw = w;
	if ( x + w > ix + iw ) cw = iw;

	ch = h;
	if ( y + h > iy + ih ) ch = ih;

	MSG_HIGH("Cropped XWYH(%d,%d,%d,%d)", cx, cy, cw, ch);

// Get rect on image.
	int gx, gy, gw, gh;

	gx = ( cx - ix ) * (double)ow/iw;
	gy = ( cy - iy ) * (double)ow/iw;

	gw = cw * (double)ow/iw;
	gh = ch * (double)ow/iw;

	MSG_HIGH("On image rect : (%d,%d,%d,%d) Scale=%d", gx, gy, gw, gh, evas_object_image_load_scale_down_get(image));

	const char *file = NULL;
	const char *key = NULL;

	evas_object_image_file_get(image, &file, &key);

	if ( file == NULL )
	{
		MSG_ERROR("File is not loaded");
		return NULL;
	}

	MSG_HIGH("File = %s %s", file, key );

	MSG_HIGH("Image size get : (%d,%d)", ow, oh);

	Evas_Object *result = _crop_image(file, gx, gy, gw, gh, w, h);

	return result;
}


EAPI Evas_Object *
ivug_image_internal_image_get(Evas_Object *obj)
{
	return elm_photocam_internal_image_get(obj);
}


EAPI void ivug_image_animated_set(const Evas_Object *obj, Eina_Bool bAnimation)
{
	MSG_ERROR("%s is not supported", __func__);
}

EAPI void ivug_image_rotate_set(Evas_Object *obj, int degree)
{
	MSG_ERROR("%s is not supported", __func__);
}

EAPI int ivug_image_rotate_get(Evas_Object *obj)
{
	MSG_ERROR("%s is not supported", __func__);

	return 0;
}

EAPI void ivug_image_coordinate_lcd_to_image(Evas_Object *photocam, int lcd_x, int lcd_y, int *img_x, int *img_y)
{
	MSG_ASSERT(photocam != NULL);

	int image_w, image_h;

	ivug_image_image_size_get(photocam, &image_w, &image_h);	// Get original image size.

	MSG_UTIL_HIGH("size w=%d, h=%d", image_w, image_h);

	ivug_ret_if(image_w <= 0 || image_h <= 0);

	int gx, gy, gw, gh;

	ivug_image_region_get(photocam,  &gx, &gy, &gw, &gh);

	MSG_UTIL_HIGH("region x=%d, y=%d, w=%d, h=%d", gx, gy, gw, gh);

	int image_x, image_y;

	image_x = lcd_x - gx;
	image_y = lcd_y - gy;

	image_x = image_x*image_w/gw;
	image_y = image_y*image_h/gh;

	MSG_UTIL_HIGH("changed x=%d, y=%d", image_x, image_y);

	*img_x = image_x;
	*img_y = image_y;
}

EAPI void ivug_image_coordinate_image_to_lcd(Evas_Object *photocam, int img_x, int img_y, int *lcd_x, int *lcd_y)
{
	MSG_ASSERT(photocam != NULL);

	int image_w, image_h;

	ivug_ret_if(photocam == NULL);

	ivug_image_image_size_get(photocam, &image_w, &image_h);	// Get original image size.

	MSG_UTIL_HIGH("size w=%d, h=%d", image_w, image_h);

	ivug_ret_if(image_w <= 0 || image_h <= 0);

	int gx, gy, gw, gh;

	ivug_image_region_get(photocam,  &gx, &gy, &gw, &gh);

	MSG_UTIL_HIGH("region x=%d, y=%d, w=%d, h=%d", gx, gy, gw, gh);

	int scr_x, scr_y;

	scr_x = img_x*gw/image_w;
	scr_y = img_y*gh/image_h;

	scr_x = scr_x+gx;
	scr_y = scr_y+gy;

	MSG_UTIL_HIGH("changed x=%d, y=%d", scr_x, scr_y);

	*lcd_x = scr_x;
	*lcd_y = scr_y;
}

