/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	  http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "ivug-debug.h"

#include <Elementary.h>
#include "ivug-listpopup.h"

#undef LOG_LVL
#define LOG_LVL DBG_MSG_LVL_HIGH

#undef LOG_CAT
#define LOG_CAT "IV-POPUP"

typedef struct {
	Evas_Object *obj;

	Evas_Object *popup;
	Evas_Object *parent;
	Eina_List *list;

	const char *title;
	const char *caption;
} ListPopup;


typedef struct {
	Evas_Object *obj;		// This object

	Elm_Object_Item *item;

	const char *iconpath;
	const char *caption;
	void *data;

	bool bDisabled;

} ListPopupItem;

#define IV_LISTPOPUP(obj) \
		(ListPopup *)(evas_object_data_get((obj), "LISTPOPUP"))

static char *
_on_label_set(void *data, Evas_Object *obj, const char *part)
{
	IV_ASSERT( data != NULL);

	ListPopupItem *item = (ListPopupItem *)data;

	return strdup(item->caption); //dump
}


static void
_on_genlist_selected(void *data, Evas_Object *obj, void *event_info)
{
	MSG_ASSERT(data != NULL);

	ListPopupItem *item = data;
//	ListPopup *pListPopup = IV_LISTPOPUP(item->obj);

	MSG_HIGH("Genlist item selected");

	evas_object_smart_callback_call(item->obj, "popup,selected", item);

}


static void
_on_ctxpopup_selected(void *data, Evas_Object *obj, void *event_info)
{
	MSG_ASSERT(data != NULL);

//
//	Elm_Object_Item *item = (Elm_Object_Item *)event_info;
	ListPopupItem *item = data;
//	ListPopup *pListPopup = IV_LISTPOPUP(item->obj);

	MSG_HIGH("Ctxpopup item selected. obj=0x%08x", item->obj);

	evas_object_smart_callback_call(item->obj, "popup,selected", item);

}


static void _on_item_del(void *data, Evas_Object *obj)
{
	MSG_ASSERT(data != NULL);

	ListPopupItem *item = data;

	MSG_HIGH("Remove item");

	if ( item->caption )
		eina_stringshare_del(item->caption);

	item->caption = NULL;

	if ( item->iconpath )
		eina_stringshare_del(item->iconpath);
	item->iconpath = NULL;

	free(item);
}


static void
_on_popup_close(void *data, Evas_Object *obj, void *event_info)
{
	MSG_ASSERT(data != NULL);

	ListPopup *pListPopup = data;

	MSG_HIGH("Genlist Popup Dismissed");

	evas_object_smart_callback_call(pListPopup->obj, "popup,dismissed", NULL);
}

static void _on_genpopup_deleted(void * data, Evas * e, Evas_Object * obj, void * event_info)
{
	MSG_HIGH("Remove genlist popup");

	ListPopup *pListPopup = data;

	if ( pListPopup->title )
		eina_stringshare_del(pListPopup->title);

	pListPopup->title = NULL;

	if ( pListPopup->popup )
	{
		evas_object_del(pListPopup->popup);
	}


	free(pListPopup);
}


static void _on_ctxpopup_deleted(void * data, Evas * e, Evas_Object * obj, void * event_info)
{
	MSG_HIGH("Remove ctx popup");

	ListPopup *pListPopup = data;

	if ( pListPopup->title )
		eina_stringshare_del(pListPopup->title);

	pListPopup->title = NULL;

	EINA_LIST_FREE(pListPopup->list, data)
	{
		_on_item_del(data, NULL);
	}

	pListPopup->list = NULL;

	if ( pListPopup->popup )
	{
		evas_object_del(pListPopup->popup);
	}


	free(pListPopup);
}

static void _on_dismissed_cb(void *data, Evas_Object *obj, void *event_info)
{
	ListPopup *pListPopup = data;

	MSG_HIGH("Ctx Popup Dismissed");
	evas_object_smart_callback_call(pListPopup->obj, "popup,dismissed", NULL);
}


Evas_Object *ivug_listpopup_add(Evas_Object *parent)
{
	Evas_Object *obj = NULL;

// Creating dummy object because we cannot
	obj = elm_box_add(parent);

	ListPopup *pListPopup = (ListPopup *)calloc(1, sizeof(ListPopup));

	pListPopup->parent = parent;

	pListPopup->list = NULL;
	pListPopup->obj = obj;
	pListPopup->title = NULL;
	pListPopup->caption = NULL;
	evas_object_data_set(obj, "LISTPOPUP", (void *)pListPopup);

	MSG_HIGH("List popup is added. obj=0x%08x", obj);

	return obj;
}


Ivug_ListPopup_Item *ivug_listpopup_item_append(Evas_Object *obj, const char *iconpath, const char *caption, void *data)
{
	ListPopup *pListPopup = IV_LISTPOPUP(obj);

	ListPopupItem *item = calloc(1,sizeof(ListPopupItem));

	item->obj = obj;

	if (iconpath )
	{
		item->iconpath = eina_stringshare_add(iconpath);
	}

	if ( caption )
	{
		item->caption = eina_stringshare_add(caption);
	}

	item->bDisabled = false;
	item->data = data;

	MSG_HIGH("Item append: %s", caption);

	pListPopup->list = eina_list_append(pListPopup->list, item);

	return item;
}


Ivug_ListPopup_Item *ivug_listpopup_item_prepend(Evas_Object *obj, const char *iconpath, const char *caption, void *data)
{
	ListPopup *pListPopup = IV_LISTPOPUP(obj);

	ListPopupItem *item = (ListPopupItem *)calloc(1, sizeof(ListPopupItem));

	item->obj = obj;
	item->iconpath = eina_stringshare_add(item->iconpath);
	item->caption = eina_stringshare_add(item->caption);
	item->bDisabled = false;
	item->data = data;

	MSG_HIGH("Item prepend: %s", caption);

	pListPopup->list = eina_list_prepend(pListPopup->list, item);

	return item;
}


Eina_List *ivug_listpopup_items_get(Evas_Object *obj)
{
	ListPopup *pListPopup = IV_LISTPOPUP(obj);

	return pListPopup->list;
}


void ivug_listpopup_item_disabled_set(Ivug_ListPopup_Item *item)
{
	ListPopupItem *item_ = item;

	item_->bDisabled = true;
}

void ivug_listpopup_item_enabled_set(Ivug_ListPopup_Item *item)
{
	ListPopupItem *item_ = item;

	item_->bDisabled = false;

}

void *ivug_listpopup_item_get_data(Ivug_ListPopup_Item *item)
{
	ListPopupItem *item_ = (Ivug_ListPopup_Item *)item;

	return item_->data;
}

const char *ivug_listpopup_item_get_text(Ivug_ListPopup_Item *item)
{
	ListPopupItem *item_ = (Ivug_ListPopup_Item *)item;

	return item_->caption;
}

Elm_Object_Item *ivug_listpopup_item_get_item(Ivug_ListPopup_Item *item)
{
	ListPopupItem *item_ = (Ivug_ListPopup_Item *)item;

	return item_->item;
}

void ivug_listpopup_title_set(Evas_Object *obj, const char* title)
{
	ListPopup *pListPopup = IV_LISTPOPUP(obj);

	MSG_ASSERT(title != NULL);

	pListPopup->title = eina_stringshare_add(title);

	MSG_MED("Popup title set : %s", title);
}

void ivug_listpopup_button_set(Evas_Object *obj, const char* caption)
{
	ListPopup *pListPopup = IV_LISTPOPUP(obj);

	MSG_ASSERT(caption != NULL);

	pListPopup->caption = eina_stringshare_add(caption);

	MSG_MED("Popup caption set : %s", caption);
}

bool ivug_listpopup_popup_show(Evas_Object *obj)
{
	ListPopup *pListPopup = IV_LISTPOPUP(obj);

	if ( pListPopup->popup )
	{
		MSG_WARN("Previous popup is remained");
	}

	Evas_Object *popup;
// create popup
	popup = elm_popup_add(pListPopup->parent);

	if (!popup)
	{
		MSG_ERROR("Error : popup create failed.");
		return false;
	}

	pListPopup->popup = popup;

	elm_object_style_set(popup, "min_menustyle");
   	elm_object_part_text_set(popup, "title,text", pListPopup->title);

	evas_object_size_hint_weight_set(popup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_weight_set(popup, EVAS_HINT_FILL, EVAS_HINT_FILL);

	Evas_Object *btn_close = NULL;

	if ( pListPopup->caption )
	{
		btn_close = elm_button_add(popup);
		elm_object_text_set(btn_close, pListPopup->caption);
		elm_object_part_content_set(popup, "button1", btn_close);
	}

// create genlist
	Evas_Object *genlist;
	static Elm_Genlist_Item_Class itc = {0,};

	genlist = elm_genlist_add(popup);

	itc.version = ELM_GENLIST_ITEM_CLASS_VERSION;
	itc.item_style = "1text";
	itc.func.text_get = _on_label_set;
   	itc.func.content_get = NULL;
   	itc.func.state_get = NULL;
   	itc.func.del = _on_item_del;

	evas_object_size_hint_weight_set(genlist, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(genlist, EVAS_HINT_FILL, EVAS_HINT_FILL);

	int count = eina_list_count(pListPopup->list);

	ListPopupItem *pItem = NULL;
	Eina_List *tmp;
	Elm_Object_Item *gItem;

	EINA_LIST_FOREACH(pListPopup->list, tmp, pItem)
	{
		MSG_MED("Add popup item. %s", pItem->caption);
		gItem = elm_genlist_item_append(genlist, &itc, pItem, NULL /* parent */, ELM_GENLIST_ITEM_NONE, _on_genlist_selected, pItem);
		pItem->item = gItem;

		elm_object_item_disabled_set(gItem, pItem->bDisabled);
	}


// Put together

#define GET_POPUP_HEIGHT(icnt)	(((113*icnt-1) > 408) ? (408) : (113*icnt-1))
#define GET_POPUP_WIDTH(icnt)	(614)

	Evas_Object *box;
	box = elm_box_add(popup);
	evas_object_size_hint_min_set(box, GET_POPUP_WIDTH(count) * elm_config_scale_get(), GET_POPUP_HEIGHT(count) * elm_config_scale_get());
	evas_object_show(genlist);
	elm_box_pack_end(box, genlist);
	elm_object_content_set(popup, box);

	if ( btn_close )
	{
		evas_object_smart_callback_add(btn_close, "clicked", _on_popup_close, pListPopup);
	}

	evas_object_event_callback_add(pListPopup->obj, EVAS_CALLBACK_DEL, _on_genpopup_deleted, pListPopup);

	MSG_HIGH("Show Genlist Popup");

	evas_object_show(popup);

	return true;
}


bool ivug_listpopup_context_show(Evas_Object *obj, Evas_Object *hover, int x, int y)
{
	ListPopup *pListPopup = IV_LISTPOPUP(obj);

	if ( pListPopup->popup )
	{
		MSG_WARN("Previous popup is remained");
	}

	Evas_Object* ctxpopup = NULL;
	ctxpopup = elm_ctxpopup_add(obj);

	if (!ctxpopup)
	{
		MSG_ERROR("Error : popup create failed.");
		return false;
	}

	pListPopup->popup = ctxpopup;

	elm_ctxpopup_hover_parent_set(ctxpopup, hover);

	elm_ctxpopup_direction_priority_set(ctxpopup, ELM_CTXPOPUP_DIRECTION_DOWN,
												ELM_CTXPOPUP_DIRECTION_UP,
												ELM_CTXPOPUP_DIRECTION_RIGHT,
												ELM_CTXPOPUP_DIRECTION_LEFT);


	ListPopupItem *pItem = NULL;
	Eina_List *tmp;
	Elm_Object_Item *gItem;

	Evas_Object *icon;

	EINA_LIST_FOREACH(pListPopup->list, tmp, pItem)
	{
		MSG_MED("Add popup item. %s", pItem->caption);

		if ( pItem->iconpath )
		{
			icon = elm_icon_add(ctxpopup);
			elm_image_file_set(icon, pItem->iconpath, NULL);
		}
		else
		{
			icon = NULL;
		}

		gItem = elm_ctxpopup_item_append(ctxpopup, pItem->caption, icon, _on_ctxpopup_selected, pItem);
		pItem->item = gItem;

		elm_object_item_disabled_set(gItem, pItem->bDisabled);
	}

	evas_object_event_callback_add(pListPopup->obj, EVAS_CALLBACK_DEL, _on_ctxpopup_deleted, pListPopup);

	evas_object_smart_callback_add(ctxpopup, "dismissed", _on_dismissed_cb, pListPopup);

	evas_object_move(ctxpopup, x, y);

	MSG_HIGH("Show Context Popup(%d,%d)", x, y);

	evas_object_show(ctxpopup);

	return true;
}


